//
//  Bar.m
//  3D adjustable bar
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Bar.h"
#import "UIView+CommonDrawingFuncs.h"

#define HORIZ 0
#define VERT  1

@implementation Bar

- (void)setDefaults
{
    self.width = 118.0;
    self.height = 14.0;
    self.startX = 50;
    self.startY = 50;
    self.lowLimit = 0.15;
    self.highLimit = 0.25;
    self.scaleValue = 2.0;
    self.fontSize = 11;
    self.percentToFill = 0.0;
    self.orientation = HORIZ;
    self.tag = 90109;
    self.shadowSize = 0;
    self.useGradientFill = YES;
    self.emphasizeRightLabel = NO;
    self.emphasizeLeftLabel = NO;
    self.leftLabelText = @"";
    self.rightLabelText = @"";
    [self setBackgroundColor:[UIColor clearColor]];
}

- (id)init
{
    if ((self = [super init]))
    {
        // Initialization code
        [self setDefaults];
    }
    return self;
}

- (id)initWithFrame:(CGRect)frame {
    if ((self = [super initWithFrame:frame]))
    {
        [self setDefaults];
    }
    return self;
}

- (void)updateBar:(float)fillValue leftLabelText:(NSString *)leftText leftLabelEmphasis:(BOOL)leftEmphasis
   rightLabelText:(NSString *)rightText rightLabelEmphasis:(BOOL)rightEmphasis
{
	if (fillValue != NAN)
	{
		fillValue *= self.scaleValue;
		if (fillValue > 1.0) self.percentToFill = 1.0;
		else if (fillValue < 0.0) self.percentToFill = 0.0;
        else if (isnan(fillValue) != 0) self.percentToFill = 0.0;   // for some uknown reason, fillValue float can be a NaN value
		else self.percentToFill = fillValue;
	}
	if (leftText != NULL) self.leftLabelText = leftText;
    else self.leftLabelText = @"  ";
    if (leftEmphasis) self.emphasizeLeftLabel = YES;
    else self.emphasizeLeftLabel = NO;
    
	if (rightText != NULL) self.rightLabelText = rightText;
    else self.rightLabelText = @"  ";
    if (rightEmphasis) self.emphasizeRightLabel = YES;
    else self.emphasizeRightLabel = NO;
    
	[self setNeedsDisplay];
}

- (void)updateBar:(float)fillValue leftLabelText:(NSString *)leftText rightLabelText:(NSString *)rightText
{
    [self updateBar:fillValue leftLabelText:leftText leftLabelEmphasis:NO rightLabelText:rightText rightLabelEmphasis:NO];
}

CGRect rectFor1PxStroke(CGRect rect)
{
	return CGRectMake(rect.origin.x + 0.5, rect.origin.y + 0.5, rect.size.width - 1, rect.size.height - 1);
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
	// setup the colors to be used
	CGColorRef whiteColor = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.6].CGColor;
    
	CGColorRef topColor = [UIColor colorWithRed:252.0/255.0 green:252.0/255.0 blue:123.0/255.0 alpha:1.0].CGColor;
	CGColorRef topFadeColor = [UIColor colorWithRed:136.0/255.0 green:136.0/255.0 blue:22.0/255.0 alpha:1.0].CGColor;
	CGColorRef bottomFadeColor = [UIColor colorWithRed:136.0/255.0 green:136.0/255.0 blue:22.0/255.0 alpha:0.0].CGColor;
	
	// set the correct fill color based on the percentToFill amount
	if (self.percentToFill < (self.lowLimit * self.scaleValue))
	{
		topColor = [UIColor colorWithRed:216.0/255.0 green:39.0/255.0 blue:39.0/255.0 alpha:1.0].CGColor;
		topFadeColor = [UIColor colorWithRed:138.0/255.0 green:22.0/255.0 blue:22.0/255.0 alpha:1.0].CGColor;
		bottomFadeColor = [UIColor colorWithRed:138.0/255.0 green:22.0/255.0 blue:22.0/255.0 alpha:0.0].CGColor;
	}
	else if (self.percentToFill > (self.highLimit * self.scaleValue))
	{
		topColor = [UIColor colorWithRed:105.0/255.0 green:216.0/255.0 blue:123.0/255.0 alpha:1.0].CGColor;
		topFadeColor = [UIColor colorWithRed:22.0/255.0 green:137.0/255.0 blue:41.0/255.0 alpha:1.0].CGColor;
		bottomFadeColor = [UIColor colorWithRed:22.0/255.0 green:137.0/255.0 blue:41.0/255.0 alpha:0.0].CGColor;
	}
    
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextSaveGState(context);
	
	// create the rectangle representing the full bar (or 100%) as well as the outline of the bar
	CGRect mainBar = CGRectMake((CGFloat)self.startX, (CGFloat)self.startY, self.width, self.height);
	CGRect mainBarOutline = rectFor1PxStroke(mainBar);
	
	// clear the contents of the bar
	CGContextAddRect(context, mainBar);
	CGContextSetFillColorWithColor(context, [UIColor blackColor].CGColor);
	CGContextFillRect(context, mainBar);
	
	// draw the outline of the full bar
	CGContextAddRect(context, mainBarOutline);
	CGContextSetStrokeColorWithColor(context, whiteColor);
	CGContextStrokeRect(context, mainBarOutline);
    
	// fill the bar up to the percentToFill amount
	CGRect fillBar = CGRectMake((CGFloat)self.startX, (CGFloat)self.startY, (CGFloat)(self.width * self.percentToFill), self.height);
	CGContextAddRect(context, fillBar);
	if (self.useGradientFill)
	{
		drawLinearGradient(context, fillBar, topColor, topFadeColor);
		drawGlossAndGradient(context, fillBar, topColor, topFadeColor);
	}
	else
	{
		CGContextSetFillColorWithColor(context, topColor);
		CGContextFillRect(context, fillBar);
	}
	
	// draw the corresponding reflection value (if non-zero)
	CGRect reflection = CGRectMake((CGFloat)(self.startX), (CGFloat)(self.startY+self.height+1), (CGFloat)(self.width * self.percentToFill), self.shadowSize);
	CGContextAddRect(context, reflection);
	if (self.useGradientFill) drawLinearGradient(context, reflection, topFadeColor, bottomFadeColor);
	else
	{
		CGContextSetFillColorWithColor(context, topColor);
		CGContextFillRect(context, reflection);
	}
    
	// position text to the right of the bar
	CGContextSetTextDrawingMode (context, kCGTextFill);
    CGContextTranslateCTM(context, 0, rect.size.height);
    CGContextScaleCTM(context, 1, -1);
    NSString *fontToUse;
    if (self.emphasizeRightLabel)
    {
        fontToUse = @"Helvetica-Bold";
        CGContextSetRGBFillColor (context, 1, 1, 1, 1);
    }
    else
    {
        fontToUse = @"Helvetica";
        CGContextSetRGBFillColor (context, 0.75, 0.75, 0.75, 1);
    }
    [self.rightLabelText drawAtPoint:CGPointMake((self.startX + self.width + 5),
                                                 (rect.size.height - self.startY - (self.height/2) - (self.fontSize * 0.3)))
                      withAttributes:@{NSFontAttributeName:[UIFont fontWithName:fontToUse
                                                                           size:self.fontSize]
                                       }];

	// position text to the left of the bar
    if (self.emphasizeLeftLabel)
    {
        fontToUse = @"Helvetica-Bold";
        CGContextSetRGBFillColor (context, 1, 1, 1, 1);
    }
    else
    {
        fontToUse = @"Helvetica";
        CGContextSetRGBFillColor (context, 0.75, 0.75, 0.75, 1);
    }
    [self.leftLabelText drawAtPoint:CGPointMake((self.startX - ([self.leftLabelText length] * (self.fontSize * 0.5)) - 7),
                                                (rect.size.height - self.startY - (self.height/2) - (self.fontSize * 0.3)))
                      withAttributes:@{NSFontAttributeName:[UIFont fontWithName:fontToUse
                                                                           size:self.fontSize]
                                       }];

	CGContextRestoreGState(context);
	
}


@end
