//
//  Blackjack+ADSBDataMgr.h
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Blackjack.h"

@interface Blackjack (ADSBDataMgr)

- (void)parseADSBStreamData:(uint8_t *)pLine length:(NSUInteger)len;
- (void)crcTableInit;

- (NSString *)bytes_to_binary:(unsigned long)val numberOfBytes:(unsigned char)numberOfBytes;

- (CLLocationCoordinate2D)convertBlockNumberToLatLon:(uint32_t)blockNumber;
- (uint32_t)convertLatLongToBlockNumber:(double)lat longitude:(double)lon;

- (NSArray *)getBlockListFromLowestBlock:(NSNumber *)lowestBlock toHighestBlock:(NSNumber *)highestBlock;
- (NSArray *)getBlockListForUpperLeftLat:(double)upperLeftLat
                            UpperLeftLon:(double)upperLeftLon
                           lowerRightLat:(double)lowerRightLat
                           lowerRightLon:(double)lowerRightLon;

- (void)clearNEXRADData;
- (void)clearCONUSData;

- (void)expandCONUSDataBlocks:(NSData *)conusBlockData isEmptyBlock:(BOOL)blockIsEmpty blockNumber:(uint32_t)blockNumber scaleFactor:(uint8_t)conusResolution;
- (BOOL)calculateNEXRADDataCoverageAreaUsingCONUS:(BOOL)useCONUS;

- (UIImage *)createWeatherImageWithRegion:(MKCoordinateRegion)coordinateRegion fromCONUSData:(BOOL)useCONUS;
- (UIImage *)createRegionalWeatherImage;
- (UIImage *)createNationalWeatherImage;
- (UIImage *)createNEXRADImageWithUpperLeftLat:(double)upperLeftLat
                                  upperLeftLon:(double)upperLeftLon
                                 lowerRightLat:(double)lowerRightLat
                                 lowerRightLon:(double)lowerRightLon
                                         width:(uint32_t)width
                                        height:(uint32_t)height
                                      useCONUS:(BOOL)useCONUS;
- (UIImage *)createNEXRADImageWithUpperLeftLat:(double)upperLeftLat
                                  upperLeftLon:(double)upperLeftLon
                                      deltaLat:(double)deltaLat
                                      deltaLon:(double)deltaLon
                                         width:(uint32_t)width
                                        height:(uint32_t)height
                                      useCONUS:(BOOL)useCONUS;

- (void)runTestData;

@end
