//
//  Blackjack+ControlMgr.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Blackjack.h"
#import "Blackjack+ControlMgr.h"
#import "BlackjackControlDefs.h"
#import "AppDelegate.h"

#define kProcessTimerDelay		0.6     // See note in the processConnectNotifications method for explanation of this.
#define kQueueTimerInterval     0.25    // Time interval between device commands

#define FLAG_BYTE		0x7E
#define CONTROL_ESCAPE	0x7D

#define DEBUG_DEVICE_RESPONSES  NO
#define DEBUG_DEVICE_SETTINGS   NO

@implementation Blackjack (ControlMgr)

// Device response variables
volatile int                rsp170_cmd;
volatile unsigned char		*rsp170_buf; // needs to be an array of length = 256
volatile unsigned int		rsp170_len;

// CRC table for device set commands
static const unsigned short crc16tbl[256] = {
    0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
    0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
    0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6,
    0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de,
    0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485,
    0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d,
    0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4,
    0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc,
    0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823,
    0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b,
    0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12,
    0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a,
    0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41,
    0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49,
    0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70,
    0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78,
    0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f,
    0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067,
    0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e,
    0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256,
    0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d,
    0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
    0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c,
    0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634,
    0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab,
    0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3,
    0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a,
    0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92,
    0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9,
    0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1,
    0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
    0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
};


#pragma mark - Parse device responses
- (void)parseDeviceResponse:(uint8_t *)msg length:(NSUInteger)len
{
    NSString *errorMsg, *resultMsg;
    
    switch (msg[0]) {
        case cmd170_ack:
        {
            switch (msg[1]) {
                case item_MessageFilter:
                    errorMsg = @"Blackjack acknowledged command to change the output messages.";
                    if (msg[2] == 1) resultMsg = @"Change was successful.";
                    else resultMsg = @"Change was unsuccessful.";
                    self.messageFilterValueRead = NO;
                    break;
                    
                case item_LedBrightness:
                    errorMsg = @"Blackjack acknowledged command to change LED brightness.";
                    if (msg[2] == 1) resultMsg = @"Change was successful.";
                    else resultMsg = @"Change was unsuccessful.";
                    self.ledBrightnessSettingsRead = NO;
                    break;
                    
                default:
                    errorMsg = @"";
                    resultMsg = @"";
                    break;
            }
            if (DEBUG_DEVICE_RESPONSES) DLog(@"%@\n%@", errorMsg, resultMsg);
            break;
        }
            
        case cmd170_nack:
        {
            switch (msg[1]) {
                case item_MessageFilter:
                    errorMsg = @"Blackjack did not acknowledge command to change the output messages.";
                    if (msg[2] == 1) resultMsg = @"But change was successful.";
                    else resultMsg = @"Change was unsuccessful.";
                    break;
                    
                case item_LedBrightness:
                    errorMsg = @"Blackjack did not acknowledge command to change LED brightness.";
                    if (msg[2] == 1) resultMsg = @"But change was successful.";
                    else resultMsg = @"Change was unsuccessful.";
                    break;
                    
                default:
                    errorMsg = @"";
                    resultMsg = @"";
                    break;
            }
            
            if (DEBUG_DEVICE_RESPONSES) DLog(@"%@\n%@", errorMsg, resultMsg);
            break;
        }
            
        case cmd170_response:
        {
            switch (msg[2]) {
                case item_MessageFilter:
                    errorMsg = @"Blackjack responded to message filter query.";
                    self.detailedGPSInfoEnabled = msg[3] & 0x01;
                    self.basicAndLongTrafficMessagesEnabled = msg[3] & 0x02;
                    self.trafficMessagesEnabled = msg[3] & 0x04;
                    resultMsg = [NSString stringWithFormat:@"Response value: %d. Detailed sat enabled: %@. Basic/Long Traffic messages enabled: %@. Traffic messages enabled: %@.",
                                 msg[3], (self.detailedGPSInfoEnabled)?@"Yes":@"No", (self.basicAndLongTrafficMessagesEnabled)?@"Yes":@"No", (self.trafficMessagesEnabled)?@"Yes":@"No"];
                    self.messageFilterValueRead = YES;
                    break;
                    
               case item_LedBrightness:
                    errorMsg = @"Blackjack responded to LED brightness query.";
                    self.ledBrightnessLevel = msg[3];
                    self.dimmingPointForADSBLED = msg[4];
                    resultMsg = [NSString stringWithFormat:@"Response values: %d (msgs[3] and %d (msg[4]).\nLED Brightness = %d%%. ADS-B dims to %d%%.", msg[3], msg[4], self.ledBrightnessLevel, self.dimmingPointForADSBLED];
                    self.ledBrightnessSettingsRead = YES;
                    break;
                    
                default:
                    errorMsg = @"";
                    resultMsg = @"";
                    break;
            }
            
            if (DEBUG_DEVICE_RESPONSES) DLog(@"%@\n%@", errorMsg, resultMsg);
            break;
        }
            
        default:
        {
            break;
        }
    }
    
    // trigger a notification that new ownship message was recevied
    NSNotification *statusReceived = [NSNotification notificationWithName:@"DeviceStatusMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:statusReceived];
}

#pragma mark - Methods to manage command queuing to device
- (void)addCommandRequestToQueue:(NSString *)methodToRun
{
    [self.queue addObject:methodToRun];
    [self startQueueTimer];
}

- (void)doCommand
{
    if ([self.queue count] == 0)
    {
        [self stopQueueTimer];
        return;
    }
    
    NSString *selectorName = [self.queue objectAtIndex:0];
    
    //[self performSelector:NSSelectorFromString(selectorName) withObject:nil];     // this generate a compiler warning
    // See http://stackoverflow.com/questions/7017281/performselector-may-cause-a-leak-because-its-selector-is-unknown
    // Eliminate compiler warning by using the following:
    SEL selector = NSSelectorFromString(selectorName);
    IMP imp = [self methodForSelector:selector];
    void (*func)(id, SEL) = (void *)imp;
    func(self, selector);
    
    [self.queue removeObjectAtIndex:0];
}

- (void)startQueueTimer
{
    if (self.queueTimerStarted) return;
    
    self.queueTimer = [NSTimer timerWithTimeInterval:(kQueueTimerInterval)
                                              target:self
                                            selector:@selector(doCommand)
                                            userInfo:nil
                                             repeats:YES];
    
    [[NSRunLoop currentRunLoop] addTimer:self.queueTimer forMode:NSDefaultRunLoopMode];
    self.queueTimerStarted = YES;
}

- (void)stopQueueTimer
{
    if (self.queueTimer != nil)
    {
        if ([self.queueTimer isValid])
        {
            [self.queueTimer invalidate];
            self.queueTimer = nil;
        }
    }
    self.queueTimerStarted = NO;
}


#pragma mark - Device querying & setting
static UInt16 putbyte( unsigned char* p, unsigned char x, unsigned short* pCrc )
{
	unsigned short crc = *pCrc;
    
	*pCrc = crc16tbl[crc>>8] ^ (crc<<8) ^ x;
    
	if( x == FLAG_BYTE ) {
		*p++ = CONTROL_ESCAPE;
		*p = (FLAG_BYTE ^ 0x20);
		return 2;
	}
	else if( x == CONTROL_ESCAPE ) {
		*p++ = CONTROL_ESCAPE;
		*p = (CONTROL_ESCAPE ^ 0x20);
		return 2;
	}
	else {
		*p = x;
		return 1;
	}
}


UInt16 gdl90_build_packet( unsigned char* Buff, unsigned char messageId, unsigned char command, unsigned char* pIn, unsigned char Length )
{
	unsigned char*	p = Buff;
	UInt16	length;
	UInt16	i;
	UInt16	crc = 0;
    
	// Byte #0 - Flag Byte
	*p++ = FLAG_BYTE;
	// Byte #1 - Message ID
	p += putbyte( p, messageId, &crc );
	// byte #2 - Command byte: query/response, set/ack, nack
    p += putbyte(p, command, &crc);
	// byte #3 - length including item+data
    p += putbyte(p, Length, &crc);
    
	switch( messageId ) {
            
        case GDL90MSG_TO_XGPS170:
            for( i=0; i<Length; i++ ) {
                p += putbyte( p, pIn[i], &crc );
            }
            break;
            
        default:
            // unknown message request
            return 0;
	}
    
	p += putbyte( p, (unsigned char) (crc & 0xFF), &i );
	p += putbyte( p, (unsigned char) ((crc>>8) & 0xFF), &i );
	*p++ = FLAG_BYTE;
    
	length = (UInt16) (p - Buff);
    
	return (length);
}

-(int)sendDataToBlackjack:(unsigned char *)cmd length:(UInt16)len
{
    int written = 0;
    
    if( self.session && [self.session outputStream] )
    {
        if( [[self.session outputStream] hasSpaceAvailable] ) {
            written = [[self.session outputStream] write:cmd maxLength:len];
        }
    }
    return written;
}

# pragma mark • Message Filter value
-(void)queryMessageFilterValue
{
	UInt16	size;
	unsigned char	buf[64];
	unsigned char	item[32];
    
    if (DEBUG_DEVICE_SETTINGS) DLog(@"Querying DOP.");
	
	item[0] = item_MessageFilter;
	size = gdl90_build_packet( buf, GDL90MSG_TO_XGPS170, cmd170_query, item, 1 );
	if( size > 0 ) {
		// send 'size' amount of data in 'buf'
        int bytesOut = [self sendDataToBlackjack:buf length:size];
        if (DEBUG_DEVICE_SETTINGS)
        {
            DLog(@"%d bytes written out to device:", bytesOut);
            for (int j=0; j<size; j++) printf("0x%.2X ", buf[j]);
            printf("\n");
        }
	}
}

-(void)setMessageFilterValue:(uint8_t)setting
{
	UInt16	size;
	unsigned char	buf[64];
	unsigned char	item[32];
    
    if (DEBUG_DEVICE_SETTINGS) DLog(@"Setting message filter value.");
    
	item[0] = item_MessageFilter;
    item[1] = setting;
	size = gdl90_build_packet( buf, GDL90MSG_TO_XGPS170, cmd170_set, item, 2 );
	if( size > 0 ) {
		// send 'size' amount of data in 'buf'
        int bytesOut = [self sendDataToBlackjack:buf length:size];
        
        if (DEBUG_DEVICE_SETTINGS)
        {
            printf("-->These %d bytes written out to device: ", bytesOut);
            for (int j=0; j<size; j++) printf("0x%.2X ", buf[j]);
            printf("\n");
        }
	}
}

# pragma mark • Detailed satellite data
-(void)enableDetailedSatData
{
    uint8_t value = 0;
    
    if (self.messageFilterValueRead == NO)
    {
        DLog(@"Message filter values not read yet. Unable to enable detailed satellite data.");
        return;
    }

    value = self.trafficMessagesEnabled;
    value <<= 1;
    value |= self.basicAndLongTrafficMessagesEnabled;
    value <<= 1;
    value |= 1;

    [self setMessageFilterValue:value];
}

-(void)disableDetailedSatData
{
    uint8_t value = 0;
    
    if (self.messageFilterValueRead == NO)
    {
        DLog(@"Message filter values not read yet. Unable to disable detailed satellite data.");
        return;
    }
    
    value = self.trafficMessagesEnabled;
    value <<= 1;
    value |= self.basicAndLongTrafficMessagesEnabled;
    value <<= 1;
    
    [self setMessageFilterValue:value];
}

# pragma mark • Basic & Long Traffic Messages
-(void)enableBasicAndLongTrafficMessages
{
    uint8_t value = 0;
    
    if (self.messageFilterValueRead == NO)
    {
        DLog(@"Message filter values not read yet. Unable to enable detailed satellite data.");
        return;
    }
    
    value = self.trafficMessagesEnabled;
    value <<= 1;
    value |= 1;
    value <<= 1;
    value |= self.detailedGPSInfoEnabled;
    
    [self setMessageFilterValue:value];
}

-(void)disableBasicAndLongTrafficMessages
{
    uint8_t value = 0;
    
    if (self.messageFilterValueRead == NO)
    {
        DLog(@"Message filter values not read yet. Unable to enable detailed satellite data.");
        return;
    }
    
    value = self.trafficMessagesEnabled;
    value <<= 1;
    value |= 0;
    value <<= 1;
    value |= self.detailedGPSInfoEnabled;
    
    [self setMessageFilterValue:value];
}

# pragma mark • Regular Traffic Messages
-(void)enableShortTrafficMessages
{
    uint8_t value = 0;
    
    if (self.messageFilterValueRead == NO)
    {
        DLog(@"Message filter values not read yet. Unable to enable detailed satellite data.");
        return;
    }
    
    value = 1;
    value <<= 1;
    value |= self.basicAndLongTrafficMessagesEnabled;
    value <<= 1;
    value |= self.detailedGPSInfoEnabled;
    
    [self setMessageFilterValue:value];
}

-(void)disableShortTrafficMessages
{
    uint8_t value = 0;
    
    if (self.messageFilterValueRead == NO)
    {
        DLog(@"Message filter values not read yet. Unable to enable detailed satellite data.");
        return;
    }
    
    value |= self.basicAndLongTrafficMessagesEnabled;
    value <<= 1;
    value |= self.detailedGPSInfoEnabled;
    
    [self setMessageFilterValue:value];
}

# pragma mark • LED brightness
-(void)queryLEDBrightness
{
	UInt16	size;
	unsigned char	buf[64];
	unsigned char	item[32];
    
    if (DEBUG_DEVICE_SETTINGS) DLog(@"Querying LED brightness setting.");
	
	item[0] = item_LedBrightness;
	size = gdl90_build_packet( buf, GDL90MSG_TO_XGPS170, cmd170_query, item, 1 );
	if( size > 0 ) {
		// send 'size' amount of data in 'buf'
        int bytesOut = [self sendDataToBlackjack:buf length:size];
        if (DEBUG_DEVICE_SETTINGS)
        {
            DLog(@"%d bytes written out to device:", bytesOut);
            for (int j=0; j<size; j++) printf("0x%.2X ", buf[j]);
            printf("\n");
        }
	}
}

-(void)setLEDBrightness:(uint8_t)brightLevel ADSBDimLevel:(uint8_t)dimLevel
{
	UInt16	size;
	unsigned char	buf[64];
	unsigned char	item[32];
    
    if (DEBUG_DEVICE_SETTINGS) DLog(@"Setting LED brightness to %d%% and %d%%.", brightLevel, dimLevel);
    
    // do not turn LEDs all the way off
    if (brightLevel < 1) brightLevel = 10;
    if (dimLevel < 1) dimLevel = 10;
	
	item[0] = item_LedBrightness;
	item[1] = brightLevel;
    item[2] = dimLevel;
	size = gdl90_build_packet( buf, GDL90MSG_TO_XGPS170, cmd170_set, item, 3);
	if( size > 0 )
    {
		// send 'size' amount of data in 'buf'
        int bytesOut = [self sendDataToBlackjack:buf length:size];
        if (DEBUG_DEVICE_SETTINGS)
        {
            DLog(@"%d bytes written out to device:", bytesOut);
            for (int j=0; j<size; j++) printf("0x%.2X ", buf[j]);
            printf("\n");
        }
	}
}

# pragma mark • Read Settings
-(void)readDeviceSettings
{
    [self addCommandRequestToQueue:NSStringFromSelector(@selector(queryMessageFilterValue))];
    [self addCommandRequestToQueue:NSStringFromSelector(@selector(queryLEDBrightness))];
}

@end
