//
//  Blackjack.h
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <UIKit/UIKit.h>
#import <ExternalAccessory/ExternalAccessory.h>
#import <MapKit/MapKit.h>

#define DLog(fmt, ...) if (1) NSLog((@"%s [Line %d] " fmt), __PRETTY_FUNCTION__, __LINE__, ##__VA_ARGS__);
#define create_delegate_instance_d AppDelegate *d = (AppDelegate *)[[UIApplication sharedApplication] delegate];

@interface Blackjack : NSObject <EAAccessoryDelegate, NSStreamDelegate>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// These are the variables which are significant to your app:

@property bool appWantsADSBData;    // Set to YES/NO to start/stop data from from the XGPS170. Alternatively,
                                    // post ADSBData_Start or ADSBData_Stop notifications in your code.

// This is the positional information available
@property uint8_t fixType;
@property bool gpsPositionIsValid;
@property bool waasInUse;
@property (nonatomic, strong) NSNumber    *latitude;
@property (nonatomic, strong) NSNumber    *longitude;
@property (nonatomic, strong) NSNumber    *altitude;
@property (nonatomic, strong) NSNumber    *velocity;
@property (nonatomic, strong) NSNumber    *heading;
@property (nonatomic, strong) NSNumber    *numOfSatInUse;
@property (nonatomic, strong) NSNumber    *numOfSatInView;
@property (nonatomic, strong) NSMutableArray        *satsUsedInPosCalc;
@property (nonatomic, strong) NSMutableDictionary   *dictOfSatInfo;

// This is the information available about the XGPS170 hardware
@property bool   isPaired;       // YES when the XGPS170 is paired to bluetooth and available to the iOS
@property bool   isConnected;    // YES if connection established between the iPxx and the blackjack
@property float  batteryVoltage; // device battery voltage. 0.0 to 1.0.
@property bool   batteryIsLow;   // YES when device needs recharged
@property bool   isCharging;     // YES when device is connected to power
@property (nonatomic, strong) NSString *firmwareRev;    // Blackjack firmware version
@property (nonatomic, strong) NSString *serialNumber;   // Blackjack serial number

// These are for the decoded ADS-B data
@property (atomic, strong) NSMutableDictionary *fisbBlocks;
@property (atomic, strong) NSMutableDictionary *fisbCONUSBlocks;
@property (nonatomic, strong) NSDate *timeOfLastNEXRADReception;
@property (nonatomic, strong) NSDate *timeOfLastCONUSNEXRADReception;
@property MKCoordinateRegion nexradWeatherCoverageArea;
@property MKCoordinateRegion conusWeatherCoverageArea;

// This contains information about what data is available
@property uint32_t  adsbDataAvailable;

// These are the device settings
@property bool  detailedGPSInfoEnabled;
@property bool  basicAndLongTrafficMessagesEnabled;
@property bool  trafficMessagesEnabled;
@property uint8_t   ledBrightnessLevel;         // device LED brightness in %
@property uint8_t   dimmingPointForADSBLED;     // LED brightness level which ADS-B dims to when flashing (during reception)

//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////






//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// The variables below have interesting statistics about the ADS-B data coming in:

@property uint16_t towerList;   // the TIS-B site IDs (0-15) of all radio stations being picked up.
                                // Note: only 1 tower is needed to get a complete set of FIS-B data.
@property uint16_t messageCount;    // a count of the ADS-B weather and traffic messages coming. Reset this
                                    // value to zero after every Heartbeat to get a msgs/sec value.
@property NSMutableDictionary *towerLocations;   // The CLLocationCoordinate2D for each tower received is stored
                                            // here
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////







//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// The variables below are used only by the Blackjack class and are of no interest to your app:

// These are for communicating with the blackjack device
@property bool notificationType;
@property (strong, nonatomic) NSNotification *mostRecentNotification;
@property (strong, nonatomic) EAAccessory *accessory;
@property (strong, nonatomic) EASession   *session;
@property (strong, nonatomic) NSString *protocolString;
@property bool messageFilterValueRead;
@property bool detailedGPSInfoSettingRead;
@property bool ledBrightnessSettingsRead;

// These are for managing the data streams
@property bool  gdl90MessageFragmentContainsValidData;
@property bool  adsbSessionIsClosed;
@property bool  sessionsWithBlackjackAreClosed;
@property (nonatomic, strong) NSData    *gdl90MessageFragment;
@property (nonatomic, strong) NSMutableArray    *crc16Table;

// These are for managing the command queuing
@property (nonatomic, strong) NSMutableArray *queue;
@property (nonatomic, strong) NSTimer *queueTimer;
@property bool queueTimerStarted;

// Reserved for future use
@property uint32_t icaoAddress;                             // currently unused
@property uint8_t  emitterCategory;                         // currently unused
@property (nonatomic, strong) NSString    *callSign;        // currently unused

typedef struct
{
    float latitude;
    float longitude;
} LatLon;

//                                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

@end
