//
//  Blackjack.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#import "Blackjack.h"
#import "Blackjack+ADSBDataMgr.h"
#import "Blackjack+ConnectionMgr.h"
#import "Blackjack+LifeCycleMgr.h"
#import <QuartzCore/QuartzCore.h>

#define kADSBStreamBufferSize	1024    // I/O stream buffer size

@implementation Blackjack

#pragma mark - Stream Processing
- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)eventCode
{
	switch(eventCode)
    {
		case NSStreamEventEndEncountered:
		{
			[aStream close];
            [aStream removeFromRunLoop:[NSRunLoop currentRunLoop]
                               forMode:NSDefaultRunLoopMode];
			[aStream setDelegate:nil];
            
            // Disconnect from the blackjack. NOTE: a NSStreamEndEventEncountered can occur before the
			// EAAccessoryDidDisconnectNotification is generated if the blackjack is turned off. But
            // both will events will be generated. So use [self queueDisconnectNotifications:nil]
            // to close the connection here instead of [self sessionClose] to prevent a race condition
            // that results in an error.
            [self queueDisconnectNotifications:nil];
			
			// Post a notification that the blackjack has disconnected
			NSNotification *notification = [NSNotification notificationWithName:@"BlackjackDisconnected" object:self];
            [[NSNotificationCenter defaultCenter] postNotification:notification];
			
            break;
		}
		case NSStreamEventHasBytesAvailable:
		{
			int				total = 0;
			uint			len = 0;
			uint8_t         adsbDataBuffer[kADSBStreamBufferSize];
			
            // read ADSB stream
            total = 0;
            len = 0;
            if (self.appWantsADSBData)
            {    
                do {
                    len = [[_session inputStream] read:adsbDataBuffer maxLength:(kADSBStreamBufferSize-1)];
                    
                    if( len <= 0 || len >= kADSBStreamBufferSize ) {
                        if( total == 0 )
                        {
                            //NSLog(@"%s. Received 0 bytes.", __FUNCTION__);
                        }
                        break;
                    }
                    
                    total += len;
                    
                    // check for errors in the stream
                    adsbDataBuffer[len] = 0;
                    if( adsbDataBuffer[0] != '@' )
                    {
                        //NSString* str3 = [NSString stringWithFormat:@"%s", adsbDataBuffer];
                        //NSLog(@"%s. buffer: %@.", __FUNCTION__, str3);
                    }
                    
                    if (len > 0)
                        ////// TODO: PLACE THE CALL TO YOUR DATA PARSING METHOD HERE
                        [self parseADSBStreamData:(uint8_t *)adsbDataBuffer length:len];
                        ///////////////////////////////////////////////////////////////
                    
                } while (1);
            }
            
			break;
		}
			
		case NSStreamEventHasSpaceAvailable:
		{
			//NSLog(@"%s. NSStreamEventHasSpaceAvailable\n", __FUNCTION__);
			break;
		}
			
		case NSStreamEventErrorOccurred:
		{
			//NSLog(@"%s. NSStreamEventErrorOccurred\n", __FUNCTION__);
			break;
		}
		case NSStreamEventNone:
		{
			//NSLog(@"%s. NSStreamEventNone\n", __FUNCTION__);
			break;			
		}
		case NSStreamEventOpenCompleted:
		{
			//NSLog(@"%s. NSStreamEventOpenCompleted\n", __FUNCTION__);
			break;			
		}
		default:
		{
			//NSLog(@"%s. Some other stream event occurred.\n", __FUNCTION__);
			break;						
		}
	}
	
}

@end
