//
//  CONUSTestWeatherDataOverlay.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2014 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "CONUSTestWeatherDataOverlay.h"
#import "AppDelegate.h"

@implementation CONUSTestWeatherDataOverlay

@synthesize coordinate;
@synthesize boundingMapRect;

- (instancetype)initWithRegion:(MKCoordinateRegion)region
{
    if ((self = [super init]))
    {
        coordinate = region.center;
        
        double upperLeftLat = region.center.latitude + (0.5 * region.span.latitudeDelta);
        double upperLeftLon = region.center.longitude - (0.5 * region.span.longitudeDelta);
        
        double upperRightLat = upperLeftLat;
        double upperRightLon = region.center.longitude + (0.5 * region.span.longitudeDelta);
        
        double lowerLeftLat = region.center.latitude - (0.5 * region.span.latitudeDelta);
        double lowerLeftLon = upperLeftLon;
        
        CLLocationCoordinate2D overlayTopLeftCoordinate = CLLocationCoordinate2DMake(upperLeftLat, upperLeftLon);
        CLLocationCoordinate2D overlayTopRightCoordinate = CLLocationCoordinate2DMake(upperRightLat, upperRightLon);
        CLLocationCoordinate2D overlayBottomLeftCoordinate = CLLocationCoordinate2DMake(lowerLeftLat, lowerLeftLon);
        
        MKMapPoint topLeft = MKMapPointForCoordinate(overlayTopLeftCoordinate);
        MKMapPoint topRight = MKMapPointForCoordinate(overlayTopRightCoordinate);
        MKMapPoint bottomLeft = MKMapPointForCoordinate(overlayBottomLeftCoordinate);
        
        boundingMapRect = MKMapRectMake(topLeft.x,
                                        topLeft.y,
                                        fabs(topLeft.x - topRight.x),
                                        fabs(topLeft.y - bottomLeft.y));
        
        
    }
    return self;
}

@end