//
//  Light.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2014 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Light.h"
#import <QuartzCore/QuartzCore.h>

@implementation Light

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        [self setBackgroundColor:[UIColor clearColor]];
        self.userInteractionEnabled = NO;
    }
    return self;
}


- (void)flash
{
    float onDuration = 1.0;
    float fadeDuration = 0.3;
    
    CALayer *caLayer = self.layer;
    
    self.alpha = 1.0;
    
    [UIView animateWithDuration:fadeDuration delay:onDuration options:UIViewAnimationCurveEaseOut animations:^{
        
        // Change the opacity implicitly.
        caLayer.opacity = 0.0;
     
        // Change the position explicitly.
        CABasicAnimation* theAnim = [CABasicAnimation animationWithKeyPath:@"position"];
        theAnim.fromValue = [NSNumber numberWithFloat:1.0f];
        theAnim.toValue = [NSNumber numberWithFloat:0.0f];
        theAnim.duration = onDuration;
        [caLayer addAnimation:theAnim forKey:@"AnimateFrame"];
    } completion:nil];
    
    [CATransaction flush];
}


void drawLinearGradientForLight(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef  endColor) {
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGFloat locations[] = { 0.0, 1.0 };
    
    NSArray *colors = [NSArray arrayWithObjects:(__bridge id)startColor, (__bridge id)endColor, nil];
    
    CGGradientRef gradient = CGGradientCreateWithColors(colorSpace, (CFArrayRef) colors, locations);
    
    CGPoint startPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMinY(rect));
    CGPoint endPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMaxY(rect));
    
    CGContextSaveGState(context);
    CGContextAddRect(context, rect);
    CGContextClip(context);
    CGContextDrawLinearGradient(context, gradient, startPoint, endPoint, 0);
    CGContextRestoreGState(context);
    
    CGGradientRelease(gradient);
}

void drawGlossAndGradientForLight(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef endColor) {
    
    drawLinearGradientForLight(context, rect, startColor, endColor);
    
    CGColorRef glossColor1 = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.35].CGColor;
    CGColorRef glossColor2 = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.1].CGColor;
    
    CGRect topHalf = CGRectMake(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height/2);
    
    drawLinearGradientForLight(context, topHalf, glossColor1, glossColor2);
    
}

CGMutablePathRef createRoundedRectForRectForLight(CGRect rect, CGFloat radius) {
    
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
    CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect),
                        CGRectGetMaxX(rect), CGRectGetMaxY(rect), radius);
    CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect),
                        CGRectGetMinX(rect), CGRectGetMaxY(rect), radius);
    CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect),
                        CGRectGetMinX(rect), CGRectGetMinY(rect), radius);
    CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect),
                        CGRectGetMaxX(rect), CGRectGetMinY(rect), radius);
    CGPathCloseSubpath(path);
    
    return path;
}

- (void)drawGradientCircleForLight:(CGContextRef)context
              centerPointX:(float)centerX
              centerPointY:(float)centerY
                    radius:(int)radius
              initialColor:(CGColorRef)topColor
                 fadeColor:(CGColorRef)bottomColor
{
    CGRect outerRect = CGRectMake(centerX - radius, centerY - radius, 2*radius, 2*radius);
    CGMutablePathRef outerPath = createRoundedRectForRectForLight(outerRect, radius);
    
    CGContextSaveGState(context);
    CGContextAddPath(context, outerPath);
    CGContextClip(context);
    drawGlossAndGradientForLight(context, outerRect, topColor, bottomColor);
    CGContextRestoreGState(context);
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect
 {
     CGContextRef context = UIGraphicsGetCurrentContext();
     CGContextSaveGState(context);
     
     CGColorRef startColor, fadeToColor;
     float originX = self.frame.size.width / 2.0;
     float originY = self.frame.size.height / 2.0;
     float bulbRadius = self.frame.size.width / 4.0;

     // set the correct fill color
     startColor = [UIColor colorWithRed:105.0/255.0 green:216.0/255.0 blue:123.0/255.0 alpha:1.0].CGColor;
     fadeToColor = [UIColor colorWithRed:22.0/255.0 green:137.0/255.0 blue:41.0/255.0 alpha:1.0].CGColor;
     
     // draw the filled dot
     [self drawGradientCircleForLight:context centerPointX:originX+1.0 centerPointY:originY+1.0 radius:bulbRadius initialColor:startColor fadeColor:fadeToColor];
     
     CGContextRestoreGState(context);
 }


@end
