//
//  Lightbulb.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2014 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#import "Lightbulb.h"

@implementation Lightbulb

static inline double radians (double degrees) { return degrees * M_PI/180; }

- (void)flash
{
    [self.light flash];
}

- (void)setDefaults
{
    self.originX = self.frame.size.width / 2.0;
    self.originY = self.frame.size.height / 2.0;
    
    [self setBackgroundColor:[UIColor clearColor]];
    self.userInteractionEnabled = NO;
    self.bulbRadius = self.frame.size.width / 4.0;

    self.light = [[Light alloc] initWithFrame:CGRectMake(0.0, 0.0, self.frame.size.width - 2.0, self.frame.size.height - 2.0)];
    self.light.alpha = 0.0;
    [self addSubview:self.light];
    [self bringSubviewToFront:self.light];
}

- (id)initWithFrame:(CGRect)frame {
    
    if ((self = [super initWithFrame:frame]))
    {
        // Initialization code.
        [self setDefaults];
    }
    
	return self;
}

// draw a circle centered on the passed in x,y coordinates
- (void) drawCenteredCircle:(CGContextRef)context
			   centerPointX:(float)centerX
			   centerPointY:(float)centerY
					 radius:(int)radius
			 colorForStroke:(UIColor *)strokeColor
			   colorForFill:(UIColor *)fillColor
{
	float sides = 100;
	
	CGContextSaveGState(context);
	
	if (strokeColor == NULL) CGContextSetStrokeColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	
	if (fillColor == NULL) CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetFillColorWithColor(context, fillColor.CGColor);
	
	CGContextSetLineWidth(context, 2);
	
	for(int i = 0; i <= sides; i++)
	{
		CGFloat pointRatio = (float)i / sides;
		CGFloat xSteps = cos( pointRatio * 2.0f * M_PI);
		CGFloat ySteps = sin( pointRatio * 2.0f * M_PI );
		CGFloat pointX = centerX + xSteps * (float)radius;
		CGFloat pointY = centerY + ySteps * (float)radius;
		
		if (i == 0) CGContextMoveToPoint(context, pointX, pointY);
		CGContextAddLineToPoint(context, pointX, pointY);
	}
	CGContextDrawPath(context, kCGPathFillStroke);
    
	CGContextRestoreGState(context);
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextSaveGState(context);
    
	// draw the outline of the lightbulb (the part that doesn't fade out)
	[self drawCenteredCircle:context centerPointX:self.originX centerPointY:self.originY radius:self.bulbRadius colorForStroke:[UIColor colorWithWhite:1.0 alpha:0.5] colorForFill:NULL];

	CGContextRestoreGState(context);
}

@end