//
//  SettingsPopoverVC.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#import "SettingsPopoverVC.h"
#import "Blackjack+ControlMgr.h"
#import "AppDelegate.h"

#define DEBUG_CONTROLS  NO
#define FW_IODELAY      0.2

#define kShortTrafficMessagesOnly     0
#define kBasicLongTrafficMessagesOnly       1
#define kAllTrafficMessages                 2

#define kDimmedLEDBrightnessLevel   5
#define kDimmedADSBBrightnessLevel  0
#define kNormalLEDBrightnessLevel   25
#define kNormalADSBBrightnessLevel  10
#define kBrightLEDBrightnessLevel   100
#define kBrightADSBBrightnessLevel  40

@implementation SettingsPopoverVC

#pragma mark - UISegmentedControl action methods
- (IBAction)enabledTrafficControlTouched:(id)sender
{
    create_delegate_instance_d

    UISegmentedControl *control = (UISegmentedControl *)sender;
    
    switch (control.selectedSegmentIndex) {
        case kShortTrafficMessagesOnly:
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(enableShortTrafficMessages))];
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(queryMessageFilterValue))];
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(disableBasicAndLongTrafficMessages))];
            break;
            
        case kBasicLongTrafficMessagesOnly:
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(disableShortTrafficMessages))];
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(queryMessageFilterValue))];
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(enableBasicAndLongTrafficMessages))];
            break;
            
        case kAllTrafficMessages:
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(enableRegularTrafficMessages))];
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(queryMessageFilterValue))];
            [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(enableBasicAndLongTrafficMessages))];
            break;
            
        default:
            break;
    }
    
    [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(queryMessageFilterValue))];
    
    if (DEBUG_CONTROLS) NSLog(@"%s. Control value is %d.", __FUNCTION__, control.selectedSegmentIndex);
}

- (IBAction)detailedGPSInfoControlTouched:(id)sender
{
    create_delegate_instance_d

    UISegmentedControl *control = (UISegmentedControl *)sender;
    
    if (control.selectedSegmentIndex == 0) [d.blackjack enableDetailedSatData];
    else [d.blackjack disableDetailedSatData];
    
    [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(queryMessageFilterValue))];
}

- (IBAction)ledBrightnessControlTouched:(id)sender
{
    create_delegate_instance_d

    UISegmentedControl *control = (UISegmentedControl *)sender;
    
    switch (control.selectedSegmentIndex) {
        case 0:
            [d.blackjack setLEDBrightness:kDimmedLEDBrightnessLevel ADSBDimLevel:kDimmedADSBBrightnessLevel];
            break;
            
        case 1:
            [d.blackjack setLEDBrightness:kNormalLEDBrightnessLevel ADSBDimLevel:kNormalADSBBrightnessLevel];
            break;
            
        case 2:
            [d.blackjack setLEDBrightness:kBrightLEDBrightnessLevel ADSBDimLevel:kBrightADSBBrightnessLevel];
            break;
            
        default:
            break;
    }
    
    [d.blackjack addCommandRequestToQueue:NSStringFromSelector(@selector(queryLEDBrightness))];
}

#pragma mark - View lifecycle
-(void)updateScreenValues
{
    create_delegate_instance_d
    
    if (d.blackjack.messageFilterValueRead)
    {
        if (d.blackjack.detailedGPSInfoEnabled) [self.detailedGPSInfoControl setSelectedSegmentIndex:0];
        else [self.detailedGPSInfoControl setSelectedSegmentIndex:1];
        
        if (d.blackjack.trafficMessagesEnabled)
            if (d.blackjack.basicAndLongTrafficMessagesEnabled) [self.enabledTrafficMessagesControl setSelectedSegmentIndex:2];
            else [self.enabledTrafficMessagesControl setSelectedSegmentIndex:0];
        else
            if (d.blackjack.basicAndLongTrafficMessagesEnabled) [self.enabledTrafficMessagesControl setSelectedSegmentIndex:1];
    }
    
    if (d.blackjack.ledBrightnessSettingsRead)
    {
        if (d.blackjack.ledBrightnessLevel == kDimmedLEDBrightnessLevel) [self.ledBrightnessControl setSelectedSegmentIndex:0];
        else if (d.blackjack.ledBrightnessLevel == kNormalLEDBrightnessLevel) [self.ledBrightnessControl setSelectedSegmentIndex:1];
        else [self.ledBrightnessControl setSelectedSegmentIndex:2];
    }
}

- (void)refreshUIAfterAwakening
{
    create_delegate_instance_d
    
    if (d.blackjack.messageFilterValueRead == NO)
    {
        [d.blackjack addCommandRequestToQueue:@"readDeviceSettings"];
    
        // Give the device a chance to respond with all of the settings values before updating the UI
        [self performSelector:@selector(updateScreenValues) withObject:nil afterDelay:1.0];
    }
    else [self updateScreenValues];
}

- (void)refreshUIAfterDelay
{
    // It takes about 2 seconds after the blackjack connects for the device settings to be
    // read and be validated. So refresh the UI after this delay.
    [self performSelector:@selector(refreshUIAfterAwakening) withObject:nil afterDelay:2.8];
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    create_delegate_instance_d
    Blackjack *b = d.blackjack;
    
    [b addObserver:self forKeyPath:@"messageFilterValueRead" options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionInitial) context:nil];
    [b addObserver:self forKeyPath:@"ledBrightnessSettingsRead" options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionInitial) context:nil];

    [self refreshUIAfterAwakening];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
	return YES;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if ([keyPath isEqualToString:@"messageFilterValueRead"])
    {
        [self.detailedGPSInfoControl setValue:[change objectForKey:NSKeyValueChangeNewKey] forKey:@"enabled"];
        [self.enabledTrafficMessagesControl setValue:[change objectForKey:NSKeyValueChangeNewKey] forKey:@"enabled"];
    }
    else if ([keyPath isEqualToString:@"ledBrightnessSettingsRead"])
    {
        [self.ledBrightnessControl setValue:[change objectForKey:NSKeyValueChangeNewKey] forKey:@"enabled"];
    }
}

#pragma mark -
#pragma mark Application lifecycle
- (void)viewWillDisappear:(BOOL)animated
{
    create_delegate_instance_d
    Blackjack *b = d.blackjack;
    
    [b removeObserver:self forKeyPath:@"messageFilterValueRead"];
    [b removeObserver:self forKeyPath:@"ledBrightnessSettingsRead"];
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        
        // Look for notifications that the blackjack has connected or disconnected
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(refreshUIAfterDelay)
													 name:@"BlackjackConnected"
												   object:nil];

        [[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(refreshUIAfterAwakening)
													 name:@"BlackjackDisconnected"
												   object:nil];

        [[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(updateScreenValues)
													 name:@"DeviceSettingValueReceived"
												   object:nil];
        
        // If the iPod/iPad/iPhone goes to sleep while a view controller is open, there is no notification
		// that the app is back to life, other than this applicationDidBecomeActive method being called. The viewWillAppear,
		// viewDidAppear, or viewDidOpen methods are not triggered when the iPod/iPad/iPhone is woken and the app resumes.
		// Consequently, listen for notification from the app delegate that the app has resumed becuase the UI will need
		// updated if the blackjack status changed while the iPod/iPad/iPhone was asleep.
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(refreshUIAfterAwakening)
													 name:@"refreshUIAfterAwakening"
												   object:nil];
    }
    
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

@end
