//
//  WaitingToConnectView.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "WaitingToConnectView.h"

#define HORIZ 0
#define VERT  1

@implementation WaitingToConnectView

- (void)setDefaults
{
    self.width = 118.0;
    self.height = 14.0;
    self.fontSize = 11;
    self.orientation = HORIZ;
    self.labelText = @"";
    [self setBackgroundColor:[UIColor clearColor]];
}

- (id)init
{
    if ((self = [super init]))
    {
        // Initialization code
        [self setDefaults];
    }
    return self;    
}
- (id)initWithFrame:(CGRect)frame
{
    if ((self = [super initWithFrame:frame]))
    {
        // Initialization code
        [self setDefaults];
    }
    return self;
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
	
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextSaveGState(context);

    // draw the rounded rectangle (pretty much copied from Apples example)
    CGFloat radius = 32;
    CGFloat minx = CGRectGetMinX(rect), midx = CGRectGetMidX(rect), maxx = CGRectGetMaxX(rect);
    CGFloat miny = CGRectGetMinY(rect), midy = CGRectGetMidY(rect), maxy = CGRectGetMaxY(rect);
	
	// Start at 1
	CGContextMoveToPoint(context, minx, midy);
	// Add an arc through 2 to 3
	CGContextAddArcToPoint(context, minx, miny, midx, miny, radius);
	// Add an arc through 4 to 5
	CGContextAddArcToPoint(context, maxx, miny, maxx, midy, radius);
	// Add an arc through 6 to 7
	CGContextAddArcToPoint(context, maxx, maxy, midx, maxy, radius);
	// Add an arc through 8 to 9
	CGContextAddArcToPoint(context, minx, maxy, minx, midy, radius);
	// Close the path
	CGContextClosePath(context);
	
	//CGContextSetStrokeColorWithColor(context, [[UIColor whiteColor] CGColor]);
    CGContextSetRGBFillColor(context, 1.0, 0.0, 0.0, 1.0);
	
	// Fill & stroke the path
	CGContextDrawPath(context, kCGPathFill);
		
	CGContextRestoreGState(context);
	
	// add the labels, spinner and troubleshooting button
	UILabel *waitingLabel = [[UILabel alloc] initWithFrame:CGRectMake(11,5,179,53)];
	waitingLabel.backgroundColor = [UIColor clearColor];
	waitingLabel.textColor = [UIColor whiteColor];
	waitingLabel.textAlignment = NSTextAlignmentCenter;
	waitingLabel.font = [UIFont fontWithName:@"Helvetica-Bold" size:16];
	waitingLabel.numberOfLines = 2;
	waitingLabel.text = NSLocalizedString(@"Waiting for the XGPS170...", @"WaitingToConnectView message");
	UIActivityIndicatorView *spinner = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(82,66,37,37)];
	spinner.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
    spinner.isAccessibilityElement = NO;
	[spinner startAnimating];
    
	[self addSubview:waitingLabel];
	[self addSubview:spinner];
	
}

@end
