//
//  iPadViewController.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define kNoblackjackMessageView          99
#define kNumberOfBatteryVoltageSamplesToAverage 5
#define kFontSize	17
#define kThresholdBetweenRedAndYellowSNR    0.15
#define kThresholdBetweenYellowAndGreenSNR  0.25

#define kLowResolutionCONUSDataScaling      9
#define kMediumResolutionCONUSDataScaling   5
#define kHighResoltuionCONUSDataScaling     1

#define kNEXRADRegionPolygonTag   99
#define kCONUSRegionPolygonTag    101

#define kNEXRADProcessingDelayTime  10
#define kCONUSProcessingDelayTime   10

#define kTestDataOriginLatitude  28.7778
#define kTestDataOriginLongitude -81.2375
#define kTestDataNEXRADRegionSize 0.25

#define kZoomDistance 240000

#define kScrollViewCharacterLimit   10000

#import "iPadViewController.h"
#import "AppDelegate.h"
#import "WaitingToConnectView.h"
#import "Blackjack.h"
#import "Blackjack+ADSBDataMgr.h"
#import "Blackjack+ControlMgr.h"
#import "BlackjackDataStructures.h"
#import "SettingsPopoverVC.h"
#import "NEXRADWeatherOverlay.h"
#import "NEXRADWeatherOverlayRenderer.h"
#import "CONUSWeatherOverlay.h"
#import "CONUSWeatherOverlayRenderer.h"
#import "NEXRADTestWeatherDataOverlay.h"
#import "CONUSTestWeatherDataOverlay.h"
#import "NEXRADTestWeatherDataOverlayRenderer.h"
#import "CONUSTestWeatherDataOverlayRenderer.h"
#import "TargetPointAnnotation.h"
#import "ADSBTowerAnnotation.h"
#import "ADSBTowerAnnotationView.h"
#import "OwnshipPositionAnnotation.h"
#import "OwnshipPositionAnnotationView.h"
#import "ReceivedNEXRADRegionPolygon.h"
#import "ReceivedCONUSRegionPolygon.h"

@interface iPadViewController()

@property char shifter;
@property BOOL connectedFlasher;
@property BOOL NEXRADTestPatternCreated;
@property BOOL CONUSTestPatternCreated;
@property BOOL nexradProcessingDelayTimerStarted;
@property BOOL conusProcessingDelayTimerStarted;
@property BOOL newNEXRADReceived;
@property BOOL newCONUSReceived;
@property unsigned int towerCount;

@property (nonatomic, strong) NEXRADWeatherOverlay *nexradOverlay;
@property (nonatomic, strong) CONUSWeatherOverlay *conusOverlay;
@property (nonatomic, strong) NEXRADTestWeatherDataOverlay *nexradTestWeatherDataOverlay;
@property (nonatomic, strong) CONUSTestWeatherDataOverlay *conusTestWeatherDataOverlay;

@property (nonatomic, strong) NSMutableArray *arrayOfNEXRADBlockOutlineOverlays;
@property (nonatomic, strong) NSMutableArray *arrayOfCONUSBlockOutlineOverlays;
@property (nonatomic, strong) NSMutableArray *batVoltagesArray;
@property (nonatomic, strong) NSMutableArray *towerLocationOverlays;

@property (nonatomic, strong) UIImage *createdNEXRADImage;
@property (nonatomic, strong) UIImage *createdCONUSImage;
@property (nonatomic, strong) UIImage *createdNEXRADImageFromTestData;
@property (nonatomic, strong) UIImage *createdCONUSImageFromTestData;

@property (nonatomic, strong) UIImageView *dualLogoImage;
@property (nonatomic, strong) UIImageView *blackjackImageView;

@property (nonatomic, strong) OwnshipPositionAnnotation *ownshipMarker;

@property (nonatomic, strong) Bar *batLevelBargraph;

@property (nonatomic, retain) UIPopoverController *currentPopover;

@property (nonatomic, strong) UILabel *dualLogoImageLabel;
@property (nonatomic, strong) UILabel *deviceInformationLabel;
@property (nonatomic, strong) UILabel *statusLabel;
@property (nonatomic, strong) UILabel *chargingLabel;
@property (nonatomic, strong) UILabel *firmwareVersionLabel;
@property (nonatomic, strong) UILabel *statusValue;
@property (nonatomic, strong) UILabel *chargingValue;
@property (nonatomic, strong) UILabel *firmwareVersionValue;
@property (nonatomic, strong) UILabel *adsbServiceLabel;
@property (nonatomic, strong) UILabel *receivingDataLabel;
@property (nonatomic, strong) UILabel *receivingDataValueWaiting;
@property (nonatomic, strong) UILabel *receivingDataValueReceiving;
@property (nonatomic, strong) UILabel *nexradLabel;
@property (nonatomic, strong) UILabel *conusLabel;
@property (nonatomic, strong) UILabel *trafficLabel;
@property (nonatomic, strong) UILabel *notamsLabel;
@property (nonatomic, strong) UILabel *metarsLabel;
@property (nonatomic, strong) UILabel *tafsLabel;
@property (nonatomic, strong) UILabel *datisLabel;
@property (nonatomic, strong) UILabel *airmetLabel;
@property (nonatomic, strong) UILabel *sigmetLabel;
@property (nonatomic, strong) UILabel *pirepsLabel;
@property (nonatomic, strong) UILabel *windsLabel;
@property (nonatomic, strong) UILabel *airspaceUpdateLabel;
@property (strong, nonatomic) UILabel *nexradAgeLabel;
@property (strong, nonatomic) UILabel *conusNexradAgeLabel;
@property (strong, nonatomic) UILabel *nexradAgeValue;
@property (strong, nonatomic) UILabel *conusNexradAgeValue;
@property (nonatomic, strong) UILabel *receivedMsgsLabel;
@property (nonatomic, strong) UILabel *positionInfoLabel;
@property (nonatomic, strong) UILabel *latitudeLabel;
@property (nonatomic, strong) UILabel *longitudeLabel;
@property (nonatomic, strong) UILabel *altitudeLabel;
@property (nonatomic, strong) UILabel *timeLabel;
@property (nonatomic, strong) UILabel *headingLabel;
@property (nonatomic, strong) UILabel *speedLabel;
@property (nonatomic, strong) UILabel *latitudeValue;
@property (nonatomic, strong) UILabel *longitudeValue;
@property (nonatomic, strong) UILabel *altitudeValue;
@property (nonatomic, strong) UILabel *timeValue;
@property (nonatomic, strong) UILabel *headingValue;
@property (nonatomic, strong) UILabel *speedValue;
@property (nonatomic, strong) UILabel *waasInUseLabel;
@property (nonatomic, strong) UILabel *waasInUseValue;
@property (nonatomic, strong) UILabel *towerCountLabel;
@property (nonatomic, strong) UILabel *towerCountValue;
@property (nonatomic, strong) UILabel *messageCountLabel;
@property (nonatomic, strong) UILabel *messageCountValue;

@property (strong, nonatomic) Lightbulb *nexradBulb;
@property (strong, nonatomic) Lightbulb *conusBulb;
@property (strong, nonatomic) Lightbulb *trafficBulb;
@property (strong, nonatomic) Lightbulb *notamsBulb;
@property (strong, nonatomic) Lightbulb *metarsBulb;
@property (strong, nonatomic) Lightbulb *tafsBulb;
@property (strong, nonatomic) Lightbulb *datisBulb;
@property (strong, nonatomic) Lightbulb *airmetsBulb;
@property (strong, nonatomic) Lightbulb *sigmetsBulb;
@property (strong, nonatomic) Lightbulb *windsBulb;
@property (strong, nonatomic) Lightbulb *pirepsBulb;
@property (strong, nonatomic) Lightbulb *airspaceBulb;

@property (nonatomic, strong) WaitingToConnectView *blackjackNotConnectedMessage;

@property (nonatomic, strong) NSMutableAttributedString *scrollViewText;

@property (nonatomic, strong) NSTimer *nexradProcessingDelayTimer;
@property (nonatomic, strong) NSTimer *conusProcessingDelayTimer;

@end

#pragma mark -
@implementation iPadViewController

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - Class methods
- (void)updateADSBTransmissionStatus
{
    // This method makes the Receiver Status label change value
    self.receivingDataValueWaiting.alpha = 0.0;
    self.receivingDataValueReceiving.alpha = 1.0;
    
    [UIView animateWithDuration:0.0 delay:1.0 options:UIViewAnimationOptionTransitionNone animations:^{
        self.receivingDataValueWaiting.alpha = 1.0;
        self.receivingDataValueReceiving.alpha = 0.0;

    } completion:nil];
}

- (void)updateWithNewSatData
{
	create_delegate_instance_d
    
	// update the position labels
	if ((d.blackjack.isConnected == NO) && (self.notConMessageSwitch.on == YES))
	{
		self.latitudeValue.text = @"";
		self.longitudeValue.text = @"";
		self.altitudeValue.text = @"";
		self.headingValue.text = @"";
		self.speedValue.text = @"";
        self.waasInUseValue.text = @"";
	}
    else if (d.blackjack.fixType < 2) // ([d.blackjack.latitude intValue] == 0)
	{
		self.latitudeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.latitudeValue.text = NSLocalizedString(@"   Waiting for", @"iPadVC:updateWithNewPosition:common:Waiting for satellite info text");
		
		self.longitudeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.longitudeValue.text = NSLocalizedString(@"     satellite", @"iPadVC:updateWithNewPosition:common:Waiting for satellite info text");
		
		self.altitudeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.altitudeValue.text = NSLocalizedString(@"        data", @"iPadVC:updateWithNewPosition:common:Waiting for satellite info text");
		
		self.headingValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.headingValue.text = @"--";
        
		self.speedValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.speedValue.text = @"--";
        
		self.waasInUseValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.waasInUseValue.text = @"--";
        
        [self clearGPSPositionPin];
	}
    else
    {
        self.latitudeValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
        self.longitudeValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
        self.waasInUseValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
        
        float degrees, mins, secs;
        float lat = fabsf([d.blackjack.latitude floatValue]);
        float lon = fabsf([d.blackjack.longitude floatValue]);
		if (d.useDMSForPosition)
        {
            degrees = (int)lat;
            mins = (lat - degrees) * 60.0;
            secs = (mins - (int)mins) * 60.0;
            self.latitudeValue.text = [NSString stringWithFormat:@"%.0f˚ %.0f' %.0f\" %@", degrees, mins, secs, ([d.blackjack.latitude floatValue] > 0) ? @"N" : @"S"];
            
            degrees = (int)lon;
            mins = (lon - degrees) * 60.0;
            secs = (mins - (int)mins) * 60.0;
            self.longitudeValue.text = [NSString stringWithFormat:@"%.0f˚ %.0f' %.0f\" %@", degrees, mins, secs, ([d.blackjack.longitude floatValue] < 0) ? @"W" : @"E"];
        }
        else if (d.useDMForPosition)
        {
            degrees = (int)lat;
            mins = (lat - degrees) * 60.0;
            self.latitudeValue.text = [NSString stringWithFormat:@"%.0f˚ %.3f' %@", degrees, mins, ([d.blackjack.latitude floatValue] > 0) ? @"N" : @"S"];
            
            degrees = (int)lon;
            mins = (lon - degrees) * 60.0;
            self.longitudeValue.text = [NSString stringWithFormat:@"%.0f˚ %.3f' %@", degrees, mins, ([d.blackjack.longitude floatValue] < 0) ? @"W" : @"E"];
        }
        else
        {
            self.latitudeValue.text = [NSString stringWithFormat:@"%.3f˚ %@", fabsf([d.blackjack.latitude floatValue]), ([d.blackjack.latitude floatValue] > 0) ? @"N" : @"S"];
            self.longitudeValue.text = [NSString stringWithFormat:@"%.3f˚ %@",fabsf([d.blackjack.longitude floatValue]), ([d.blackjack.longitude floatValue] > 0) ? @"E" : @"W"];
        }
        
        if (d.blackjack.fixType == 2)   // 2D fix
        {
            self.altitudeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
            self.altitudeValue.text = @"need 1 more sat";
            self.waasInUseValue.text = @"No";
        }
        else   // 3D fix
        {
            self.altitudeValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
            if (d.useMetricForAlt == YES) self.altitudeValue.text = [NSString stringWithFormat:@"%.0f m", [d.blackjack.altitude floatValue] / 3.2808399];
            else self.altitudeValue.text = [NSString stringWithFormat:@"%.0f ft", [d.blackjack.altitude floatValue]];
            self.waasInUseValue.text = @"Yes";
        }

        self.speedValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
		if (d.useKnots == YES) self.speedValue.text = [NSString stringWithFormat:@"%.0f kts", [d.blackjack.velocity floatValue]];
		else if (d.useMetricForSpeed == YES) self.speedValue.text = [NSString stringWithFormat:@"%.0f kph", [d.blackjack.velocity floatValue] * 1.85200f];
		else self.speedValue.text = [NSString stringWithFormat:@"%.0f mph", ([d.blackjack.velocity floatValue] * 1.15077945)];
		
        // only show a heading if moving
        self.headingValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
        if ([d.blackjack.velocity floatValue] > 2.0)
            self.headingValue.text = [NSString stringWithFormat:@"%.0f˚ T", [d.blackjack.heading floatValue]];
        else self.headingValue.text = @"---";
        
        // show the location on the map
        [self markGPSLocationWithPin];
    }
}

- (void)updateWithNewDeviceData
{
    create_delegate_instance_d
    
	// update the device labels
	if (d.blackjack.isConnected == NO)
	{
		self.statusValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
		self.statusValue.textColor = [UIColor redColor];
		self.statusValue.text = NSLocalizedString(@"Not connected", @"iPadVC:updateWithNewDeviceData:statusValue:Not connected text");
        self.chargingValue.hidden = YES;
		self.chargingValue.text = @"";
        self.batLevelBargraph.hidden = NO;
		[self.batLevelBargraph updateBar:0.0 leftLabelText:NULL leftLabelEmphasis:YES rightLabelText:@"- %" rightLabelEmphasis:YES];
        self.firmwareVersionValue.text = @"";
        self.nexradAgeValue.text = @"--";
        self.conusNexradAgeValue.text = @"--";
	}
	else
	{
		self.statusValue.font = [UIFont fontWithName:@"Helvetica-Bold" size:kFontSize];
		self.statusValue.textColor = [UIColor greenColor];
        
		// animate the Connected label
		switch (self.connectedFlasher) {
			case YES:
				self.statusValue.text = NSLocalizedString(@"Connected", @"iPadVC:updateWithNewDeviceData:statusValue:Connected text");
				self.connectedFlasher = NO;
				break;
			case NO:
				self.statusValue.text = NSLocalizedString(@"Connected •", @"iPadVC:updateWithNewDeviceData:statusValue:Connected • text");
				self.connectedFlasher = YES;
				break;
			default:
				break;
		}
        
		if (d.blackjack.isCharging)
		{
			self.chargingValue.hidden = NO;
			self.batLevelBargraph.hidden = YES;

			// animate the Charging label
			switch (self.shifter)
            {
				case 0:
					self.chargingValue.text = NSLocalizedString(@"Charging", @"iPadVC:updateWithNewDeviceData:statusValue:Charging text");
					self.shifter++;
					break;
				case 1:
					self.chargingValue.text = NSLocalizedString(@"Charging.", @"iPadVC:updateWithNewDeviceData:statusValue:Charging. text");
					self.shifter++;
					break;
				case 2:
					self.chargingValue.text = NSLocalizedString(@"Charging..", @"iPadVC:updateWithNewDeviceData:statusValue:Charging.. text");
					self.shifter++;
					break;
				case 3:
					self.chargingValue.text = NSLocalizedString(@"Charging...", @"iPadVC:updateWithNewDeviceData:statusValue:Charging... text");
					self.shifter = 0;
					break;
				default:
					break;
			}
		}
		else  // blackjack is connected but not charging
		{
			self.chargingValue.hidden = YES;
			self.batLevelBargraph.hidden = NO;
            self.shifter = 0;
		}
		
        // max battery voltage of the blackjack is 4.15V, min voltage is 3.5
        NSString *batLevelStr;
        
        // average out the battery voltage to smooth the display
        float sumOfSamples = 0.0;
        if ([self.batVoltagesArray count] == kNumberOfBatteryVoltageSamplesToAverage) [self.batVoltagesArray removeObjectAtIndex:0];
        [self.batVoltagesArray addObject:[NSNumber numberWithFloat:d.blackjack.batteryVoltage]];
        for (int i=0; i<[self.batVoltagesArray count]; i++) sumOfSamples += [[self.batVoltagesArray objectAtIndex:i] floatValue];
        float avgBatLevel = sumOfSamples / (float)[self.batVoltagesArray count];
        if (avgBatLevel > 1.0) batLevelStr = @"100%";
        else if (avgBatLevel < 0) batLevelStr = @"0 %";
        else batLevelStr = [NSString stringWithFormat:@"%d%%", (int)(avgBatLevel*100)];
        [self.batLevelBargraph updateBar:avgBatLevel leftLabelText:NULL leftLabelEmphasis:NO rightLabelText:batLevelStr rightLabelEmphasis:YES];
        
        self.firmwareVersionValue.text = [NSString stringWithFormat:@"ver. %@", d.blackjack.firmwareRev];
        
        // update the NEXRAD times
        if (d.blackjack.timeOfLastNEXRADReception == NULL)
        {
            self.nexradAgeValue.text = @"--";
            self.nexradAgeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
        }
        else
        {
            NSDate *now = [NSDate dateWithTimeIntervalSinceNow:0];
            NSTimeInterval nexradAge = [now timeIntervalSinceDate:d.blackjack.timeOfLastNEXRADReception];
            
            self.nexradAgeValue.font = [UIFont fontWithName:@"Helvetica" size:kFontSize];
            
            double mins;
            
            mins = nexradAge / 60;
            if (mins > 59) self.nexradAgeValue.text = @"> 1 hour";
            else
            {
                if (mins < 0.99) self.nexradAgeValue.text = [NSString stringWithFormat:@"%.0f sec.", nexradAge];
                else self.nexradAgeValue.text = [NSString stringWithFormat:@"%d min.", (int)mins];
            }
            
        }
        
        if (d.blackjack.timeOfLastCONUSNEXRADReception == NULL)
        {
            self.conusNexradAgeValue.text = @"--";
            self.conusNexradAgeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:kFontSize];
        }
        else
        {
            NSDate *now = [NSDate dateWithTimeIntervalSinceNow:0];
            NSTimeInterval conusAge = [now timeIntervalSinceDate:d.blackjack.timeOfLastCONUSNEXRADReception];
            
            self.conusNexradAgeValue.font = [UIFont fontWithName:@"Helvetica" size:kFontSize];
            
            double mins;
            
            mins = conusAge / 60;
            if (mins > 59) self.conusNexradAgeValue.text = @"> 1 hour";
            else
            {
                if (mins < 0.99) self.conusNexradAgeValue.text = [NSString stringWithFormat:@"%.0f sec.", conusAge];
                else self.conusNexradAgeValue.text = [NSString stringWithFormat:@"%d min.", (int)mins];
            }
        }
        
	}
} //updateWithNewDeviceData

- (void)displayblackjackNotAttachedMessage
{
	[self updateWithNewDeviceData];
    self.receivingDataValueWaiting.text = @"Off";
    
    if (self.notConMessageSwitch.on == NO) self.blackjackNotConnectedMessage.hidden = YES;
	else self.blackjackNotConnectedMessage.hidden = NO;
}

- (void)dismissblackjackNotAttachedMessage
{
	[self updateWithNewDeviceData];
    self.receivingDataValueWaiting.text = @"Listening...";
	self.blackjackNotConnectedMessage.hidden = YES;
}

- (void)blackjackConnected
{
	[self dismissblackjackNotAttachedMessage];
}

- (void)blackjackDisconnected
{
	[self displayblackjackNotAttachedMessage];
}

- (void)refreshUIAfterAwakening
{
	create_delegate_instance_d
    
	if (d.blackjack.isConnected == NO)
	{
		[self displayblackjackNotAttachedMessage];
	}
	else
	{
		[self dismissblackjackNotAttachedMessage];
	}
}

- (void)updateUIFromNewData
{
	create_delegate_instance_d
    
    if (d.blackjack.adsbDataAvailable & kTRAFFIC) [self.trafficBulb flash];
    if (d.blackjack.adsbDataAvailable & kNOTAM) [self.notamsBulb flash];
    if (d.blackjack.adsbDataAvailable & kMETAR) [self.metarsBulb flash];
    if (d.blackjack.adsbDataAvailable & kTAF) [self.tafsBulb flash];
    if (d.blackjack.adsbDataAvailable & kDATIS) [self.datisBulb flash];
    if (d.blackjack.adsbDataAvailable & kAIRMET) [self.airmetsBulb flash];
    if (d.blackjack.adsbDataAvailable & kSIGMET) [self.sigmetsBulb flash];
    if (d.blackjack.adsbDataAvailable & kPIREP) [self.pirepsBulb flash];
    if (d.blackjack.adsbDataAvailable & kWINDS) [self.windsBulb flash];
    if (d.blackjack.adsbDataAvailable & kSUAUPDATE) [self.airspaceBulb flash];
    if (d.blackjack.adsbDataAvailable & kRegionalNEXRAD)
    {
        self.newNEXRADReceived = YES;
        [self.nexradBulb flash];
        [self processIncomingNEXRADAfterDelay];
    }
    if (d.blackjack.adsbDataAvailable & kConusNEXRAD)
    {
        self.newCONUSReceived = YES;
        [self.conusBulb flash];
        [self processIncomingCONUSAfterDelay];
    }
    
    // Reset the new data flag
    d.blackjack.adsbDataAvailable = 0;

    // Update the msgs/sec label
    self.messageCountValue.text = [NSString stringWithFormat:@"%d", d.blackjack.messageCount];
    d.blackjack.messageCount = 0;
    
    // Update the lookahead range values
    if (d.blackjack.towerList & 0xE000) // high altitude radio tier
    {
        self.lookaheadNEXRADLabel.text = @"250 NM";
        self.lookaheadCONUSLabel.text = @"All US";
        self.lookaheadNOTAMLabel.text = @"100 NM";
        self.lookaheadMETARLabel.text = @"500 NM";
        self.lookaheadTAFLabel.text = @"500 NM";
        self.lookaheadAIRMETLabel.text = @"500 NM";
        self.lookaheadSIGMETLabel.text = @"500 NM";
        self.lookaheadPIREPLabel.text = @"500 NM";
        self.lookaheadWINDSLabel.text = @"1000 NM";
        self.lookaheadSUALabel.text = @"500 NM";
    }
    else if (d.blackjack.towerList & 0x1C00) // medium altitude radio tier
    {
        self.lookaheadNEXRADLabel.text = @"200 NM";
        self.lookaheadCONUSLabel.text = @"all US";
        self.lookaheadNOTAMLabel.text = @"100 NM";
        self.lookaheadMETARLabel.text = @"375 NM";
        self.lookaheadTAFLabel.text = @"375 NM";
        self.lookaheadAIRMETLabel.text = @"375 NM";
        self.lookaheadSIGMETLabel.text = @"375 NM";
        self.lookaheadPIREPLabel.text = @"375 NM";
        self.lookaheadWINDSLabel.text = @"750 NM";
        self.lookaheadSUALabel.text = @"375 NM";
    }
    else if (d.blackjack.towerList & 0x0380) // low altitude radio tier
    {
        self.lookaheadNEXRADLabel.text = @"150 NM";
        self.lookaheadCONUSLabel.text = @"None";
        self.lookaheadNOTAMLabel.text = @"100 NM";
        self.lookaheadMETARLabel.text = @"250 NM";
        self.lookaheadTAFLabel.text = @"250 NM";
        self.lookaheadAIRMETLabel.text = @"250 NM";
        self.lookaheadSIGMETLabel.text = @"250 NM";
        self.lookaheadPIREPLabel.text = @"250 NM";
        self.lookaheadWINDSLabel.text = @"500 NM";
        self.lookaheadSUALabel.text = @"250 NM";
    }
    else if (d.blackjack.towerList & 0x001E) // surface radio tier
    {
        self.lookaheadNEXRADLabel.text = @"150 NM";
        self.lookaheadCONUSLabel.text = @"None";
        self.lookaheadNOTAMLabel.text = @"100 NM";
        self.lookaheadMETARLabel.text = @"100 NM";
        self.lookaheadTAFLabel.text = @"100 NM";
        self.lookaheadAIRMETLabel.text = @"100 NM";
        self.lookaheadSIGMETLabel.text = @"100 NM";
        self.lookaheadPIREPLabel.text = @"None";
        self.lookaheadWINDSLabel.text = @"500 NM";
        self.lookaheadSUALabel.text = @"None";
    }
    else  // reserved tower ID values - should not be in use
    {
        self.lookaheadNEXRADLabel.text = @"";
        self.lookaheadCONUSLabel.text = @"";
        self.lookaheadNOTAMLabel.text = @"";
        self.lookaheadMETARLabel.text = @"";
        self.lookaheadTAFLabel.text = @"";
        self.lookaheadAIRMETLabel.text = @"";
        self.lookaheadSIGMETLabel.text = @"";
        self.lookaheadPIREPLabel.text = @"";
        self.lookaheadWINDSLabel.text = @"";
        self.lookaheadSUALabel.text = @"";
    }
    
    NSMutableString *tiersInUse = [[NSMutableString alloc] initWithString:@""];
    if (d.blackjack.towerList & 0xE000) [tiersInUse appendString:@" High,"];
    if (d.blackjack.towerList & 0x1C00) [tiersInUse appendString:@" Medium,"];
    if (d.blackjack.towerList & 0x0380) [tiersInUse appendString:@" Low,"];
    if (d.blackjack.towerList & 0x001E) [tiersInUse appendString:@" Surface"];
    self.receivedRadioTierLabel.text = tiersInUse;
    
    // Display the number of towers being recieved. (This is really unnecessary. Only
    if (d.blackjack.towerList == 0) self.towerCountValue.text = @"0";
    else
    {
        unsigned char count=0;
        uint16_t mask = 1;
        for (char i=0; i<16; i++)
        {
            if (d.blackjack.towerList & mask) count++;
            mask <<= 1;
        }
        self.towerCountValue.text = [NSString stringWithFormat:@"%d", count];
        d.blackjack.towerList = 0;
    }
    
    // Drop pins on the map at the tower locations
    if ([d.blackjack.towerLocations count] > [self.towerLocationOverlays count])
    {
        [self.mapView removeAnnotations:self.towerLocationOverlays];
        
        NSNumber *lon;
        for (NSNumber *lat in [d.blackjack.towerLocations allKeys])
        {
            lon = [d.blackjack.towerLocations objectForKey:lat];
            CLLocationCoordinate2D towerLoc = CLLocationCoordinate2DMake([lat floatValue], [lon floatValue]);
            [self markADSBTowerLocationAtPoint:towerLoc];
        }
    }
}

- (void)updateUIFromNewNEXRADWeatherData
{
    if (self.newNEXRADReceived == YES)
    {
        self.newNEXRADReceived = NO;
    }
    else
    {
        // stop the timer
        if (self.nexradProcessingDelayTimer != nil)
        {
            if ([self.nexradProcessingDelayTimer isValid])
            {
                [self.nexradProcessingDelayTimer invalidate];
                self.nexradProcessingDelayTimer = nil;
            }
        }
        self.nexradProcessingDelayTimerStarted = NO;
        
        // generate the updated weather image
        [self generateCurrentNEXRADWeatherImage];
    }
}

- (void)processIncomingNEXRADAfterDelay
{
    /* NEXRAD data is broadcast every 2.5 to 3 minutes and comes through across several broadcast sentences.
     It can also be intermingled with other textual data. So there is really no way to know when the incoming
     broadcast of weather is "completed".
     
     In order to avoid the CPU load incurred by generating a new weather image after sentence is received, you
     can use a timer to defer processing until a few seconds after the last message is received. That's what is
     happening here.
     
     A repeating timer is started. When the timer ends, there is a check whether new data has arrived. If so, the
     timer is allowed to repeat. If no new data has been received, the timer is cancelled and the received data
     is processed.
    */
    
	if (self.nexradProcessingDelayTimerStarted == NO)
	{
		//create timer
		self.nexradProcessingDelayTimer = [NSTimer timerWithTimeInterval:(kNEXRADProcessingDelayTime)
                                                                  target:self
                                                                selector:@selector(updateUIFromNewNEXRADWeatherData)
                                                                userInfo:nil
                                                                 repeats:YES];
		
		[[NSRunLoop currentRunLoop] addTimer:self.nexradProcessingDelayTimer forMode:NSDefaultRunLoopMode];
		self.nexradProcessingDelayTimerStarted = YES;
	}
	else return;
}

- (void)updateUIFromNewCONUSWeatherData
{
    if (self.newCONUSReceived == YES)
    {
        self.newCONUSReceived = NO;
    }
    else
    {
        // generate the updated weather image
        [self generateCurrentCONUSWeatherImage];

        // stop the timer
        if (self.conusProcessingDelayTimer != nil)
        {
            if ([self.conusProcessingDelayTimer isValid])
            {
                [self.conusProcessingDelayTimer invalidate];
                self.conusProcessingDelayTimer = nil;
            }
        }
        self.conusProcessingDelayTimerStarted = NO;
    }
}

- (void)processIncomingCONUSAfterDelay
{
	if (self.conusProcessingDelayTimerStarted == NO)
	{
		//create timer
		self.conusProcessingDelayTimer = [NSTimer timerWithTimeInterval:(kCONUSProcessingDelayTime)
                                                                  target:self
                                                                selector:@selector(updateUIFromNewCONUSWeatherData)
                                                                userInfo:nil
                                                                 repeats:YES];
		
		[[NSRunLoop currentRunLoop] addTimer:self.conusProcessingDelayTimer forMode:NSDefaultRunLoopMode];
		self.conusProcessingDelayTimerStarted = YES;
	}
	else return;
}

- (void)updateTextInScrollView
{
    // keep the scroll text from getting too long
    if ([self.scrollViewText length] > kScrollViewCharacterLimit)
    {
        [self.scrollViewText deleteCharactersInRange:NSMakeRange(0, (kScrollViewCharacterLimit * 0.1))];
    }
    
    // update the text view
    self.receivedMsgsTextView.attributedText = self.scrollViewText;
    
    // make sure the scroller is at the end
    //[self.receivedMsgsTextView scrollRangeToVisible:NSMakeRange([self.scrollViewText length]-1, 1)];
}

- (void)displayReceivedTextualProducts:(NSMutableDictionary *)dictionaryOfItems withType:(uint32_t)type
{
    NSString *textToAdd;
    UIColor *lineColor = [UIColor cyanColor];
    
    for (NSNumber *n in [dictionaryOfItems allKeys])
    {
        textToAdd = [NSString stringWithFormat:@"%@\n", [dictionaryOfItems objectForKey:n]];

        if (type == kNOTAM) lineColor = [UIColor redColor];
        else if (type == kMETAR) lineColor = [UIColor orangeColor];
        else if (type == kTAF) lineColor = [UIColor yellowColor];
        else if (type == kDATIS) lineColor = [UIColor greenColor];
        else if (type == kAIRMET) lineColor = [UIColor blueColor];
        else if (type == kSIGMET) lineColor = [UIColor purpleColor];
        else if (type == kPIREP) lineColor = [UIColor magentaColor];
        else if (type == kWINDS) lineColor = [UIColor whiteColor];
        else if (type == kSUAUPDATE) lineColor = [UIColor lightGrayColor];
        
        // create the attriuted string to display
        NSDictionary *attrsDictionary = [NSDictionary dictionaryWithObjectsAndKeys: lineColor, NSForegroundColorAttributeName,
                                         [UIFont fontWithName:@"HelveticaNeue-Bold" size:16.0], NSFontAttributeName, nil];
        NSAttributedString *attrString = [[NSAttributedString alloc] initWithString:textToAdd attributes:attrsDictionary];
        [self.scrollViewText appendAttributedString:attrString];
    }
    
    // update the text view
    [self updateTextInScrollView];
}

- (void)displayShortTrafficReport:(ShortTrafficObjectInfo *)stoi
{
    NSString *textToAdd;
    UIColor *lineColor = [UIColor greenColor];
    
    textToAdd = [NSString stringWithFormat:@"<SHORT> Traffic at %.5f/%.5f at %d feet. Heading %.0f˚ at %d kts. Tail # is %s.\n\n",
                 stoi->latitude, stoi->longitude, stoi->altitude, stoi->trackHeading, stoi->horizVelocity, stoi->callsign];
    
    // create the attriuted string to display
    NSDictionary *attrsDictionary = [NSDictionary dictionaryWithObjectsAndKeys: lineColor, NSForegroundColorAttributeName,
                                     [UIFont fontWithName:@"HelveticaNeue-BoldItalic" size:16.0], NSFontAttributeName, nil];
    NSAttributedString *attrString = [[NSAttributedString alloc] initWithString:textToAdd attributes:attrsDictionary];
    [self.scrollViewText appendAttributedString:attrString];
    
    // update the text view
    [self updateTextInScrollView];
}

- (void)displayBasicAndLongTrafficReport:(LongTrafficObjectInfo *)ltoi
{
    NSString *textToAdd, *title, *source, *position, *movement, *heading, *aircraftType, *tail;
    UIColor *lineColor = [UIColor greenColor];
    float velocity=0;
    
    if (ltoi->adsbTarget) source = @"Data coming directly from other aircraft.";
    else if (ltoi->tisbTarget)
    {
        if (ltoi->adsrTarget) source = @"Data is either TIS-B or ADS-rebroadcast from a ground tower.";
        else source = @"Data is TIS-B broadcast from a ground tower.";
    }
    else if (ltoi->adsrTarget) source = @"Data is coming from ADS-rebroadcast from a ground tower.";
    else if (ltoi->surfaceVehicle) source = @"Data is coming from a ground vehicle.";
    else if (ltoi->adsbBeacon) source = @"Data is coming from a ground beacon.";
    else source = @"Data source unknown.";
    
    if (ltoi->isBasicMessage)
    {
        title = @"<BASIC>";
        aircraftType = @"";
        tail = @"";
    }
    else
    {
        title = @"<LONG>";
        
        if (ltoi->emitterCategoryDescr[0] != '\0') aircraftType = [NSString stringWithFormat:@"Type is %s.", ltoi->emitterCategoryDescr];
        else aircraftType = @"";
        
        if (ltoi->callSignFlightPlanID[0] != '\0') tail = [NSString stringWithFormat:@"Tail # is %s.", ltoi->callSignFlightPlanID];
        else tail = @"";
    }
    
    position = [NSString stringWithFormat:@"Traffic at %.5f/%.5f", ltoi->latitude, ltoi->longitude];
    
    if (ltoi->airborne)
    {
        if ((ltoi->northerlyVelocityAvail) && (ltoi->easterlyVelocityAvail))
        {
            velocity = sqrtf((ltoi->northerlyVelocity ^ 2) + (ltoi->easterlyVelocity ^ 2));
            
            if (ltoi->verticalVelocityRate != 0) movement = [NSString stringWithFormat:@"at %d' and %@ at %d FPM.", ltoi->altitude,
                                                                 (ltoi->verticalVelocityDirection)?@"descending":@"ascending", ltoi->verticalVelocityRate];
            else movement = [NSString stringWithFormat:@"level at %d.", ltoi->altitude];
        }
        else
        {
            movement = [NSString stringWithFormat:@"at %d'. Speed N/A.", ltoi->altitude];
        }
    }
    else
    {
        if (ltoi->groundSpeedAvail)
        {
            if (ltoi->groundSpeed != 0) movement = [NSString stringWithFormat:@"on the ground moving at %d kts.", ltoi->groundSpeed];
            else movement = @"stationary on the ground.";
        }
        else
        {
            movement = @"on the ground. Speed N/A.";
        }
    }
    
    if (ltoi->heading == -1) heading = @"Heading N/A.";
    else heading = [NSString stringWithFormat:@"Heading %.0f˚.", ltoi->heading];
    
    textToAdd = [NSString stringWithFormat:@"%@ %@ %@ %@ %@ %@\n%@\n\n", title, position, movement, heading, aircraftType, tail, source];
    
    // create the attriuted string to display
    NSDictionary *attrsDictionary = [NSDictionary dictionaryWithObjectsAndKeys: lineColor, NSForegroundColorAttributeName,
                                     [UIFont fontWithName:@"HelveticaNeue-BoldItalic" size:16.0], NSFontAttributeName, nil];
    NSAttributedString *attrString = [[NSAttributedString alloc] initWithString:textToAdd attributes:attrsDictionary];
    [self.scrollViewText appendAttributedString:attrString];
    
    // update the text view
    [self updateTextInScrollView];
}

#pragma mark - UISwitch action handlers
- (IBAction)nexradReceivedDataSwitchChanged:(id)sender
{
    if (self.nexradReceivedDataSwitch.on == YES) [self.mapView addOverlay:self.nexradOverlay];
    else [self.mapView removeOverlay:self.nexradOverlay];
}

- (IBAction)conusReceivedDataSwitchChanged:(id)sender
{
    if (self.conusReceivedDataSwitch.on == YES) [self.mapView addOverlay:self.conusOverlay];
    else [self.mapView removeOverlay:self.conusOverlay];
}

- (IBAction)nexradTestDataSwitchChanged:(id)sender
{
    if (self.nexradTestDataSwitch.on == YES)
    {
        [self generateTestPatterns];
        [self.mapView addOverlay:self.nexradTestWeatherDataOverlay];
    }
    else [self.mapView removeOverlay:self.nexradTestWeatherDataOverlay];
}

- (IBAction)conusTestDataSwitchChanged:(id)sender
{
    if (self.conusTestDataSwitch.on == YES)
    {
        [self generateTestPatterns];
        [self.mapView addOverlay:self.conusTestWeatherDataOverlay];
    }
    else [self.mapView removeOverlay:self.conusTestWeatherDataOverlay];
}

- (IBAction)notConMsgSwitchChanged:(id)sender
{
    create_delegate_instance_d
    
    if (d.blackjack.isConnected == NO)
    {
        if (self.notConMessageSwitch.on == NO) self.blackjackNotConnectedMessage.hidden = YES;
        else self.blackjackNotConnectedMessage.hidden = NO;;
    }
}

- (IBAction)outlineNexradRegionSwitchChanged:(id)sender
{
    [self synthesizeReceivedRegionOverlaysUsingCONUS:NO];
}

- (IBAction)outlineConusRegionSwitchChanged:(id)sender
{
    [self synthesizeReceivedRegionOverlaysUsingCONUS:YES];
}

- (IBAction)dataSourceSwitchChanged:(id)sender
{
    create_delegate_instance_d
    
    // clear the NEXRAD and CONUS memory before changing streams
    [d.blackjack clearNEXRADData];
    [d.blackjack clearCONUSData];
    [self.mapView removeOverlay:self.nexradTestWeatherDataOverlay];
    [self.mapView removeOverlay:self.conusTestWeatherDataOverlay];
    
    if (self.dataSourceSwitch.on) d.blackjack.appWantsADSBData = YES;
    else
    {
        d.blackjack.appWantsADSBData = NO;
        [d.blackjack runTestData];
    }
}

- (IBAction)settingsButtonTouched:(id)sender
{
    //NSLog(@"%s. Settings button tapped.", __FUNCTION__);
    SettingsPopoverVC *spvc = [[SettingsPopoverVC alloc] initWithNibName:@"SettingsPopover" bundle:[NSBundle mainBundle]];
    [self setupNewPopoverControllerForViewController:spvc];
    self.currentPopover.popoverContentSize = spvc.view.frame.size;
    [self.currentPopover presentPopoverFromBarButtonItem:sender permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
}

#pragma mark - Weather View Methods
- (void)synthesizeReceivedRegionOverlaysUsingCONUS:(BOOL)useCONUS
{
    // Overlay images cannot have their region change once created. Since the weather data can occupy an ever-changing
    // region, any existing overlay image must be destroyed before creating a new one.
    
    if (useCONUS)
    {
        if (self.outlineConusRegionSwitch.on == YES) [self outlineCONUSRegions];
        else
        {
            [self.mapView removeOverlays:self.arrayOfCONUSBlockOutlineOverlays];
            [self.arrayOfCONUSBlockOutlineOverlays removeAllObjects];
        }
    }
    else
    {
        if (self.outlineNexradRegionSwitch.on == YES) [self outlineNexradRegions];
        else
        {
            [self.mapView removeOverlays:self.arrayOfNEXRADBlockOutlineOverlays];
            [self.arrayOfNEXRADBlockOutlineOverlays removeAllObjects];
        }
    }
}

- (void)synthesizeWeatherImageOverlayUsingCONUS:(BOOL)useCONUS
{
    create_delegate_instance_d
    
    // Overlay images cannot have their region change once created. Since the weather data can occupy an ever-changing
    // region, any existing overlay image must be destroyed before creating a new one.
    
    if (useCONUS)
    {
        self.conusOverlay = nil;
        self.conusOverlay = [[CONUSWeatherOverlay alloc] initWithRegion:d.blackjack.conusWeatherCoverageArea];
        if (self.conusReceivedDataSwitch.on == YES) [self.mapView addOverlay:self.conusOverlay];
    }
    else
    {
        self.nexradOverlay = nil;
        self.nexradOverlay = [[NEXRADWeatherOverlay alloc] initWithRegion:d.blackjack.nexradWeatherCoverageArea];
        if (self.nexradReceivedDataSwitch.on == YES) [self.mapView addOverlay:self.nexradOverlay];
    }
}

- (void)synthesizeTestWeatherDataImageOverlayUsingCONUS:(BOOL)useCONUS
{
    create_delegate_instance_d
    
    // Overlay images cannot have their region change once created. Since the weather data can occupy an ever-changing
    // region, any existing overlay image must be destroyed before creating a new one.
    
    if (useCONUS)
    {
        self.conusTestWeatherDataOverlay = nil;
        self.conusTestWeatherDataOverlay = [[CONUSTestWeatherDataOverlay alloc] initWithRegion:d.blackjack.conusWeatherCoverageArea];
        if (self.conusTestDataSwitch.on == YES) [self.mapView addOverlay:self.conusTestWeatherDataOverlay];
    }
    else
    {
        self.nexradTestWeatherDataOverlay = nil;
        self.nexradTestWeatherDataOverlay = [[NEXRADTestWeatherDataOverlay alloc] initWithRegion:d.blackjack.nexradWeatherCoverageArea];
        if (self.nexradTestDataSwitch.on == YES) [self.mapView addOverlay:self.nexradTestWeatherDataOverlay];
    }
}

#pragma mark - View Creation
- (void)createTargetPointAnnotationAtPoint:(CLLocationCoordinate2D)coord
{
    TargetPointAnnotation *annotation = [[TargetPointAnnotation alloc] init];
    annotation.coordinate = coord;
    [self.mapView addAnnotation:annotation];
}

- (void)markADSBTowerLocationAtPoint:(CLLocationCoordinate2D)coord
{
    ADSBTowerAnnotation *annotation = [[ADSBTowerAnnotation alloc] init];
    annotation.coordinate = coord;
    annotation.title = @"ADS-B Tower";
    [self.mapView addAnnotation:annotation];
    [self.towerLocationOverlays addObject:annotation];
}

- (void)clearGPSPositionPin
{
    if (self.ownshipMarker)
    {
        [self.mapView removeAnnotation:self.ownshipMarker];
        self.ownshipMarker = nil;
    }
}

- (void)markGPSLocationWithPin
{
    create_delegate_instance_d
    
    // get rid of the old pin
    [self clearGPSPositionPin];

    if (d.blackjack.gpsPositionIsValid == NO) return;
    
    // create new pin in the correct location
    OwnshipPositionAnnotation *annotation = [[OwnshipPositionAnnotation alloc] init];
    annotation.coordinate = CLLocationCoordinate2DMake([d.blackjack.latitude floatValue], [d.blackjack.longitude floatValue]);
    annotation.title = @"You are here";
    [self.mapView addAnnotation:annotation];
    self.ownshipMarker = annotation;
}

- (void)outlineNexradRegions
{
    create_delegate_instance_d
    
    float deltaLat, deltaLon;
    CLLocationCoordinate2D pointsToUse[4];
    
    NSArray *blocks = [d.blackjack.fisbBlocks allKeys];
    
    for (NSNumber *n in blocks)
    {
        uint32_t blockNumber = [n unsignedLongValue];
        
        if (blockNumber < 405000)
        {
            deltaLat = 4.0 / 60.0;
            deltaLon = 32.0 * 1.5 / 60.0;
        }
        else
        {
            deltaLat = 4.0 / 60.0;
            deltaLon = 32.0 * 3.0 / 60.0;
        }
        
        pointsToUse[0] = [d.blackjack convertBlockNumberToLatLon:blockNumber];
        pointsToUse[1] = CLLocationCoordinate2DMake(pointsToUse[0].latitude + deltaLat, pointsToUse[0].longitude);
        pointsToUse[2] = CLLocationCoordinate2DMake(pointsToUse[0].latitude + deltaLat, pointsToUse[0].longitude + deltaLon);
        pointsToUse[3] = CLLocationCoordinate2DMake(pointsToUse[0].latitude, pointsToUse[0].longitude + deltaLon);
        
        ReceivedNEXRADRegionPolygon *region = [ReceivedNEXRADRegionPolygon polygonWithCoordinates:pointsToUse count:4];
        
        [self.arrayOfNEXRADBlockOutlineOverlays addObject:region];    // keep a pointer to the overlay so that it can be removed later
    }
    
    [self.mapView addOverlays:self.arrayOfNEXRADBlockOutlineOverlays];
} // outlineNexradRegions

- (void)outlineCONUSRegions
{
    create_delegate_instance_d
    
    float deltaLat, deltaLon;
    CLLocationCoordinate2D pointsToUse[4];
    
    NSArray *blocks = [d.blackjack.fisbCONUSBlocks allKeys];
    
    for (NSNumber *n in blocks)
    {
        uint32_t blockNumber = [n unsignedLongValue];
        //NSLog(@"outlining CONUS block #: %d", blockNumber);
        
        if (blockNumber < 405000)
        {
            deltaLat = 4.0 / 60.0;
            deltaLon = 32.0 * 1.5 / 60.0;
        }
        else
        {
            deltaLat = 4.0 / 60.0;
            deltaLon = 32.0 * 3.0 / 60.0;
        }
        
        pointsToUse[0] = [d.blackjack convertBlockNumberToLatLon:blockNumber];
        pointsToUse[1] = CLLocationCoordinate2DMake(pointsToUse[0].latitude + deltaLat, pointsToUse[0].longitude);
        pointsToUse[2] = CLLocationCoordinate2DMake(pointsToUse[0].latitude + deltaLat, pointsToUse[0].longitude + deltaLon);
        pointsToUse[3] = CLLocationCoordinate2DMake(pointsToUse[0].latitude, pointsToUse[0].longitude + deltaLon);
        
        ReceivedCONUSRegionPolygon *region = [ReceivedCONUSRegionPolygon polygonWithCoordinates:pointsToUse count:4];
        
        [self.arrayOfCONUSBlockOutlineOverlays addObject:region];    // keep a pointer to the overlay so that it can be removed later
    }
    
    [self.mapView addOverlays:self.arrayOfCONUSBlockOutlineOverlays];
}  // outlineCONUSRegions

- (void)generateCurrentNEXRADWeatherImage
{
    create_delegate_instance_d

    dispatch_async(dispatch_get_main_queue(), ^{
        [d.blackjack calculateNEXRADDataCoverageAreaUsingCONUS:NO];
        self.createdNEXRADImage = [d.blackjack createRegionalWeatherImage];
        [self synthesizeWeatherImageOverlayUsingCONUS:NO];
    
        /*
        //////////DEBUG: drop annotation points at the corners of the radar data region
        MKCoordinateRegion region = d.blackjack.nexradWeatherCoverageArea;
        double upperLeftLat = region.center.latitude + (0.5 * region.span.latitudeDelta);
        double upperLeftLon = region.center.longitude - (0.5 * region.span.longitudeDelta);

        double upperRightLat = upperLeftLat;
        double upperRightLon = region.center.longitude + (0.5 * region.span.longitudeDelta);

        double lowerLeftLat = region.center.latitude - (0.5 * region.span.latitudeDelta);
        double lowerLeftLon = upperLeftLon;

        double lowerRightLat = lowerLeftLat;
        double lowerRightLon = upperRightLon;

        NSLog(@"%s",__FUNCTION__);
        NSLog(@"--> upper left lat/lon = %.6f/%.6f", upperLeftLat, upperLeftLon);
        NSLog(@"--> upper right lat/lon = %.6f/%.6f", upperRightLat, upperRightLon);
        NSLog(@"--> lower left lat/lon = %.6f/%.6f", lowerLeftLat, lowerLeftLon);
        NSLog(@"--> lower right lat/lon = %.6f/%.6f", lowerRightLat, lowerRightLon);

        CLLocationCoordinate2D overlayTopLeftCoordinate = CLLocationCoordinate2DMake(upperLeftLat, upperLeftLon);
        CLLocationCoordinate2D overlayTopRightCoordinate = CLLocationCoordinate2DMake(upperRightLat, upperRightLon);
        CLLocationCoordinate2D overlayBottomLeftCoordinate = CLLocationCoordinate2DMake(lowerLeftLat, lowerLeftLon);
        CLLocationCoordinate2D overlayBottomRightCoordinate = CLLocationCoordinate2DMake(lowerRightLat, lowerRightLon);

        [self createTargetPointAnnotationAtPoint:overlayTopLeftCoordinate];
        [self createTargetPointAnnotationAtPoint:overlayTopRightCoordinate];
        [self createTargetPointAnnotationAtPoint:overlayBottomLeftCoordinate];
        [self createTargetPointAnnotationAtPoint:overlayBottomRightCoordinate];
        /////////////////////////////////////////////
        */
    });

} // generateCurrentNEXRADWeatherImage

- (void)generateCurrentCONUSWeatherImage
{
    create_delegate_instance_d

    dispatch_async(dispatch_get_main_queue(), ^{
        [d.blackjack calculateNEXRADDataCoverageAreaUsingCONUS:YES];
        self.createdCONUSImage = [d.blackjack createNationalWeatherImage];
        [self synthesizeWeatherImageOverlayUsingCONUS:YES];
        
        /*
        //////////DEBUG: drop annotation points at the corners of the radar data region
        MKCoordinateRegion region = d.blackjack.conusWeatherCoverageArea;
        double upperLeftLat = region.center.latitude + (0.5 * region.span.latitudeDelta);
        double upperLeftLon = region.center.longitude - (0.5 * region.span.longitudeDelta);
        
        double upperRightLat = upperLeftLat;
        double upperRightLon = region.center.longitude + (0.5 * region.span.longitudeDelta);
        
        double lowerLeftLat = region.center.latitude - (0.5 * region.span.latitudeDelta);
        double lowerLeftLon = upperLeftLon;
        
        double lowerRightLat = lowerLeftLat;
        double lowerRightLon = upperRightLon;
        
        CLLocationCoordinate2D overlayTopLeftCoordinate = CLLocationCoordinate2DMake(upperLeftLat, upperLeftLon);
        CLLocationCoordinate2D overlayTopRightCoordinate = CLLocationCoordinate2DMake(upperRightLat, upperRightLon);
        CLLocationCoordinate2D overlayBottomLeftCoordinate = CLLocationCoordinate2DMake(lowerLeftLat, lowerLeftLon);
        CLLocationCoordinate2D overlayBottomRightCoordinate = CLLocationCoordinate2DMake(lowerRightLat, lowerRightLon);
        
        [self createTargetPointAnnotationAtPoint:overlayTopLeftCoordinate];
        [self createTargetPointAnnotationAtPoint:overlayTopRightCoordinate];
        [self createTargetPointAnnotationAtPoint:overlayBottomLeftCoordinate];
        [self createTargetPointAnnotationAtPoint:overlayBottomRightCoordinate];
        /////////////////////////////////////////////
        */
    });
    
} // generateCurrentCONUSWeatherImage

- (void)generateWeatherImages
{
    // [self generateTestPatterns];  // Uncomment here to generate test patterns on startup
    [self generateCurrentNEXRADWeatherImage];
    [self generateCurrentCONUSWeatherImage];
}

- (void)setupWeatherView
{
    create_delegate_instance_d
 
    CLLocationCoordinate2D zoomLocation;

    // Set up the weather radar view
    if ([d.blackjack.latitude intValue] == 0)
    {
        zoomLocation.latitude  = kTestDataOriginLatitude - (0.5 * kTestDataNEXRADRegionSize);
        zoomLocation.longitude = kTestDataOriginLongitude + (0.5 * kTestDataNEXRADRegionSize);
        self.mapView.centerCoordinate = CLLocationCoordinate2DMake(kTestDataOriginLatitude, kTestDataOriginLongitude);
    }
    else
    {
        zoomLocation.latitude  = [d.blackjack.latitude floatValue] - (0.5 * kTestDataNEXRADRegionSize);
        zoomLocation.longitude = [d.blackjack.longitude floatValue] + (0.5 * kTestDataNEXRADRegionSize);
        self.mapView.centerCoordinate = CLLocationCoordinate2DMake([d.blackjack.latitude floatValue], [d.blackjack.longitude floatValue]);
    }

    self.NEXRADTestPatternCreated = NO;
    self.CONUSTestPatternCreated = NO;
    
    MKCoordinateRegion viewRegion = MKCoordinateRegionMakeWithDistance(zoomLocation, kZoomDistance, kZoomDistance);
    [self.mapView setRegion:viewRegion animated:YES];
    
    // The view controller initializes faster than the blackjack object. So throw in a delay just to give the
    // blackjack object time to complete initialization
    [self performSelector:@selector(generateWeatherImages) withObject:nil afterDelay:1.5];
}

- (void)setLayout:(UIInterfaceOrientation)orientation
{
    float commonLabelHeight = 21.0;
    float commonLabelWidth = 150.0;
    float commonSection1LabelSpacing;
    float refX, refY, groupLineSpacing, interColSpacing;
    float section1LeftMargin;
    float section1StartY;
    
    float dualLogoImageX, dualLogoImageY, dualLogoImageWidth, dualLogoImageHeight;
    float dualLogoImageLabelX, dualLogoImageLabelY, dualLogoImageLabelWidth;
    
    float deviceInformationLabelX, deviceInformationLabelY, deviceInformationLabelWidth;
    float statusLabelX, statusLabelY, statusLabelWidth;
    float chargingLabelX, chargingLabelY, chargingLabelWidth;
    float firmwareVersionLabelX, firmwareVersionLabelY, firmwareVersionLabelWidth;
    float statusValueX, statusValueY, statusValueWidth;
    float chargingValueX, chargingValueY, chargingValueWidth;
    float firmwareVersionValueX, firmwareVersionValueY, firmwareVersionValueWidth;
    float batLevelBargraphX, batLevelBargraphY, batLevelBargraphWidth, batLevelBargraphHeight;
    
    float adsbServiceLabelX, adsbServiceLabelY, adsbServiceLabelWidth;
    float receivingDataLabelX, receivingDataLabelY, receivingDataLabelWidth;
    float receivingDataValueX, receivingDataValueY, receivingDataValueWidth;
    float nexradLabelX, nexradLabelY, nexradLabelWidth;
    float conusLabelX, conusLabelY, conusLabelWidth;
    float trafficLabelX, trafficLabelY, trafficLabelWidth;
    float notamsLabelX, notamsLabelY, notamsLabelWidth;
    float metarsLabelX, metarsLabelY, metarsLabelWidth;
    float tafsLabelX, tafsLabelY, tafsLabelWidth;
    float datisLabelX, datisLabelY, datisLabelWidth;
    float airmetLabelX, airmetLabelY, airmetLabelWidth;
    float sigmetLabelX, sigmetLabelY, sigmetLabelWidth;
    float pirepsLabelX, pirepsLabelY, pirepsLabelWidth;
    float windsLabelX, windsLabelY, windsLabelWidth;
    float airspaceUpdateLabelX, airspaceUpdateLabelY, airspaceUpdateLabelWidth;
    float towerCountLabelX, towerCountLabelY, towerCountLabelWidth;
    float towerCountValueX, towerCountValueY, towerCountValueWidth;
    float messageCountLabelX, messageCountLabelY, messageCountLabelWidth;
    float messageCountValueX, messageCountValueY, messageCountValueWidth;
    float nexradAgeLabelX, nexradAgeLabelY, nexradAgeLabelWidth;
    float nexradAgeValueX, nexradAgeValueY, nexradAgeValueWidth;
    float conusNexradAgeLabelX, conusNexradAgeLabelY, conusNexradAgeLabelWidth;
    float conusNexradAgeValueX, conusNexradAgeValueY, conusNexradAgeValueWidth;
    
    float positionInfoLabelX, positionInfoLabelY, positionInfoLabelWidth;
    float localGroupLabelWidth, localGroupValueWidth;
    float latitudeLabelX, latitudeLabelY, latitudeLabelWidth;
    float longitudeLabelX, longitudeLabelY, longitudeLabelWidth;
    float altitudeLabelX, altitudeLabelY, altitudeLabelWidth;
    float headingLabelX, headingLabelY, headingLabelWidth;
    float speedLabelX, speedLabelY, speedLabelWidth;
    float latitudeValueX, latitudeValueY, latitudeValueWidth;
    float longitudeValueX, longitudeValueY, longitudeValueWidth;
    float altitudeValueX, altitudeValueY, altitudeValueWidth;
    float headingValueX, headingValueY, headingValueWidth;
    float speedValueX, speedValueY, speedValueWidth;
    float waasInUseLabelX, waasInUseLabelY, waasInUseLabelWidth;
    float waasInUseValueX, waasInUseValueY, waasInUseValueWidth;
    
    float blackjackNotConnectedMsgX, blackjackNotConnectedMsgY, blackjackNotConnectedMsgWidth, blackjackNotConnectedMsgHeight;
        
    //////////////////////////////////////////////////////////////////
    //
    // set the layout for portrait ortientations here
    //
    //////////////////////////////////////////////////////////////////
    
    // ----> SECTION 1 START
    section1LeftMargin = 0;
    section1StartY = 35.0;
    commonSection1LabelSpacing = 23.0;
    
    // red dual logo and associated label text
    dualLogoImageX = 165.0;
    dualLogoImageY = 35.5;
    dualLogoImageWidth = 95.0; //46.0;
    dualLogoImageHeight = 90.0; // 44.0;
    dualLogoImageLabelX = 230.0;   // for center
    dualLogoImageLabelY = 35.0; //73.0;    // for center
    dualLogoImageLabelWidth = 320.0;
    
    // device info group
    refX = 0;
    refY = 102.0; //173.0;
    interColSpacing = 6.0;
    groupLineSpacing = 23.0;
    commonLabelWidth = 120.0;
    deviceInformationLabelX = refX + 40.0;
    deviceInformationLabelY = refY - 23.0;
    deviceInformationLabelWidth = 220.0;
    statusLabelX = refX;
    statusLabelY = refY;
    statusLabelWidth = commonLabelWidth;
    firmwareVersionLabelX = refX;
    firmwareVersionLabelY = refY + groupLineSpacing;
    firmwareVersionLabelWidth = commonLabelWidth;
    chargingLabelX = refX;
    chargingLabelY = refY + (2.0*groupLineSpacing);
    chargingLabelWidth = commonLabelWidth;
    statusValueX = refX + statusLabelWidth + interColSpacing;
    statusValueY = refY;
    statusValueWidth = 193.0;
    firmwareVersionValueX = refX + firmwareVersionLabelWidth + interColSpacing;
    firmwareVersionValueY = refY + groupLineSpacing;
    firmwareVersionValueWidth = 120.0;
    chargingValueX = refX + chargingLabelWidth + interColSpacing;
    chargingValueY = refY + (2.0*groupLineSpacing);
    chargingValueWidth = 193.0;
    batLevelBargraphX = refX + chargingLabelWidth + interColSpacing;
    batLevelBargraphY = refY + (2.0*groupLineSpacing) + 4.0;
    batLevelBargraphWidth = 180.0;
    batLevelBargraphHeight = 50.0;
    
    // ads-b information group
    refX = 20;
    refY += 130.0; // = 310.0;
    commonLabelWidth = 150.0;
    interColSpacing = 6.0;
    groupLineSpacing = 28.0;
    localGroupLabelWidth = commonLabelWidth;
    localGroupValueWidth = 270.0;
    adsbServiceLabelX = refX + 20.0;
    adsbServiceLabelY = refY - 23.0; // 291
    adsbServiceLabelWidth = 220.0;
    receivingDataLabelX = refX;
    receivingDataLabelY = refY;
    receivingDataLabelWidth = commonLabelWidth;
    receivingDataValueX = refX + receivingDataLabelWidth + interColSpacing;
    receivingDataValueY = refY;
    receivingDataValueWidth = commonLabelWidth;
    nexradLabelX = refX;
    nexradLabelY = refY + groupLineSpacing;
    nexradLabelWidth = commonLabelWidth;
    conusLabelX = refX;
    conusLabelY = refY + (2.0*groupLineSpacing);
    conusLabelWidth = commonLabelWidth;
    trafficLabelX = refX;
    trafficLabelY = refY + (3.0*groupLineSpacing);
    trafficLabelWidth = commonLabelWidth;
    notamsLabelX = refX;
    notamsLabelY = refY + (4.0*groupLineSpacing);
    notamsLabelWidth = commonLabelWidth;
    metarsLabelX = refX;
    metarsLabelY = refY + (5.0*groupLineSpacing);
    metarsLabelWidth = commonLabelWidth;
    tafsLabelX = refX;
    tafsLabelY = refY + (6.0*groupLineSpacing);
    tafsLabelWidth = commonLabelWidth;
    datisLabelX = refX;
    datisLabelY = refY + (7.0*groupLineSpacing);
    datisLabelWidth = commonLabelWidth;
    airmetLabelX = refX;
    airmetLabelY = refY + (8.0*groupLineSpacing);
    airmetLabelWidth = commonLabelWidth;
    sigmetLabelX = refX;
    sigmetLabelY = refY + (9.0*groupLineSpacing);
    sigmetLabelWidth = commonLabelWidth;
    pirepsLabelX = refX;
    pirepsLabelY = refY + (10.0*groupLineSpacing);
    pirepsLabelWidth = commonLabelWidth;
    windsLabelX = refX;
    windsLabelY = refY + (11.0*groupLineSpacing);
    windsLabelWidth = commonLabelWidth;
    airspaceUpdateLabelX = refX;
    airspaceUpdateLabelY = refY + (12.0*groupLineSpacing);
    airspaceUpdateLabelWidth = commonLabelWidth;
    towerCountLabelX = refX;
    towerCountLabelY = refY + (13.0*groupLineSpacing);
    towerCountLabelWidth = commonLabelWidth;
    towerCountValueX = towerCountLabelX + towerCountLabelWidth + interColSpacing;
    towerCountValueY = towerCountLabelY;
    towerCountValueWidth = 135.0;
    messageCountLabelX = refX;
    messageCountLabelY = refY + (14.0*groupLineSpacing);
    messageCountLabelWidth = commonLabelWidth;
    messageCountValueX = messageCountLabelX + messageCountLabelWidth + interColSpacing;
    messageCountValueY = messageCountLabelY;
    messageCountValueWidth = 135.0;
    nexradAgeLabelX = refX; // + 260;
    nexradAgeLabelY = refY + (13.0*groupLineSpacing); // refY + 287;
    nexradAgeLabelWidth = commonLabelWidth;
    nexradAgeValueX = nexradAgeLabelX + nexradAgeLabelWidth + interColSpacing;
    nexradAgeValueY = nexradAgeLabelY;
    nexradAgeValueWidth = 135.0;
    conusNexradAgeLabelX = refX; //nexradAgeLabelX + 200;
    conusNexradAgeLabelY = refY + (14.0*groupLineSpacing); //nexradAgeLabelY;
    conusNexradAgeLabelWidth = commonLabelWidth;
    conusNexradAgeValueX = conusNexradAgeLabelX + nexradAgeLabelWidth + interColSpacing;
    conusNexradAgeValueY = conusNexradAgeLabelY;
    conusNexradAgeValueWidth = 135.0;
    
    // position information group
    refX = 0;
    refY += 440.0; // = 670.0;
    interColSpacing = 6.0;
    groupLineSpacing = 23.0;
    localGroupLabelWidth = commonLabelWidth;
    localGroupValueWidth = 150.0;
    positionInfoLabelX = refX + 35.0;
    positionInfoLabelY = refY;
    positionInfoLabelWidth = 220.0;
    latitudeLabelX = refX;
    latitudeLabelY = refY + groupLineSpacing;
    latitudeLabelWidth = localGroupLabelWidth;
    longitudeLabelX = refX;
    longitudeLabelY = refY + (2*groupLineSpacing);
    longitudeLabelWidth = localGroupLabelWidth;
    altitudeLabelX = refX;
    altitudeLabelY = refY + (3*groupLineSpacing);
    altitudeLabelWidth = localGroupLabelWidth;
    headingLabelX = refX;
    headingLabelY = refY + (4*groupLineSpacing);
    headingLabelWidth = localGroupLabelWidth;
    speedLabelX = refX;
    speedLabelY = refY + (5*groupLineSpacing);
    speedLabelWidth = localGroupLabelWidth;
    waasInUseLabelX = refX;
    waasInUseLabelY = refY + (6*groupLineSpacing);
    waasInUseLabelWidth = localGroupLabelWidth;
    latitudeValueX = refX + latitudeLabelWidth + interColSpacing;
    latitudeValueY = refY + groupLineSpacing;
    latitudeValueWidth = localGroupValueWidth;
    longitudeValueX = refX + longitudeLabelWidth + interColSpacing;
    longitudeValueY = refY + (2*groupLineSpacing);
    longitudeValueWidth = localGroupValueWidth;
    altitudeValueX = refX + altitudeLabelWidth + interColSpacing;
    altitudeValueY = refY + (3*groupLineSpacing);
    altitudeValueWidth = localGroupValueWidth;
    headingValueX = refX + headingLabelWidth + interColSpacing;
    headingValueY = refY + (4*groupLineSpacing);
    headingValueWidth = localGroupValueWidth;
    speedValueX = refX + speedLabelWidth + interColSpacing;
    speedValueY = refY + (5*groupLineSpacing);
    speedValueWidth = localGroupValueWidth;
    waasInUseValueX = refX + waasInUseLabelWidth + interColSpacing;
    waasInUseValueY = refY + (6*groupLineSpacing);
    waasInUseValueWidth = localGroupValueWidth;

    // position the "blackjack Not Attached" message
    blackjackNotConnectedMsgWidth = 200.0;
    blackjackNotConnectedMsgHeight = 120.0;
    blackjackNotConnectedMsgX = (768.0 / 2.0) - (blackjackNotConnectedMsgWidth / 2); // 60.0;
    blackjackNotConnectedMsgY = (1024.0 / 2.0) - (blackjackNotConnectedMsgHeight / 2); // 130.0;

    // set frame position of the UI elements
    self.dualLogoImage.frame = CGRectMake(dualLogoImageX, dualLogoImageY, dualLogoImageWidth, dualLogoImageWidth);
    self.dualLogoImageLabel.frame = CGRectMake(dualLogoImageLabelX, dualLogoImageLabelY, dualLogoImageLabelWidth, commonLabelHeight);
    
    self.deviceInformationLabel.frame = CGRectMake(deviceInformationLabelX, deviceInformationLabelY, deviceInformationLabelWidth, commonLabelHeight);
    self.statusLabel.frame = CGRectMake(statusLabelX, statusLabelY, statusLabelWidth, commonLabelHeight);
    self.chargingLabel.frame = CGRectMake(chargingLabelX, chargingLabelY, chargingLabelWidth, commonLabelHeight);
    self.firmwareVersionLabel.frame = CGRectMake(firmwareVersionLabelX, firmwareVersionLabelY, firmwareVersionLabelWidth, commonLabelHeight);
    
    self.statusValue.frame = CGRectMake(statusValueX, statusValueY, statusValueWidth, commonLabelHeight);
    self.chargingValue.frame = CGRectMake(chargingValueX, chargingValueY, chargingValueWidth, commonLabelHeight);
    self.firmwareVersionValue.frame = CGRectMake(firmwareVersionValueX, firmwareVersionValueY,
                                            firmwareVersionValueWidth, commonLabelHeight);
    self.batLevelBargraph.frame = CGRectMake(batLevelBargraphX, batLevelBargraphY, batLevelBargraphWidth, batLevelBargraphHeight);

    self.adsbServiceLabel.frame = CGRectMake(adsbServiceLabelX, adsbServiceLabelY, adsbServiceLabelWidth, commonLabelHeight);
    self.receivingDataLabel.frame = CGRectMake(receivingDataLabelX, receivingDataLabelY, receivingDataLabelWidth, commonLabelHeight);
    self.receivingDataValueReceiving.frame = CGRectMake(receivingDataValueX, receivingDataValueY, receivingDataValueWidth, commonLabelHeight);
    self.receivingDataValueWaiting.frame = CGRectMake(receivingDataValueX, receivingDataValueY, receivingDataValueWidth, commonLabelHeight);
    self.nexradLabel.frame = CGRectMake(nexradLabelX, nexradLabelY, nexradLabelWidth, commonLabelHeight);
    self.conusLabel.frame = CGRectMake(conusLabelX, conusLabelY, conusLabelWidth, commonLabelHeight);
    self.trafficLabel.frame = CGRectMake(trafficLabelX, trafficLabelY, trafficLabelWidth, commonLabelHeight);
    self.notamsLabel.frame = CGRectMake(notamsLabelX, notamsLabelY, notamsLabelWidth, commonLabelHeight);
    self.metarsLabel.frame = CGRectMake(metarsLabelX, metarsLabelY, metarsLabelWidth, commonLabelHeight);
    self.tafsLabel.frame = CGRectMake(tafsLabelX, tafsLabelY, tafsLabelWidth, commonLabelHeight);
    self.datisLabel.frame = CGRectMake(datisLabelX, datisLabelY, datisLabelWidth, commonLabelHeight);
    self.airmetLabel.frame = CGRectMake(airmetLabelX, airmetLabelY, airmetLabelWidth, commonLabelHeight);
    self.sigmetLabel.frame = CGRectMake(sigmetLabelX, sigmetLabelY, sigmetLabelWidth, commonLabelHeight);
    self.pirepsLabel.frame = CGRectMake(pirepsLabelX, pirepsLabelY, pirepsLabelWidth, commonLabelHeight);
    self.windsLabel.frame = CGRectMake(windsLabelX, windsLabelY, windsLabelWidth, commonLabelHeight);
    self.airspaceUpdateLabel.frame = CGRectMake(airspaceUpdateLabelX, airspaceUpdateLabelY, airspaceUpdateLabelWidth, commonLabelHeight);
    self.towerCountLabel.frame = CGRectMake(towerCountLabelX, towerCountLabelY, towerCountLabelWidth, commonLabelHeight);
    self.towerCountValue.frame = CGRectMake(towerCountValueX, towerCountValueY, towerCountValueWidth, commonLabelHeight);
    self.messageCountLabel.frame = CGRectMake(messageCountLabelX, messageCountLabelY, messageCountLabelWidth, commonLabelHeight);
    self.messageCountValue.frame = CGRectMake(messageCountValueX, messageCountValueY, messageCountValueWidth, commonLabelHeight);
    self.nexradAgeLabel.frame = CGRectMake(nexradAgeLabelX, nexradAgeLabelY, nexradAgeLabelWidth, commonLabelHeight);
    self.nexradAgeValue.frame = CGRectMake(nexradAgeValueX, nexradAgeValueY, nexradAgeValueWidth, commonLabelHeight);
    self.conusNexradAgeLabel.frame = CGRectMake(conusNexradAgeLabelX, conusNexradAgeLabelY, conusNexradAgeLabelWidth, commonLabelHeight);
    self.conusNexradAgeValue.frame = CGRectMake(conusNexradAgeValueX, conusNexradAgeValueY, conusNexradAgeValueWidth, commonLabelHeight);
    
    self.positionInfoLabel.frame = CGRectMake(positionInfoLabelX, positionInfoLabelY, positionInfoLabelWidth, commonLabelHeight);
    self.latitudeLabel.frame = CGRectMake(latitudeLabelX, latitudeLabelY, latitudeLabelWidth, commonLabelHeight);
    self.longitudeLabel.frame = CGRectMake(longitudeLabelX, longitudeLabelY, longitudeLabelWidth, commonLabelHeight);
    self.altitudeLabel.frame = CGRectMake(altitudeLabelX, altitudeLabelY, altitudeLabelWidth, commonLabelHeight);
    self.speedLabel.frame = CGRectMake(speedLabelX, speedLabelY, speedLabelWidth, commonLabelHeight);
    self.headingLabel.frame = CGRectMake(headingLabelX, headingLabelY, headingLabelWidth, commonLabelHeight);
    self.latitudeValue.frame = CGRectMake(latitudeValueX, latitudeValueY, latitudeValueWidth, commonLabelHeight);
    self.longitudeValue.frame = CGRectMake(longitudeValueX, longitudeValueY, longitudeValueWidth, commonLabelHeight);
    self.altitudeValue.frame = CGRectMake(altitudeValueX, altitudeValueY, altitudeValueWidth, commonLabelHeight);
    self.speedValue.frame = CGRectMake(speedValueX, speedValueY, speedValueWidth, commonLabelHeight);
    self.headingValue.frame = CGRectMake(headingValueX, headingValueY, headingValueWidth, commonLabelHeight);
    self.waasInUseLabel.frame = CGRectMake(waasInUseLabelX, waasInUseLabelY, waasInUseLabelWidth, commonLabelHeight);
    self.waasInUseValue.frame = CGRectMake(waasInUseValueX, waasInUseValueY, waasInUseValueWidth, commonLabelHeight);
    
    self.blackjackNotConnectedMessage.frame = CGRectMake(blackjackNotConnectedMsgX, blackjackNotConnectedMsgY, blackjackNotConnectedMsgWidth, blackjackNotConnectedMsgHeight);

}

#pragma mark -
#pragma mark Popover Controller Methods
- (void)handleDismissedPopoverController:(UIPopoverController *)popoverController
{
    popoverController = nil;
}

- (void)popoverControllerDidDismissPopover:(UIPopoverController *)popoverController
{
    [self handleDismissedPopoverController:popoverController];
}

- (void)setupNewPopoverControllerForViewController:(UIViewController *)vc
{
    if (self.currentPopover)
    {
        [self.currentPopover dismissPopoverAnimated:YES];
        [self handleDismissedPopoverController:self.currentPopover];
    }
    self.currentPopover = [[UIPopoverController alloc] initWithContentViewController:vc];
    self.currentPopover.delegate = self;
}

#pragma mark - Application lifecycle
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self)
    {
        self.batVoltagesArray = [[NSMutableArray alloc] initWithCapacity:kNumberOfBatteryVoltageSamplesToAverage];
        self.arrayOfNEXRADBlockOutlineOverlays = [[NSMutableArray alloc] init];
        self.arrayOfCONUSBlockOutlineOverlays = [[NSMutableArray alloc] init];
        self.towerLocationOverlays = [[NSMutableArray alloc] init];
        self.scrollViewText = [[NSMutableAttributedString alloc] initWithString:@""];
        self.newNEXRADReceived = NO;
        self.newCONUSReceived = NO;
        self.nexradProcessingDelayTimerStarted = NO;
        self.conusProcessingDelayTimerStarted = NO;
        self.towerCount = 0;
        self.ownshipMarker = nil;

        // set the orientation-specific values
        self.view.frame = CGRectMake(0.0f, 20.0f, 768.0f, 1004.0f);     // need this to get the toolbar set correctly initially
        
        // register for notifications that a heartbeat has been recevied
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateUIFromNewData)
                                                     name:@"HeartbeatMessageReceived"
                                                   object:nil];
            
        // register for notifications that the satellite data has been updated
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateWithNewSatData)
                                                     name:@"OwnshipMessageReceived"
                                                   object:nil];
        
        // register for notifications that the device data has been updated
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateWithNewDeviceData)
                                                     name:@"BlackjackDeviceDataUpdated"
                                                   object:nil];

        // register for notifications from the app delegate that blackjack has connected
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(blackjackConnected)
                                                     name:@"BlackjackConnected"
                                                   object:nil];
        
        // register for notifications from the app delegate that blackjack has disconnected
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(blackjackDisconnected)
                                                     name:@"BlackjackDisconnected"
                                                   object:nil];
        
        // register for notifications that an ADSB transmission has been recevied
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateADSBTransmissionStatus)
                                                     name:@"ADSBTransmissionReceived"
                                                   object:nil];

        // If the iPod/iPad/iPhone goes to sleep while a view controller is open, there is no notification
        // that the app is back to life, other than this applicationDidBecomeActive method being called. The viewWillAppear,
        // viewDidAppear, or viewDidOpen methods are not triggered when the iPod/iPad/iPhone is woken and the app resumes.
        // Consequently, listen for notification from the app delegate that the app has resumed becuase the UI will need
        // updated if the blackjack status changed while the iPod/iPad/iPhone was asleep.
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(refreshUIAfterAwakening)
                                                     name:@"RefreshUIAfterAwakening"
                                                   object:nil];
    }
	return self;
}

- (void)viewDidLoad
{
    // Create the various variables here for placing the UI objects.
    // common items for labels
    int commonFontSize = 16;
    int commonSectionHeadingFontSize = 18;
    NSString *commonFontName = @"Helvetica";
    NSString *commonSectionHeadingFontName = @"Helvetica-Bold";
    UIColor *commonTextColor = [UIColor whiteColor];
    UIColor *commonBackgroundColor = [UIColor clearColor];

    /*
    // Create the red dual logo and associated label text
    dualLogoImage = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"Dual logo.jpg"]];
    dualLogoImage.userInteractionEnabled = NO;
    [self.view addSubview:dualLogoImage];
    */
    
    self.dualLogoImageLabel = [[UILabel alloc] init];
    self.dualLogoImageLabel.text = NSLocalizedString(@"DUAL XGPS170 Device Status", @"iPadVC:initWithNibName:dual logo image label text");
    self.dualLogoImageLabel.font = [UIFont fontWithName:commonSectionHeadingFontName size:commonSectionHeadingFontSize + 2];
    self.dualLogoImageLabel.backgroundColor = commonBackgroundColor;
    self.dualLogoImageLabel.textColor = commonTextColor;
    self.dualLogoImageLabel.textAlignment = NSTextAlignmentCenter;
    [self.view addSubview:self.dualLogoImageLabel];
    
    // Create the device status group
    self.deviceInformationLabel = [[UILabel alloc] init];
    self.deviceInformationLabel.text = NSLocalizedString(@"Device Information", @"iPadVC:initWithNibName:XGPS150 device status label text");
    self.deviceInformationLabel.font = [UIFont fontWithName:commonSectionHeadingFontName size:commonSectionHeadingFontSize];
    self.deviceInformationLabel.backgroundColor = commonBackgroundColor;
    self.deviceInformationLabel.textColor = commonTextColor;
    self.deviceInformationLabel.textAlignment = NSTextAlignmentLeft;
    self.deviceInformationLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.deviceInformationLabel];
    
    self.statusLabel = [[UILabel alloc] init];
    self.statusLabel.text = NSLocalizedString(@"Status:", @"iPadVC:initWithNibName:conection status label text");
    self.statusLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.statusLabel.textAlignment = NSTextAlignmentRight;
    self.statusLabel.textColor = commonTextColor;
    self.statusLabel.backgroundColor = commonBackgroundColor;
    self.statusLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.statusLabel];
    
    self.firmwareVersionLabel = [[UILabel alloc] init];
    self.firmwareVersionLabel.text = NSLocalizedString(@"Firmware:", @"iPadVC:initWithNibName:firmware version label text");
    self.firmwareVersionLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.firmwareVersionLabel.textAlignment = NSTextAlignmentRight;
    self.firmwareVersionLabel.textColor = commonTextColor;
    self.firmwareVersionLabel.backgroundColor = commonBackgroundColor;
    self.firmwareVersionLabel.numberOfLines = 2;
    [self.view addSubview:self.firmwareVersionLabel];
    
    self.chargingLabel = [[UILabel alloc] init];
    self.chargingLabel.text = NSLocalizedString(@"Battery:", @"iPadVC:initWithNibName:charging status label text");
    self.chargingLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.chargingLabel.textAlignment = NSTextAlignmentRight;
    self.chargingLabel.textColor = commonTextColor;
    self.chargingLabel.backgroundColor = commonBackgroundColor;
    [self.view addSubview:self.chargingLabel];
    
    self.statusValue = [[UILabel alloc] init];
    self.statusValue.text = @"";
    self.statusValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.statusValue.textAlignment = NSTextAlignmentLeft;
    self.statusValue.textColor = commonTextColor;
    self.statusValue.backgroundColor = commonBackgroundColor;
    self.statusValue.isAccessibilityElement = NO;
    [self.view addSubview:self.statusValue];
    
    self.firmwareVersionValue = [[UILabel alloc] init];
    self.firmwareVersionValue.text = @"";
    self.firmwareVersionValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.firmwareVersionValue.textAlignment = NSTextAlignmentLeft;
    self.firmwareVersionValue.textColor = commonTextColor;
    self.firmwareVersionValue.backgroundColor = commonBackgroundColor;
    [self.view addSubview:self.firmwareVersionValue];
    
    self.chargingValue = [[UILabel alloc] init];
    self.chargingValue.text = @"";
    self.chargingValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.chargingValue.textAlignment = NSTextAlignmentLeft;
    self.chargingValue.textColor = commonTextColor;
    self.chargingValue.backgroundColor = commonBackgroundColor;
    self.chargingValue.hidden = YES;
    [self.view addSubview:self.chargingValue];
    
    // Create the bargraph for the battery level
    self.batLevelBargraph = [[Bar alloc] init];
    self.batLevelBargraph.startX = 0;
    self.batLevelBargraph.startY = 0;
    self.batLevelBargraph.lowLimit = 0.15;
    self.batLevelBargraph.highLimit = 0.35;
    self.batLevelBargraph.scaleValue = 1.0;
    self.batLevelBargraph.width = 80.0;
    self.batLevelBargraph.fontSize = commonFontSize;
    self.batLevelBargraph.rightLabelText = @"- %";
    self.batLevelBargraph.emphasizeRightLabel = YES;
    self.batLevelBargraph.hidden = NO;
    [self.view addSubview:self.batLevelBargraph];
    
    // Create the adsb info group
    self.adsbServiceLabel = [[UILabel alloc] init];
    self.adsbServiceLabel.text = NSLocalizedString(@"ADS-B Service", @"iPadVC:initWithNibName:adsb service text");
    self.adsbServiceLabel.font = [UIFont fontWithName:commonSectionHeadingFontName size:commonSectionHeadingFontSize];
    self.adsbServiceLabel.backgroundColor = commonBackgroundColor;
    self.adsbServiceLabel.textColor = commonTextColor;
    self.adsbServiceLabel.textAlignment = NSTextAlignmentLeft;
    [self.view addSubview:self.adsbServiceLabel];
    
    self.receivingDataLabel = [[UILabel alloc] init];
    self.receivingDataLabel.text = NSLocalizedString(@"Receiver status:", @"iPadVC:initWithNibName:receiving status label text");
    self.receivingDataLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.receivingDataLabel.textAlignment = NSTextAlignmentRight;
    self.receivingDataLabel.textColor = commonTextColor;
    self.receivingDataLabel.backgroundColor = commonBackgroundColor;
    self.receivingDataLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.receivingDataLabel];
    
    self.receivingDataValueWaiting = [[UILabel alloc] init];
    self.receivingDataValueWaiting.text = @"Listening...";
    self.receivingDataValueWaiting.font = [UIFont fontWithName:@"Helvetica" size:commonFontSize];
    self.receivingDataValueWaiting.textAlignment = NSTextAlignmentLeft;
    self.receivingDataValueWaiting.textColor = commonTextColor;
    self.receivingDataValueWaiting.backgroundColor = commonBackgroundColor;
    self.receivingDataValueWaiting.isAccessibilityElement = NO;
    self.receivingDataValueWaiting.alpha = 1.0;
    [self.view addSubview:self.receivingDataValueWaiting];
    
    self.receivingDataValueReceiving = [[UILabel alloc] init];
    self.receivingDataValueReceiving.text = @"Receiving";
    self.receivingDataValueReceiving.font = [UIFont fontWithName:@"Helvetica-ObliqueBold" size:commonFontSize];
    self.receivingDataValueReceiving.textAlignment = NSTextAlignmentLeft;
    self.receivingDataValueReceiving.textColor = commonTextColor;
    self.receivingDataValueReceiving.backgroundColor = commonBackgroundColor;
    self.receivingDataValueReceiving.isAccessibilityElement = NO;
    self.receivingDataValueReceiving.alpha = 0.0;
    [self.view addSubview:self.receivingDataValueReceiving];
    
    self.nexradLabel = [[UILabel alloc] init];
    self.nexradLabel.text = NSLocalizedString(@"NEXRAD:", @"iPadVC:initWithNibName:weather label text");
    self.nexradLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.nexradLabel.textAlignment = NSTextAlignmentRight;
    self.nexradLabel.textColor = commonTextColor;
    self.nexradLabel.backgroundColor = commonBackgroundColor;
    self.nexradLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.nexradLabel];
    
    self.conusLabel = [[UILabel alloc] init];
    self.conusLabel.text = NSLocalizedString(@"CONUS:", @"iPadVC:initWithNibName:weather label text");
    self.conusLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.conusLabel.textAlignment = NSTextAlignmentRight;
    self.conusLabel.textColor = commonTextColor;
    self.conusLabel.backgroundColor = commonBackgroundColor;
    self.conusLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.conusLabel];
    
    self.trafficLabel = [[UILabel alloc] init];
    self.trafficLabel.text = NSLocalizedString(@"Traffic:", @"iPadVC:initWithNibName:weather label text");
    self.trafficLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.trafficLabel.textAlignment = NSTextAlignmentRight;
    self.trafficLabel.textColor = commonTextColor;
    self.trafficLabel.backgroundColor = commonBackgroundColor;
    self.trafficLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.trafficLabel];
    
    self.notamsLabel = [[UILabel alloc] init];
    self.notamsLabel.text = NSLocalizedString(@"NOTAMs:", @"iPadVC:initWithNibName:notams label text");
    self.notamsLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.notamsLabel.textAlignment = NSTextAlignmentRight;
    self.notamsLabel.textColor = commonTextColor;
    self.notamsLabel.backgroundColor = commonBackgroundColor;
    self.notamsLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.notamsLabel];
    
    self.metarsLabel = [[UILabel alloc] init];
    self.metarsLabel.text = NSLocalizedString(@"METARs:", @"iPadVC:initWithNibName:metar label text");
    self.metarsLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.metarsLabel.textAlignment = NSTextAlignmentRight;
    self.metarsLabel.textColor = commonTextColor;
    self.metarsLabel.backgroundColor = commonBackgroundColor;
    self.metarsLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.metarsLabel];
    
    self.tafsLabel = [[UILabel alloc] init];
    self.tafsLabel.text = NSLocalizedString(@"TAFs:", @"iPadVC:initWithNibName:tafs label text");
    self.tafsLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.tafsLabel.textAlignment = NSTextAlignmentRight;
    self.tafsLabel.textColor = commonTextColor;
    self.tafsLabel.backgroundColor = commonBackgroundColor;
    self.tafsLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.tafsLabel];
    
    self.datisLabel = [[UILabel alloc] init];
    self.datisLabel.text = NSLocalizedString(@"D-ATIS:", @"iPadVC:initWithNibName:d-atis label text");
    self.datisLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.datisLabel.textAlignment = NSTextAlignmentRight;
    self.datisLabel.textColor = commonTextColor;
    self.datisLabel.backgroundColor = commonBackgroundColor;
    self.datisLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.datisLabel];
    
    self.airmetLabel = [[UILabel alloc] init];
    self.airmetLabel.text = NSLocalizedString(@"AIRMETs:", @"iPadVC:initWithNibName:airmet label text");
    self.airmetLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.airmetLabel.textAlignment = NSTextAlignmentRight;
    self.airmetLabel.textColor = commonTextColor;
    self.airmetLabel.backgroundColor = commonBackgroundColor;
    self.airmetLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.airmetLabel];
    
    self.sigmetLabel = [[UILabel alloc] init];
    self.sigmetLabel.text = NSLocalizedString(@"SIGMETs:", @"iPadVC:initWithNibName:sigmet label text");
    self.sigmetLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.sigmetLabel.textAlignment = NSTextAlignmentRight;
    self.sigmetLabel.textColor = commonTextColor;
    self.sigmetLabel.backgroundColor = commonBackgroundColor;
    self.sigmetLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.sigmetLabel];
    
    self.pirepsLabel = [[UILabel alloc] init];
    self.pirepsLabel.text = NSLocalizedString(@"PIREPs:", @"iPadVC:initWithNibName:pireps label text");
    self.pirepsLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.pirepsLabel.textAlignment = NSTextAlignmentRight;
    self.pirepsLabel.textColor = commonTextColor;
    self.pirepsLabel.backgroundColor = commonBackgroundColor;
    self.pirepsLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.pirepsLabel];
    
    self.windsLabel = [[UILabel alloc] init];
    self.windsLabel.text = NSLocalizedString(@"Winds:", @"iPadVC:initWithNibName:winds label text");
    self.windsLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.windsLabel.textAlignment = NSTextAlignmentRight;
    self.windsLabel.textColor = commonTextColor;
    self.windsLabel.backgroundColor = commonBackgroundColor;
    self.windsLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.windsLabel];
    
    self.airspaceUpdateLabel = [[UILabel alloc] init];
    self.airspaceUpdateLabel.text = NSLocalizedString(@"SUA Update:", @"iPadVC:initWithNibName:airspace update label text");
    self.airspaceUpdateLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.airspaceUpdateLabel.textAlignment = NSTextAlignmentRight;
    self.airspaceUpdateLabel.textColor = commonTextColor;
    self.airspaceUpdateLabel.backgroundColor = commonBackgroundColor;
    self.airspaceUpdateLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.airspaceUpdateLabel];
    
    self.towerCountLabel = [[UILabel alloc] init];
    self.towerCountLabel.text = @"Towers:";
    self.towerCountLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.towerCountLabel.textAlignment = NSTextAlignmentRight;
    self.towerCountLabel.textColor = commonTextColor;
    self.towerCountLabel.backgroundColor = commonBackgroundColor;
    self.towerCountLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.towerCountLabel];
    
    self.towerCountValue = [[UILabel alloc] init];
    self.towerCountValue.text = @"-";
    self.towerCountValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:commonFontSize];
    self.towerCountValue.textAlignment = NSTextAlignmentLeft;
    self.towerCountValue.adjustsFontSizeToFitWidth = YES;
    self.towerCountValue.textColor = commonTextColor;
    self.towerCountValue.backgroundColor = commonBackgroundColor;
    self.towerCountValue.isAccessibilityElement = NO;
    [self.view addSubview:self.towerCountValue];

    self.messageCountLabel = [[UILabel alloc] init];
    self.messageCountLabel.text = @"ADS-B msgs/sec:";
    self.messageCountLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.messageCountLabel.textAlignment = NSTextAlignmentRight;
    self.messageCountLabel.textColor = commonTextColor;
    self.messageCountLabel.backgroundColor = commonBackgroundColor;
    self.messageCountLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.messageCountLabel];
    
    self.messageCountValue = [[UILabel alloc] init];
    self.messageCountValue.text = @"-";
    self.messageCountValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:commonFontSize];
    self.messageCountValue.textAlignment = NSTextAlignmentLeft;
    self.messageCountValue.adjustsFontSizeToFitWidth = YES;
    self.messageCountValue.textColor = commonTextColor;
    self.messageCountValue.backgroundColor = commonBackgroundColor;
    self.messageCountValue.isAccessibilityElement = NO;
    [self.view addSubview:self.messageCountValue];
    
    /*
    nexradAgeLabel = [[UILabel alloc] init];
    nexradAgeLabel.text = NSLocalizedString(@"NEXRAD Age:", @"iPadVC:initWithNibName:nexradAgeLabel text");
    nexradAgeLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    nexradAgeLabel.textAlignment = NSTextAlignmentRight;
    nexradAgeLabel.textColor = commonTextColor;
    nexradAgeLabel.backgroundColor = commonBackgroundColor;
    nexradAgeLabel.isAccessibilityElement = NO;
    [self.view addSubview:nexradAgeLabel];
    
    nexradAgeValue = [[UILabel alloc] init];
    nexradAgeValue.text = NSLocalizedString(@"> 1 hour", @"iPadVC:initWithNibName:nexradAgeValue text");
    nexradAgeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:commonFontSize];
    nexradAgeValue.textAlignment = NSTextAlignmentLeft;
    nexradAgeValue.adjustsFontSizeToFitWidth = YES;
    nexradAgeValue.textColor = commonTextColor;
    nexradAgeValue.backgroundColor = commonBackgroundColor;
    nexradAgeValue.isAccessibilityElement = NO;
    [self.view addSubview:nexradAgeValue];
    
    conusNexradAgeLabel = [[UILabel alloc] init];
    conusNexradAgeLabel.text = NSLocalizedString(@"CONUS Age:", @"iPadVC:initWithNibName:conusNexradAgeLabel text");
    conusNexradAgeLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    conusNexradAgeLabel.textAlignment = NSTextAlignmentRight;
    conusNexradAgeLabel.textColor = commonTextColor;
    conusNexradAgeLabel.backgroundColor = commonBackgroundColor;
    conusNexradAgeLabel.isAccessibilityElement = NO;
    [self.view addSubview:conusNexradAgeLabel];
    
    conusNexradAgeValue = [[UILabel alloc] init];
    conusNexradAgeValue.text = NSLocalizedString(@"> 1 hour", @"iPadVC:initWithNibName:conusNexradAgeValue text");
    conusNexradAgeValue.font = [UIFont fontWithName:@"Helvetica-BoldOblique" size:commonFontSize];
    conusNexradAgeValue.textAlignment = NSTextAlignmentLeft;
    conusNexradAgeValue.adjustsFontSizeToFitWidth = YES;
    conusNexradAgeValue.textColor = commonTextColor;
    conusNexradAgeValue.backgroundColor = commonBackgroundColor;
    conusNexradAgeValue.isAccessibilityElement = NO;
    [self.view addSubview:conusNexradAgeValue];
    */
    
    float lightbulbRadius = 41.0;
    float lightbulbRefX = 168.0;
    float lightbulbRefY = 250.0; // 330.0;
    float lightbulbSpacing = 28.0;
    self.nexradBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY, lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.nexradBulb];
    self.conusBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + lightbulbSpacing, lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.conusBulb];
    self.trafficBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (2.0 * lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.trafficBulb];
    self.notamsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (3.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.notamsBulb];
    self.metarsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (4.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.metarsBulb];
    self.tafsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (5.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.tafsBulb];
    self.datisBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (6.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.datisBulb];
    self.airmetsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (7.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.airmetsBulb];
    self.sigmetsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (8.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.sigmetsBulb];
    self.pirepsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (9.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.pirepsBulb];
    self.windsBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (10.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.windsBulb];
    self.airspaceBulb = [[Lightbulb alloc] initWithFrame:CGRectMake(lightbulbRefX, lightbulbRefY + (11.0 *lightbulbSpacing), lightbulbRadius, lightbulbRadius)];
    [self.view addSubview:self.airspaceBulb];
    
    // Create the position info group
    self.positionInfoLabel = [[UILabel alloc] init];
    self.positionInfoLabel.text = NSLocalizedString(@"GPS Position Information", @"iPadVC:initWithNibName:position information label text");
    self.positionInfoLabel.font = [UIFont fontWithName:commonSectionHeadingFontName size:commonSectionHeadingFontSize];
    self.positionInfoLabel.backgroundColor = commonBackgroundColor;
    self.positionInfoLabel.textColor = commonTextColor;
    self.positionInfoLabel.textAlignment = NSTextAlignmentLeft;
    self.positionInfoLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.positionInfoLabel];
    
    self.latitudeLabel = [[UILabel alloc] init];
    self.latitudeLabel.text = NSLocalizedString(@"Latitude:", @"iPadVC:initWithNibName:latitude label text");
    self.latitudeLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.latitudeLabel.textAlignment = NSTextAlignmentRight;
    self.latitudeLabel.textColor = commonTextColor;
    self.latitudeLabel.backgroundColor = commonBackgroundColor;
    self.latitudeLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.latitudeLabel];
    
    self.longitudeLabel = [[UILabel alloc] init];
    self.longitudeLabel.text = NSLocalizedString(@"Longitude:", @"iPadVC:initWithNibName:longitude label text");
    self.longitudeLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.longitudeLabel.textAlignment = NSTextAlignmentRight;
    self.longitudeLabel.textColor = commonTextColor;
    self.longitudeLabel.backgroundColor = commonBackgroundColor;
    self.longitudeLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.longitudeLabel];
    
    self.altitudeLabel = [[UILabel alloc] init];
    self.altitudeLabel.text = NSLocalizedString(@"Altitude:", @"iPadVC:initWithNibName:altitude label text");
    self.altitudeLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.altitudeLabel.textAlignment = NSTextAlignmentRight;
    self.altitudeLabel.textColor = commonTextColor;
    self.altitudeLabel.backgroundColor = commonBackgroundColor;
    self.altitudeLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.altitudeLabel];
    
    self.waasInUseLabel = [[UILabel alloc] init];
    self.waasInUseLabel.text = @"WAAS in use:";
    self.waasInUseLabel.font = [UIFont fontWithName:commonSectionHeadingFontName size:commonFontSize];
    self.waasInUseLabel.textAlignment = NSTextAlignmentRight;
    self.waasInUseLabel.textColor = commonTextColor;
    self.waasInUseLabel.backgroundColor = commonBackgroundColor;
    self.waasInUseLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.waasInUseLabel];
    
    self.speedLabel = [[UILabel alloc] init];
    self.speedLabel.text = NSLocalizedString(@"Speed:", @"iPadVC:initWithNibName:speed label text");
    self.speedLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.speedLabel.textAlignment = NSTextAlignmentRight;
    self.speedLabel.textColor = commonTextColor;
    self.speedLabel.backgroundColor = commonBackgroundColor;
    self.speedLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.speedLabel];
    
    self.headingLabel = [[UILabel alloc] init];
    self.headingLabel.text = NSLocalizedString(@"Heading:", @"iPadVC:initWithNibName:heading label text");
    self.headingLabel.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.headingLabel.textAlignment = NSTextAlignmentRight;
    self.headingLabel.textColor = commonTextColor;
    self.headingLabel.backgroundColor = commonBackgroundColor;
    self.headingLabel.isAccessibilityElement = NO;
    [self.view addSubview:self.headingLabel];
    
    self.latitudeValue = [[UILabel alloc] init];
    self.latitudeValue.text = @"";
    self.latitudeValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.latitudeValue.textAlignment = NSTextAlignmentLeft;
    self.latitudeValue.adjustsFontSizeToFitWidth = YES;
    self.latitudeValue.textColor = commonTextColor;
    self.latitudeValue.backgroundColor = commonBackgroundColor;
    self.latitudeValue.isAccessibilityElement = NO;
    [self.view addSubview:self.latitudeValue];
    
    self.longitudeValue = [[UILabel alloc] init];
    self.longitudeValue.text = @"";
    self.longitudeValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.longitudeValue.textAlignment = NSTextAlignmentLeft;
    self.longitudeValue.adjustsFontSizeToFitWidth = YES;
    self.longitudeValue.textColor = commonTextColor;
    self.longitudeValue.backgroundColor = commonBackgroundColor;
    self.longitudeValue.isAccessibilityElement = NO;
    [self.view addSubview:self.longitudeValue];
    
    self.altitudeValue = [[UILabel alloc] init];
    self.altitudeValue.text = @"";
    self.altitudeValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.altitudeValue.textAlignment = NSTextAlignmentLeft;
    self.altitudeValue.adjustsFontSizeToFitWidth = YES;
    self.altitudeValue.textColor = commonTextColor;
    self.altitudeValue.backgroundColor = commonBackgroundColor;
    self.altitudeValue.isAccessibilityElement = NO;
    [self.view addSubview:self.altitudeValue];
    
    self.speedValue = [[UILabel alloc] init];
    self.speedValue.text = @"";
    self.speedValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.speedValue.textAlignment = NSTextAlignmentLeft;
    self.speedValue.textColor = commonTextColor;
    self.speedValue.backgroundColor = commonBackgroundColor;
    self.speedValue.isAccessibilityElement = NO;
    [self.view addSubview:self.speedValue];

    self.headingValue = [[UILabel alloc] init];
    self.headingValue.text = @"";
    self.headingValue.font = [UIFont fontWithName:commonFontName size:commonFontSize];
    self.headingValue.textAlignment = NSTextAlignmentLeft;
    self.headingValue.textColor = commonTextColor;
    self.headingValue.backgroundColor = commonBackgroundColor;
    self.headingValue.isAccessibilityElement = NO;
    [self.view addSubview:self.headingValue];
    
    self.waasInUseValue = [[UILabel alloc] init];
    self.waasInUseValue.text = @"";
    self.waasInUseValue.font = [UIFont fontWithName:commonSectionHeadingFontName size:commonFontSize];
    self.waasInUseValue.textAlignment = NSTextAlignmentLeft;
    self.waasInUseValue.textColor = commonTextColor;
    self.waasInUseValue.backgroundColor = commonBackgroundColor;
    self.waasInUseValue.isAccessibilityElement = NO;
    [self.view addSubview:self.waasInUseValue];

    // create the "Waiting to connect" message view
    self.blackjackNotConnectedMessage = [[WaitingToConnectView alloc] init];
    self.blackjackNotConnectedMessage.tag = kNoblackjackMessageView;
    self.blackjackNotConnectedMessage.hidden = YES;		// start off with the view hidden
    [self.view addSubview:self.blackjackNotConnectedMessage];
    
    // Now position the objects on the screen according to the orientation of the device
    [self setLayout:[[UIApplication sharedApplication] statusBarOrientation]];
    
    [self setupWeatherView];
}

- (void)viewDidAppear:(BOOL)animated
{
	// NOTE: if the iPod/iPad/iPhone goes to sleep while this view controller is open, there is no notification
	// that the app is back to life, other than the applicationDidBecomeActive method is called. The viewWillAppear,
	// viewDidAppear, or viewDidOpen methods are not triggered when the iPod/iPad/iPhone is woken and the app resumes.
	// Consequently, UI will updated after reawakening by the "refreshUIAfterAwakening" method.

	create_delegate_instance_d
    
	if (d.blackjack.isConnected == NO) [self displayblackjackNotAttachedMessage];
    
    [self.view bringSubviewToFront:self.metarsBulb];
    
    // Check to see if test data should be used, or if live stream data should be used.
    [self dataSourceSwitchChanged:nil];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - MKMapView Delegate Methods
- (MKOverlayRenderer *)mapView:(MKMapView *)mapView rendererForOverlay:(id<MKOverlay>)overlay {

    if ([overlay isKindOfClass:NEXRADWeatherOverlay.class]) {
        NEXRADWeatherOverlayRenderer *overlayView = [[NEXRADWeatherOverlayRenderer alloc] initWithOverlay:overlay overlayImage:self.createdNEXRADImage];
        return overlayView;
    }
    else if ([overlay isKindOfClass:CONUSWeatherOverlay.class]) {
        CONUSWeatherOverlayRenderer *overlayView = [[CONUSWeatherOverlayRenderer alloc] initWithOverlay:overlay overlayImage:self.createdCONUSImage];
        return overlayView;
    }
    else if ([overlay isKindOfClass:NEXRADTestWeatherDataOverlay.class]) {
        NEXRADTestWeatherDataOverlayRenderer *overlayView = [[NEXRADTestWeatherDataOverlayRenderer alloc] initWithOverlay:overlay
                                                                                                             overlayImage:self.createdNEXRADImageFromTestData];
        return overlayView;
    }
    else if ([overlay isKindOfClass:CONUSTestWeatherDataOverlay.class]) {
        CONUSTestWeatherDataOverlayRenderer *overlayView = [[CONUSTestWeatherDataOverlayRenderer alloc] initWithOverlay:overlay
                                                                                                           overlayImage:self.createdCONUSImageFromTestData];
        return overlayView;
    }
    else if ([overlay isKindOfClass:ReceivedNEXRADRegionPolygon.class]) {
        MKPolygonRenderer *polygonView = [[MKPolygonRenderer alloc] initWithPolygon:overlay];
        polygonView.strokeColor = [UIColor redColor];
        polygonView.lineWidth = 2;
        polygonView.fillColor = [UIColor clearColor];
        return polygonView;
    }
    else if ([overlay isKindOfClass:ReceivedCONUSRegionPolygon.class]) {
        MKPolygonRenderer *polygonView = [[MKPolygonRenderer alloc] initWithPolygon:overlay];
        polygonView.strokeColor = [UIColor magentaColor];
        polygonView.lineWidth = 2;
        polygonView.fillColor = [UIColor clearColor];
        return polygonView;
    }
    return nil;
}

- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>)annotation
{
    // If the annotation is the user location, just return nil.
    if ([annotation isKindOfClass:[MKUserLocation class]])
        return nil;
    
    // Handle any custom annotations.
    if ([annotation isKindOfClass:[ADSBTowerAnnotation class]])
    {
        // Try to dequeue an existing pin view first.
        ADSBTowerAnnotationView *tower = (ADSBTowerAnnotationView*)[mapView dequeueReusableAnnotationViewWithIdentifier:@"tower"];
        
        if (!tower)
        {
            // If an existing pin view was not available, create one.
            tower = [[ADSBTowerAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"tower"];
            
            // If appropriate, customize the callout by adding accessory views (code not shown).
        }
        else
            tower.annotation = annotation;
        
        return tower;
    }
    
    if ([annotation isKindOfClass:[OwnshipPositionAnnotation class]])
    {
        // Try to dequeue an existing pin view first.
        OwnshipPositionAnnotationView *ownshipPosition = (OwnshipPositionAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:@"ownship"];
        
        if (!ownshipPosition)
        {
            // If an existing pin view was not available, create one.
            ownshipPosition = [[OwnshipPositionAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"ownship"];
            
            // If appropriate, customize the callout by adding accessory views (code not shown).
        }
        else
            ownshipPosition.annotation = annotation;
        
        return ownshipPosition;
    }
    
    return nil;
}

# pragma mark - Methods to Manage Sample Data
-(void)generateTestPatterns
{
    // return; // Uncomment this line to prevent the test patterns from being created in the ADS-B weather images
    
    if (self.NEXRADTestPatternCreated == NO)
    {
        [self createNEXRADTestPatterns];
        self.NEXRADTestPatternCreated = YES;
    }
    
    if (self.CONUSTestPatternCreated == NO)
    {
        [self createCONUSTestPatterns];
        self.CONUSTestPatternCreated = YES;
    }
}

-(void)createCONUSTestPatterns
{
    // sample test patterns
    
    /*
     unsigned char testPattern[128] = {
     0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0,
     0, 0, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4, 5, 5, 6, 6, 6, 5, 5, 4, 4, 3, 3, 3, 2, 2, 2, 1, 1, 0, 0,
     0, 0, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4, 5, 5, 6, 6, 6, 5, 5, 4, 4, 3, 3, 3, 2, 2, 2, 1, 1, 0, 0,
     0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 3,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
     0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5,
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5,
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5,
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6,
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6,
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6,
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6
     };
     */
    
    
    unsigned char testPattern[128] = {
        0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7,
        1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0,
        2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1,
        3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2
    };
    
    create_delegate_instance_d
    
    uint32_t blockNumber = [d.blackjack convertLatLongToBlockNumber:(double)kTestDataOriginLatitude longitude:(double)kTestDataOriginLongitude];
    NSData *blockData = [NSData dataWithBytes:testPattern length:128];
    
    [d.blackjack expandCONUSDataBlocks:blockData isEmptyBlock:NO blockNumber:blockNumber scaleFactor:kMediumResolutionCONUSDataScaling];
    
    // Create a second CONUS block to the right. Assuming medium resolution scaling
    // (scale factor = 5), the next block to the right will be 450 + 5 = 455.
    //[d.blackjack expandCONUSDataBlocks:blockData isEmptyBlock:NO blockNumber:(blockNumber+5) scaleFactor:kMediumResolutionCONUSDataScaling];
    
    // Create a third CONUS block one row above the first block.  Assuming medium resolution scaling
    // (scale factor = 5), the next block above will be 450 * 5 = 2250.
    //[d.blackjack expandCONUSDataBlocks:blockData isEmptyBlock:NO blockNumber:(blockNumber+2250) scaleFactor:kMediumResolutionCONUSDataScaling];
    
    // Create a fourth CONUS block one row above and one block to the right.
    [d.blackjack expandCONUSDataBlocks:blockData isEmptyBlock:NO blockNumber:(blockNumber+2255) scaleFactor:kMediumResolutionCONUSDataScaling];
    
    // Generate the image
    [d.blackjack calculateNEXRADDataCoverageAreaUsingCONUS:YES];
    self.createdCONUSImageFromTestData = [d.blackjack createNationalWeatherImage];
    [self synthesizeTestWeatherDataImageOverlayUsingCONUS:YES];

} // createCONUSTestPatterns


-(void)createNEXRADTestPatterns
{
    // sample test patterns
    
    /*
     unsigned char testPattern[128] = {
     0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0,
     0, 0, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4, 5, 5, 6, 6, 6, 5, 5, 4, 4, 3, 3, 3, 2, 2, 2, 1, 1, 0, 0,
     0, 0, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4, 5, 5, 6, 6, 6, 5, 5, 4, 4, 3, 3, 3, 2, 2, 2, 1, 1, 0, 0,
     0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 3,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
     0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
     0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4,
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5,
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5,
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5,
     3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 3, 3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5
     };
     */
    
    /*
     unsigned char testPattern[128] = {
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6,
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6,
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6,
     1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 6, 6, 6, 6, 6, 6, 6, 6
     };
     */
    
    
    unsigned char testPattern[128] = {
        0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7,
        1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0,
        2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1,
        3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2
    };
    
    create_delegate_instance_d
    
    // Create an array of block numbers which will hold the synthesized data
    NSArray *arrayOfBlockSlices = [d.blackjack getBlockListForUpperLeftLat:kTestDataOriginLatitude + kTestDataNEXRADRegionSize
                                                              UpperLeftLon:kTestDataOriginLongitude - kTestDataNEXRADRegionSize
                                                             lowerRightLat:kTestDataOriginLatitude - kTestDataNEXRADRegionSize
                                                             lowerRightLon:kTestDataOriginLongitude + kTestDataNEXRADRegionSize];

    // For each block, populate the data object with the test pattern
    for (NSArray *slice in arrayOfBlockSlices)
    {
        for (int i=0; i<[slice count]; i++)
        {
            NSNumber *blockID = [slice objectAtIndex:i];
            NSData *blockData = [NSData dataWithBytes:testPattern length:128];
            [d.blackjack.fisbBlocks setObject:blockData forKey:blockID];
        }
    }
    
    // Generate the image
    [d.blackjack calculateNEXRADDataCoverageAreaUsingCONUS:NO];
    self.createdNEXRADImageFromTestData = [d.blackjack createRegionalWeatherImage];
    [self synthesizeTestWeatherDataImageOverlayUsingCONUS:NO];
   
} // createNEXRADTestPatterns

- (void)clearTestData
{
    create_delegate_instance_d
    
    [d.blackjack.fisbBlocks removeAllObjects];
    [d.blackjack.fisbCONUSBlocks removeAllObjects];
    
    self.NEXRADTestPatternCreated = NO;
    self.CONUSTestPatternCreated = NO;
}

@end
