//
//  Blackjack+ADSBDataMgr.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define kUATPayloadApplicationDataLength    424     // see page 31, section 4.1.2 of GDL90 spec for this value
#define kMaxNumberOfEmptyBlocks             125     // see page D-18 for this value
#define kFISBTextLength                      63     // See page 40, section 5.2.4 of GDL90 spec
#define kLatLonBitResolution       2.1457672e-5
#define kTrackHeadingResolution         1.40625     // See page 18, Table 8 of GDL90 spec

#define kLowResolutionCONUSDataScaling      9
#define kMediumResolutionCONUSDataScaling   5
#define kHighResoltuionCONUSDataScaling     1
#define kNumberOfBinsPerRow                 32
#define kNumberOfRowsPerBlock               4
#define kNumberOfBinsPerBlock               128     // see page 36, section 5.1.4 of GDL90 spec for this value

#define DEBUG_SHOW_RECEIVED_MESSAGES        NO
#define DEBUG_DEGCALC                       NO
#define DEBUG_STREAM_PARSING                NO
#define DEBUG_GDL90_MESSAGE_PARSING         NO
#define DEBUG_GDL90_MESSAGE_PARSING_RESULT  NO
#define DEBUG_DATA_ACCESSORS                NO

#define DEBUG_HEARTBEAT_MESSAGE             NO
#define DEBUG_OWNSHIP_REPORT_MESSAGE        NO
#define DEBUG_OWNSHIP_GA_REPORT_MESSAGE     NO
#define DEBUG_SHORT_TRAFFIC_MESSAGE         NO
#define DEBUG_BASIC_TRAFFIC_MESSAGE         NO
#define DEBUG_LONG_TRAFFIC_MESSAGE          NO
#define DEBUG_TRAFFIC_MESSAGE_B             NO
#define DEBUG_UPLINK_MESSAGE                NO
#define DEBUG_DEVICE_INFO_MESSAGE           NO
#define DEBUG_DEVICE_RESPONSES              NO
#define DEBUG_APDU_HEADER_DATA              NO
#define DEBUG_APDU_PAYLOAD_DATA             NO
#define DEBUG_GRAPHICAL_OVERLAY_DATA_A      NO
#define DEBUG_GRAPHICAL_OVERLAY_DATA        NO
#define DEBUG_GRAPHICAL_OVERLAY_SUMMARY     NO
#define DEBUG_PRODUCTID_8                   NO
#define DEBUG_PRODUCTID_9                   NO
#define DEBUG_PRODUCTID_10                  NO
#define DEBUG_PRODUCTID_11                  NO
#define DEBUG_PRODUCTID_12                  NO
#define DEBUG_PRODUCTID_13                  NO
#define DEBUG_PRODUCTID_63                  NO
#define DEBUG_PRODUCTID_63_B                NO
#define DEBUG_PRODUCTID_63_C                NO
#define DEBUG_PRODUCTID_64                  NO
#define DEBUG_PRODUCTID_64_B                NO
#define DEBUG_PRODUCTID_413                 NO
#define DEBUG_GRAPHICAL_WEATHER_RECEIVED    NO

#define DEBUG_PGSA_INFO                     NO
#define DEBUG_PGSV_INFO                     NO

#import "Blackjack.h"
#import "BlackjackDataStructures.h"
#import "Blackjack+ADSBDataMgr.h"
#import "Blackjack+ControlMgr.h"
#import "BlackjackTestData.h"
#import "AppDelegate.h"
#import <MobileCoreServices/MobileCoreServices.h>
#import <ImageIO/ImageIO.h>

@implementation Blackjack (ADSBDataMgr)

typedef struct
{
    float degrees;
    bool  isSouthOrWest;
} CalculatedDegrees;

typedef struct
{
    unsigned char B2;
    unsigned char B1;
    unsigned char B0;
} Base40Bytes;

static unsigned char dlacCode [64] = {
    // etx, A, B, C, D, E, F, G, H, I, J, K,
    0x03, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B,
    // L, M, N, O, P, Q, R, S, T, U, V, W,
    0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0X52, 0x53, 0x54, 0x55, 0x56, 0x57,
    // X, Y, Z, nul, tab, rs, crlf=lf, "change-ciper" = nul, space, !, ", #,
    0x58, 0x59, 0x5A, 0x00, 0x09, 0x1e, 0x0a, 0x00, 0x20, 0x21, 0x22, 0x23,
    // currency = $, %, &, ', (, ), *, +, comma, -, ., /,
    0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
    // 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, :, ;,
    0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B,
    // ;, <, =, >, ?
    0x3C, 0x3D, 0x3E, 0x3F
};

# pragma mark - GDL90 CRC methods
- (void)crcTableInit
{
    uint16_t i, bitctr, crc;
    for (i=0; i < 256; i++)
    {
        crc = i << 8;
        for (bitctr = 0; bitctr < 8; bitctr++)
        {
            crc = (crc << 1) ^ ((crc & 0x8000) ? 0x1021 : 0);
        }

        [self.crc16Table addObject:[NSNumber numberWithUnsignedInt:crc]];
    }
}

- (uint16_t)crcCompute:(uint8_t *)block length:(uint32_t)length
{
    uint32_t i;
    uint16_t crc = 0;
    
    for (i=0; i < length; i++)
        crc = [[self.crc16Table objectAtIndex:(crc >>8)] unsignedIntValue] ^ (crc << 8) ^ block[i];
    
    return crc;
}

#pragma mark - Debug methods
- (void)printBuffer:(NSString *)text buffer:(uint8_t *)msg length:(int)length
{
    NSLog(@"printBuffer. %@", text);
    for (int j=0; j<length; j++) printf("0x%.2X ", msg[j]);
    printf("\n\n");
}

- (void)printLongTrafficReport:(LongTrafficObjectInfo *)ltoi;
{
    NSLog(@"Traffic info:");
    NSLog(@"Time received: %2d:%2d:%2d Zulu\n", ltoi->hour, ltoi->min, ltoi->sec);
    if (ltoi->adsbTarget) NSLog(@"Airborne traffic (reported via ADS-B).");
    if (ltoi->tisbTarget) NSLog(@"Airborne traffic (reported via TIS-B).");
    if (ltoi->surfaceVehicle) NSLog(@"Surface vehicle.");
    if (ltoi->adsbBeacon) NSLog(@"ADS-B surface beacon.");
    NSLog(@"ICAO address = %d. Address %@ self-assigned.", ltoi->icaoAddress, (ltoi->icaoAddressIsSelfAssigned)?@"is":@"is not");
    NSLog(@"Position is %.4f,%.4f", ltoi->latitude, ltoi->longitude);
    NSLog(@"Altitude is %d. (%@ altitude)", ltoi->altitude, (ltoi->altitudeType)?@"pressure":@"geometric");
    NSLog(@"NIC = %.4f.", ltoi->nic);
    if (ltoi->airborne)
    {
        NSLog(@"Object is airborne and %@ supersonic.", (ltoi->supersonic)?@"is":@"is not");
        NSLog(@"Northerly velocity vector is %d knots. Easterly velocity vector is %d knots.",
              ltoi->northerlyVelocity, ltoi->easterlyVelocity);
        if (ltoi->verticalVelocityRate > 0)
        {
            NSLog(@"Object is %@ at %d FPM", (ltoi->verticalVelocityDirection)?@"descending":@"ascending", ltoi->verticalVelocityDirection);
            NSLog(@"Vertical velocity is from a %@ source.", (ltoi->verticalVelocitySource)?@"barometric":@"geometric");            
        }
        else NSLog(@"No vertical velocity information available.");
    }
    else
    {
        NSLog(@"Object is on the ground.");
        NSLog(@"Ground speed is %d knots and heading is %.1f degrees.", ltoi->groundSpeed, ltoi->heading);
        NSString *objectHeading;
        if (ltoi->trueTrackAngle) objectHeading = @"true track";
        if (ltoi->magneticHeading) objectHeading = @"magnetic";
        if (ltoi->trueHeading) objectHeading = @"true";
        if ((ltoi->trueTrackAngle || ltoi->magneticHeading || ltoi->trueHeading) == NO) objectHeading = @"not valid";
        NSLog(@"Heading is %@.", objectHeading);
        
        if (ltoi->vehicleSizeIsValid)
        {
            NSLog(@"Object is up to %.1f meters wide and up to %d meters long.", ltoi->width, ltoi->length);
            if (ltoi->gpsAntennaInfoIsValid)
            {
                if (ltoi->gpsAntennaOffsetAxis)
                {
                    NSLog(@"GPS antenna on object is offset from the nose by %d m.", ltoi->gpsAntennaOffsetDistance);
                }
                else
                {
                    NSLog(@"GPS antenna on object is offset to the %@ by %d m.",
                          (ltoi->gpsAntennaOffsetSide)?@"right":@"left", ltoi->gpsAntennaOffsetDistance);
                }
            }
            else NSLog(@"No information available about the position of the GPS antenna on the object.");
        }
        else NSLog(@"No information available about the size of the object.");
    }
    
    NSLog(@"Object's transmitting system is %@ coupled to UTC time.", (ltoi->adsbTransmittingSourceCoupledToUTCTime)?@"is":@"is not");
    NSLog(@"Maximum number of successful ground uplink message is %d.", ltoi->uplinkFeedback);
    NSLog(@"TIS-B site id is %d.", ltoi->tisbSiteID);
    NSLog(@"Category is %s", ltoi->emitterCategoryDescr);
    NSLog(@"Call sign / tail number / flightplan ID is %s.", ltoi->callSignFlightPlanID);
    NSLog(@"Status is %s", ltoi->statusDescr);
    NSLog(@"MOPS version is %d.", ltoi->mopsVersion);
    NSLog(@"SIL value is %d.", ltoi->sil);
    NSLog(@"Transmit MSO value is %d.", ltoi->transmitMSO);
    NSLog(@"SDA value is %d.", ltoi->sda);
    NSLog(@"NACp value is %d. NACv value is %d.", ltoi->nacP, ltoi->nacV);
    NSLog(@"NIC-Barometric value is %d.", ltoi->nicBaro);
    NSLog(@"Capability codes are 0x%X.", ltoi->capabilityCodes);
    NSLog(@"Operational modes are 0x%X.", ltoi->operationalModes);
    NSLog(@"CSID value is %d.", ltoi->csid);
    NSLog(@"SILsupp value is %d.", ltoi->silSUPP);
    NSLog(@"Geo Vert Acc value is %d", ltoi->geoVertAcc);
    NSLog(@"SA flag value is %d.", ltoi->saFlag);
    NSLog(@"NIC supp value is %d.", ltoi->nicSUPP);
}

- (NSString *)bytes_to_binary:(unsigned long)val numberOfBytes:(unsigned char)numberOfBytes
{
	static char b[40];
	unsigned long z;
	unsigned char spacer = 0;
	char *p = &b[0];
    
	unsigned char length = (8 * numberOfBytes) - 1;
	for (z = pow(2,length); z > 0; z >>= 1)
	{
		spacer++;
		*p++ = (val & z) ? '1' : '0';
		if (spacer == 8)
		{
			*p++ = ' ';
			spacer = 0;
		}
	}
	*p++ = '\0';
	
    return [NSString stringWithFormat:@"%s", b];
}


#pragma mark - Helper Methods
- (float)calculateDegrees:(uint8_t)highByte middleByte:(uint8_t)midByte lowByte:(uint8_t)lowByte
{
    uint32_t position = 0;
    
    float xx;
    
    position = highByte;
    position <<= 8;
    position |= midByte;
    position <<= 8;
    position |= lowByte;
    
    if (position & 0x800000)
    {
        int yy;
        
        position |= 0xFF000000;
        
        yy = (int) position;
        xx = (double)(yy);
    }
    else
    {
        xx = (position & 0x7FFFFF);
    }    

    xx *= kLatLonBitResolution;
    
    return xx;
}

- (NSArray *)calculateZuluTime:(uint8_t)highByte middleByte:(uint8_t)midByte lowByte:(uint8_t)lowByte
{
    // Time is 24-bit number with with 80 ns resolution.
    // See page 26 of the GDL90 spec for more info: Table 13.
   
    uint32_t timeOfReception = 0;
    timeOfReception = highByte << 16;
    timeOfReception |= midByte << 8;
    timeOfReception |= lowByte;
    
    float hours = timeOfReception * 0.00008 / 3600;
    float minutes = (hours - floor(hours)) * 60;
    float seconds = (minutes - floor(minutes)) * 60;
    
    NSArray *timeInfo = [[NSArray alloc] initWithObjects:
                         [NSNumber numberWithFloat:hours],
                         [NSNumber numberWithFloat:minutes],
                         [NSNumber numberWithFloat:seconds], nil];
    
    return timeInfo;
}

- (Base40Bytes)convert_from_base40:(unsigned long)valueToConvert
{
	Base40Bytes bb;
	double bTwo, bOne, bZero;
	double val, firstFraction, secondFraction;
	
	val = (float)valueToConvert;

	firstFraction = val / 1600.0f;
	bTwo = floor(firstFraction);

	secondFraction = (firstFraction - bTwo);
	secondFraction *= 40;
	bOne = floor((float)secondFraction);
    
	bZero = val - (bTwo * 1600) - (bOne * 40);
    
	bb.B2 = (unsigned char)bTwo;
	bb.B1 = (unsigned char)bOne;
	bb.B0 = (unsigned char)bZero;
	
	return bb;
}

#pragma mark - Utility methods for block-related manipulations
- (CLLocationCoordinate2D)convertBlockNumberToLatLon:(uint32_t)blockNumber
{
    CLLocationCoordinate2D ll;
    
    // Determine lat/lon for block number (Northern hemisphere only)
    unsigned int numberOfBlocksInRing = 450;
    unsigned int completeRings = 0;
    double blocksInPartialRing = 0.0;
    double fracRings = 0.0, fracLat = 0.0, fracLon = 0.0;
    double blockLongitudeWidth = 48.0;
    double blockLatitudeHeight = 4.0;
    
    if (blockNumber < 405000)
    {
        fracRings = (double)blockNumber / numberOfBlocksInRing;
        completeRings = (unsigned int)floor(fracRings);
        blocksInPartialRing = (fracRings - completeRings) * numberOfBlocksInRing;
        
        fracLat = (double)completeRings * blockLatitudeHeight / 60.0;
        ll.latitude = fracLat;
        
        fracLon = blocksInPartialRing * blockLongitudeWidth / 60.0;
        if (fracLon > 180.0) fracLon = fracLon - 360.0;
        ll.longitude = fracLon;
        
    }
    else
    {
        fracRings = (double)(blockNumber-405000) / numberOfBlocksInRing;
        completeRings = (unsigned int)floor(fracRings);
        blocksInPartialRing = (fracRings - completeRings) * numberOfBlocksInRing;
        
        fracLat = (double)completeRings * blockLatitudeHeight / 60.0;
        ll.latitude = fracLat + 60.0;
        
        fracLon = blocksInPartialRing * blockLongitudeWidth / 60.0;
        if (fracLon > 180.0) fracLon = fracLon - 360.0;
        ll.longitude = fracLon;
    }
    
    return ll;
}  // convertBlockNumberToLatLon

- (uint32_t)convertLatLongToBlockNumber:(double)lat longitude:(double)lon
{
	/*
     Blocks contain 4 rows of bins, each row being 1 minute of latitude high.
     Blocks contain 32 columns of bins:
     a) from the equator up to, but not including, +60 degrees (latitude):
     i) a block is 48 minutes wide (longitude)
     ii) there are 450 blocks in a ring
     b) at 60 degrees (latitude) and above:
     i) a block become 96 minutes wide (longitude)
     ii) there are 225 block in a ring
     c) same rules for the southern hemisphere
     */
	
	double ring=0.0, blockInRow=0.0, blockWidth=0.0, multiplier=0.0;
	uint32_t block;
	
	if (lon < 0) lon += 360;
	
	// Determine ring number
	ring = fabs(lat) * 60.0 / 4.0;
	ring = floorf(ring);
    
	if (fabs(lat) < 60)
	{
		blockWidth = 48.0;
		multiplier = 1.0;
	}
	else
	{
		blockWidth = 96.0;
		multiplier = 2.0;
	}
    
	blockInRow = fabs(lon) * 60.0 / blockWidth;
	blockInRow = floorf(blockInRow);
    
	block = (uint32_t)((450.0 * ring) + (blockInRow * multiplier));
	if (lat < 0) block *= -1;
	
	return block;
}   // convertLatLongToBlockNumber

- (NSArray *)getBlockListForUpperLeftLat:(double)upperLeftLat
                            UpperLeftLon:(double)upperLeftLon
                           lowerRightLat:(double)lowerRightLat
                           lowerRightLon:(double)lowerRightLon
{
    // This method returns all of the blocks occupying a square region of area. It does
    // not return the FISB block data, just the block numbers.
    
    // !!! NOTE: This method does not return valid values if the block crosses the
    // prime meridian or the equator!!!
    
    unsigned int upperLeftBlockNum, lowerLeftBlockNum, upperRightBlockNum, lowerRightBlockNum;
    
    upperLeftBlockNum = [self convertLatLongToBlockNumber:upperLeftLat longitude:upperLeftLon];
    upperRightBlockNum = [self convertLatLongToBlockNumber:upperLeftLat longitude:lowerRightLon];
    lowerLeftBlockNum = [self convertLatLongToBlockNumber:lowerRightLat longitude:upperLeftLon];
    lowerRightBlockNum = [self convertLatLongToBlockNumber:lowerRightLat longitude:lowerRightLon];
    
    unsigned int numBlocksHorizontally = lowerRightBlockNum - lowerLeftBlockNum + 1;
    unsigned int numBlocksVertically = ((upperRightBlockNum - lowerRightBlockNum) / 450) + 1;
    
    NSMutableArray *arrayOfSlices = [[NSMutableArray alloc] init];
    
    for (int i=0; i<numBlocksVertically; i++)
    {
        NSMutableArray *horizSlice = [[NSMutableArray alloc] initWithCapacity:numBlocksHorizontally];
        for (int j=0; j<numBlocksHorizontally; j++)
        {
            unsigned int block = (lowerLeftBlockNum + j) + (450 * i);
            [horizSlice addObject:[NSNumber numberWithUnsignedInt:block]];
        }
        [arrayOfSlices addObject:horizSlice];
        horizSlice = nil;
    }
    
    return ((NSArray *)arrayOfSlices);
}  // getBlockListForUpperLeftLat

- (NSArray *)getBlockListForCoordinateRegion:(MKCoordinateRegion)coordinateRegion
{
    double upperLeftLat = coordinateRegion.center.latitude + (0.5 * coordinateRegion.span.latitudeDelta);
    double upperLeftLon = coordinateRegion.center.longitude - (0.5 * coordinateRegion.span.longitudeDelta);
    double lowerRightLat = coordinateRegion.center.latitude - (0.5 * coordinateRegion.span.latitudeDelta);
    double lowerRightLon = coordinateRegion.center.longitude + (0.5 * coordinateRegion.span.longitudeDelta);

    return [self getBlockListForUpperLeftLat:upperLeftLat UpperLeftLon:upperLeftLon lowerRightLat:lowerRightLat lowerRightLon:lowerRightLon];
}  // getBlockListForCoordinateRegion

- (NSArray *)getBlockListFromLowestBlock:(NSNumber *)lowestBlock toHighestBlock:(NSNumber *)highestBlock
{
    CLLocationCoordinate2D lowerLeft = [self convertBlockNumberToLatLon:[lowestBlock unsignedLongValue]];
    CLLocationCoordinate2D upperRight = [self convertBlockNumberToLatLon:[highestBlock unsignedLongValue]];
    
    double upperLeftLat = upperRight.latitude;
    double upperLeftLon = lowerLeft.longitude;
    double lowerRightLat = lowerLeft.latitude;
    double lowerRightLon = upperRight.longitude;
    
    return [self getBlockListForUpperLeftLat:upperLeftLat UpperLeftLon:upperLeftLon lowerRightLat:lowerRightLat lowerRightLon:lowerRightLon];
} // getBlockListFromLowestBlock

- (void)clearNEXRADData
{
    [self.fisbBlocks removeAllObjects];
}

- (void)clearCONUSData
{
    [self.fisbCONUSBlocks removeAllObjects];
}

#pragma mark - Methods to Generate Weather Images
- (void)expandCONUSDataBlocks:(NSData *)conusBlockData isEmptyBlock:(BOOL)blockIsEmpty blockNumber:(uint32_t)blockNumber scaleFactor:(uint8_t)conusResolution
{
    uint8_t blockData[kNumberOfBinsPerBlock];
    [conusBlockData getBytes:&blockData[0] length:kNumberOfBinsPerBlock];
    
    /* The ADS-B block numbering system is used for both NEXRAD and CONUS NEXRAD data. However, the CONUS data
     is at a different resolution. Instead of CONUS blocks being numerically sequential (e.g 450, 451, 452),
     they increment according to what the block number would be for that lat/lon. The broadcast block number
     is also the NW corner of the block.
     
     For example, in the case where the CONUS resolution is Medium (which it typically is), a CONUS block occupies
     an area 5x "wider" and 5x "higher" than the high resolution NEXRAD data. So the broadcast CONUS block numbers
     will increment by 5:
     (second row) 3600, 3605, 3610, etc.
     (first row)  1800, 1805, 1810, etc.
     When the CONUS resolution is Low, the scale factor is 9 instead of 5.
     When the CONUS resolution is High, the scale factor is 1 - the same resolution as NEXRAD.
     
     Here, the CONUS data is "expanded" by the scale factor and stored at the same resolution as the
     NEXRAD data. This makes building an image from NEXRAD and CONUS data much simpler when asking for an
     image of weather with an arbitrary lat/lon start point and arbitrary width/height.
     
     The method below expands the broadcast CONUS block data so that it is stored in memory at the same
     resolution as the broadcast NEXRAD block data.
     */
    
    unsigned int expandedBlockWidth = kNumberOfBinsPerRow * conusResolution;
    unsigned int expandedBlockHeight = kNumberOfRowsPerBlock * conusResolution;
    unsigned int i,j,k,n;
    
    if (blockIsEmpty)
    {
        uint32_t highResolutionBlockNumber=0;
        
        for (i=0; i < conusResolution; i++)
        {
            for (j=0; j<conusResolution; j++)
            {
                // calcuate the corresponding bin number
                highResolutionBlockNumber = (blockNumber + (conusResolution - i - 1) * 450) + j - (450 * (conusResolution-1));
                //printf("Block number: %d. High res block number = %d.\n", blockNumber, highResolutionBlockNumber);
                [self.fisbCONUSBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedInt:highResolutionBlockNumber]];
            }
        }
        
        return;
    }
    
    // Create a "super block" to contain the expanded data
    unsigned char superBlock[expandedBlockWidth * expandedBlockHeight]; // e.g. (4 * 5) * (32 * 5)
    unsigned int superBlockPtr = 0;
    
    // Expand the CONUS data into the superblock
    n=0;
    for (i=0; i<kNumberOfBinsPerBlock; i++)
    {
        for (j=0; j<conusResolution; j++)
        {
            for (k=0; k<conusResolution; k++)
            {
                superBlock[superBlockPtr + j + (k * expandedBlockWidth)] = blockData[i];
            }
            n++;
        }
        superBlockPtr += conusResolution;
        
        if (n == expandedBlockWidth)
        {
            n=0;
            superBlockPtr += (expandedBlockWidth * (conusResolution-1));
        }
    }
    
    // Break the superblock up into small blocks so that it is the same format as NEXRAD.
    // This routine starts with the "upper left" of the super block data, and moves right and downward.
    // But bin numbering scheme starts at the lower left and increments upward & rightward.
    
    uint8_t highResDataBlock[kNumberOfBinsPerBlock];
    uint32_t highResolutionBlockNumber=0;
    unsigned int offset = 0;
    
    for (i=0; i < conusResolution; i++)
    {
        superBlockPtr = i * (4 * expandedBlockWidth);
        offset=0;
        
        for (j=0; j<conusResolution; j++)
        {
            // calcuate the corresponding bin number
            highResolutionBlockNumber = (blockNumber + (conusResolution - i - 1) * 450) + j - (450 * (conusResolution-1));
            //NSLog(@"Block number: %d. High res block number = %d.", blockNumber, highResolutionBlockNumber);
            
            // slice up the super block to create the smaller blocks
            memcpy(&highResDataBlock[0], &superBlock[superBlockPtr + offset], kNumberOfBinsPerRow);
            memcpy(&highResDataBlock[0 + kNumberOfBinsPerRow], &superBlock[superBlockPtr + offset + expandedBlockWidth], kNumberOfBinsPerRow);
            memcpy(&highResDataBlock[0 + (2*kNumberOfBinsPerRow)], &superBlock[superBlockPtr + offset + (2*expandedBlockWidth)], kNumberOfBinsPerRow);
            memcpy(&highResDataBlock[0 + (3*kNumberOfBinsPerRow)], &superBlock[superBlockPtr + offset + (3*expandedBlockWidth)], kNumberOfBinsPerRow);
            
            NSData *highResolutionBlockData = [NSData dataWithBytes:highResDataBlock length:kNumberOfBinsPerBlock];
            
            offset += kNumberOfBinsPerRow;
            
            // store the expanded data
            [self.fisbCONUSBlocks setObject:highResolutionBlockData forKey:[NSNumber numberWithUnsignedInt:highResolutionBlockNumber]];
        }
    }
}  //expandCONUSDataBlocks

-(char)getBinValueAtLatitude:(double)latitude Longitude:(double)longitude useCONUS:(BOOL)useCONUS
{
    double targetLat = 0.0, targetLon = 0.0;
    double deltaLat = 0.0, deltaLon = 0.0;
    double binWidth = 0.0, binHeight = 0.0;
    uint32_t blockNumberContainingTargetCoord = 0;
    unsigned char rowOfBinWithTargetCoord = 0, colOfBinWithTargetCoord = 0;
    CLLocationCoordinate2D coordsOfBlockNumber;
    
    targetLat = latitude;
    targetLon = longitude;
    
    blockNumberContainingTargetCoord = [self convertLatLongToBlockNumber:targetLat longitude:targetLon];
    coordsOfBlockNumber = [self convertBlockNumberToLatLon:blockNumberContainingTargetCoord];
    
    deltaLat = fabs(targetLat - coordsOfBlockNumber.latitude);
    deltaLon = fabs(targetLon - coordsOfBlockNumber.longitude);
    
    if (blockNumberContainingTargetCoord < 405000) binWidth = 48.0/32.0/60.0;
    else binWidth = 96.0 / 32.0 / 60.0;
    binHeight = 1.0/60.0;
    
    colOfBinWithTargetCoord = (unsigned char)floorf(deltaLon / binWidth);
    rowOfBinWithTargetCoord = 3 - (unsigned char)floorf(deltaLat / binHeight);
    
    // Get the bin value for that location
    char binValue;
    unsigned char position = (32 * rowOfBinWithTargetCoord) + colOfBinWithTargetCoord;
    
    NSData *blockData;
    if (useCONUS) blockData = [self.fisbCONUSBlocks objectForKey:[NSNumber numberWithUnsignedInt:blockNumberContainingTargetCoord]];
    else blockData = [self.fisbBlocks objectForKey:[NSNumber numberWithUnsignedInt:blockNumberContainingTargetCoord]];

    if ([blockData isMemberOfClass:[NSNull class]]) binValue = 0;
    else if (blockData == NULL) binValue = -1;
    else [blockData getBytes:&binValue range:NSMakeRange(position, 1)];
    
    return binValue;
    
} // getBinValueAtLatitude

- (UIImage *)createWeatherImageFromSlices:(NSArray *)blocksInRegion usingCONUSdata:(BOOL)useCONUS
{
    // determine overall image size
    NSUInteger rows = [blocksInRegion count];
    NSUInteger cols = [(NSArray *)[blocksInRegion objectAtIndex:0] count];
    NSUInteger imageHeight = rows * 4;        // 4 rows per block
    NSUInteger imageWidth = cols * 32 * 4;       // 32 bins per block, 4 bytes per pixel
    
    // Figure out the dimension of the image to be created
    float imageWidthInPixels = (float)(cols * 32.0);
    float imageHeightInPixels = (float)(rows * 4.0);
    
    NSUInteger binIndex;
    NSUInteger rowOfImage = 0;
    NSUInteger colOfImage = 0;
    NSUInteger currentRowOfBlocks = 0;
    NSUInteger m = 0;
    char intensity;
    BOOL noWeatherData = NO;
    
    unsigned char red, blue, green, alpha;
    unsigned long imageDataSize = imageHeight * imageWidth;
    NSMutableData *imageData = [NSMutableData dataWithLength:imageDataSize];
    NSNumber *blockNumber;
    NSData *blockData;
    char binData[128];
    unsigned char pixel[4];
    
    // Remember, first row in the array is the bottom row of slices. But this method builds an image
    // from the top down. So reverse the order in which you pull the slices so the image gets built
    // right side up.
    NSEnumerator *reversedEnumerator = [blocksInRegion reverseObjectEnumerator];
    for (NSArray *rowOfBlocks in reversedEnumerator)              // grab a row of blocks
    {
        for (int r=0; r<[rowOfBlocks count]; r++)     // go through each block in the grabbed row
        {
            blockNumber = [rowOfBlocks objectAtIndex:r];
            
            if (useCONUS) blockData = [self.fisbCONUSBlocks objectForKey:blockNumber];
            else blockData = [self.fisbBlocks objectForKey:blockNumber];
            
            if ([blockData isKindOfClass:[NSNull class]]) noWeatherData = YES;
            else if (blockData == NULL) noWeatherData = YES;
            else
            {
                [blockData getBytes:&binData length:128];
                noWeatherData = NO;
                
                for (binIndex = 0; binIndex<128; binIndex++)
                {
                    rowOfImage = (int)(binIndex/32) + (currentRowOfBlocks * 4);
                    colOfImage = ((binIndex - ((int)(binIndex/32) * 32)) * 4) + (128 * r);
                    
                    m = (rowOfImage * imageWidth) + colOfImage;
                    
                    if (noWeatherData) intensity = 0;      // Change to -1 to make empty or null blocks visible.
                    else intensity = binData[binIndex];
                    
                    switch (intensity)
                    {
                        case -1:    // gray color for any blocks without data, just for debugging purposes
                            red = 100;
                            blue = 100;
                            green = 100;
                            alpha = 100;
                            break;
                            
                        case 0: // clear
                            red = 255;
                            green = 0;
                            blue = 0;
                            alpha = 255;
                            break;
                            
                        case 1: // clear
                            red = 0;
                            green = 0;
                            blue = 0;
                            alpha = 0;
                            break;
                            
                        case 2: // bright green
                            red = 0;
                            green = 255;
                            blue = 0;
                            alpha = 255;
                            break;
                            
                        case 3: // bright yellow
                            red = 255;
                            green = 255;
                            blue = 0;
                            alpha = 255;
                            break;
                            
                        case 4: // bright red
                            red = 255;
                            green = 0;
                            blue = 0;
                            alpha = 255;
                            break;
                            
                        case 5: // dark red
                            red = 150;
                            green = 0;
                            blue = 0;
                            alpha = 255;
                            break;
                            
                        case 6: // bright magenta
                            red = 255;
                            green = 0;
                            blue = 255;
                            alpha = 255;
                            break;
                            
                        case 7: // dark magenta
                            red = 153;
                            green = 102;
                            blue = 204;
                            alpha = 255;
                            break;
                            
                        default:
                            break;
                    }
                    
                    pixel[0] = red;
                    pixel[1] = green;
                    pixel[2] = blue;
                    pixel[3] = alpha;
                    
                    [imageData replaceBytesInRange:NSMakeRange(m, 4) withBytes:pixel length:4];
                }
            }
        }
        currentRowOfBlocks++;
    }
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CIContext *context = [CIContext contextWithOptions:nil];
    CIImage *ciImage = [CIImage imageWithBitmapData:imageData bytesPerRow:imageWidth size:CGSizeMake((CGFloat)imageWidthInPixels, (CGFloat)imageHeightInPixels) format:kCIFormatRGBA8 colorSpace:colorSpace];
    CGImageRef cgImageRef = [context createCGImage:ciImage fromRect:[ciImage extent]];
    UIImage *bitmapImage = [UIImage imageWithCGImage:cgImageRef];
    CGColorSpaceRelease(colorSpace);
    
    return bitmapImage;
} // createWeatherImageFromSlices

- (UIImage *)createImageWithDataArray:(NSArray *)arrayOfWeatherBlocks
{
    // Determine the range of blocks holding the weather data
    NSArray *sortedBlocks = [arrayOfWeatherBlocks sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"self" ascending:YES]]];
    NSNumber *lowestNumber = [sortedBlocks objectAtIndex:0];
    NSNumber *highestNumber = [sortedBlocks objectAtIndex:([sortedBlocks count]-1)];
    NSArray *blocksInRegion = [self getBlockListFromLowestBlock:lowestNumber toHighestBlock:highestNumber];
    
    return [self createWeatherImageFromSlices:blocksInRegion usingCONUSdata:NO];
}

- (UIImage *)createRegionalWeatherImage
{
    NSArray *nexradBlocks = [self.fisbBlocks allKeys];
    
    if ([nexradBlocks count] == 0) return nil;
    
    // Determine the range of blocks holding the weather data
    NSArray *sortedBlocks = [nexradBlocks sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"self" ascending:YES]]];
    NSNumber *lowestNumber = [sortedBlocks objectAtIndex:0];
    NSNumber *highestNumber = [sortedBlocks objectAtIndex:([sortedBlocks count]-1)];
    NSArray *blocksInRegion = [self getBlockListFromLowestBlock:lowestNumber toHighestBlock:highestNumber];
    
    return [self createWeatherImageFromSlices:blocksInRegion usingCONUSdata:NO];
}

- (UIImage *)createNationalWeatherImage
{
    if ([self.fisbCONUSBlocks count] == 0) return nil;
    
    CLLocationCoordinate2D lowerLeft, upperRight;
    upperRight.latitude = self.conusWeatherCoverageArea.center.latitude + (self.conusWeatherCoverageArea.span.latitudeDelta / 2.0);
    upperRight.longitude = self.conusWeatherCoverageArea.center.longitude + (self.conusWeatherCoverageArea.span.longitudeDelta / 2.0);
    lowerLeft.latitude = self.conusWeatherCoverageArea.center.latitude - (self.conusWeatherCoverageArea.span.latitudeDelta / 2.0);
    lowerLeft.longitude = self.conusWeatherCoverageArea.center.longitude - (self.conusWeatherCoverageArea.span.longitudeDelta / 2.0);
    
    NSArray *blocksInRegion = [self getBlockListForUpperLeftLat:upperRight.latitude UpperLeftLon:lowerLeft.longitude lowerRightLat:lowerLeft.latitude lowerRightLon:upperRight.longitude];
    
    return [self createWeatherImageFromSlices:blocksInRegion usingCONUSdata:YES];
}

- (UIImage *)createWeatherImageWithRegion:(MKCoordinateRegion)coordinateRegion fromCONUSData:(BOOL)useCONUS
{
    // Get a list of all of the blocks in the region
    NSArray *blocksInRegion = [self getBlockListForCoordinateRegion:coordinateRegion];
    
    return [self createWeatherImageFromSlices:blocksInRegion usingCONUSdata:useCONUS];
}

- (UIImage *)createNEXRADImageWithUpperLeftLat:(double)upperLeftLat
                                  upperLeftLon:(double)upperLeftLon
                                 lowerRightLat:(double)lowerRightLat
                                 lowerRightLon:(double)lowerRightLon
                                         width:(uint32_t)width
                                        height:(uint32_t)height
                                      useCONUS:(BOOL)useCONUS
{
    double latitudeSampleSize = 0.0, longitudeSampleSize = 0.0;
    int x,y;
    char binValue;
    LatLon samplingPoint;
    
    unsigned char red, blue, green, alpha;
    uint32_t bytesPerRow = width * 4; // 4 bytes per pixel (RGBA)
    uint32_t dataSize = bytesPerRow * height;
    uint32_t ptr=0;
    NSMutableData *imageData = [NSMutableData dataWithLength:(NSUInteger)dataSize];
    
    latitudeSampleSize = fabs(upperLeftLat - lowerRightLat) / height;
    longitudeSampleSize = fabs(upperLeftLon - lowerRightLon) / width;
    
    for (y=height-1; y>=0; y--)
    {
        for (x=0; x<width; x++)
        {
            samplingPoint.latitude = lowerRightLat + (y * latitudeSampleSize);
            samplingPoint.longitude = upperLeftLon + (x * longitudeSampleSize);
            binValue = [self getBinValueAtLatitude:samplingPoint.latitude Longitude:samplingPoint.longitude useCONUS:useCONUS];
            
            switch (binValue)
            {
                case -1:
                    red = 0;
                    green = 0;
                    blue = 0;
                    alpha = 0;
                    break;
                    
                case 0: // clear
                    red = 0;
                    green = 0;
                    blue = 0;
                    alpha = 0;
                    break;
                    
                case 1: // bright green
                    red = 0;
                    green = 255;
                    blue = 0;
                    alpha = 255;
                    break;
                    
                case 2: // bright yellow
                    red = 255;
                    green = 255;
                    blue = 0;
                    alpha = 255;
                    break;
                    
                case 3: // bright red
                    red = 255;
                    green = 0;
                    blue = 0;
                    alpha = 255;
                    break;
                    
                case 4: // dark red
                    red = 150;
                    green = 0;
                    blue = 0;
                    alpha = 255;
                    break;
                    
                case 5: // bright magenta
                    red = 255;
                    green = 0;
                    blue = 255;
                    alpha = 255;
                    break;
                    
                case 6: // dark magenta
                    red = 153;
                    green = 102;
                    blue = 204;
                    alpha = 255;
                    break;
                    
                case 7: // cyan
                    red = 0;
                    green = 167;
                    blue = 240;
                    alpha = 255;
                    break;
                    
                default:
                    //red = 100;
                    //green = 100;
                    //blue = 100;
                    //alpha = 255;
                    break;
            }
            
            [imageData replaceBytesInRange:NSMakeRange(ptr, 1) withBytes:red];
            [imageData replaceBytesInRange:NSMakeRange(ptr+1, 1) withBytes:green];
            [imageData replaceBytesInRange:NSMakeRange(ptr+2, 1) withBytes:blue];
            [imageData replaceBytesInRange:NSMakeRange(ptr+3, 1) withBytes:alpha];
            
            ptr += 4;
        }
    }
    
    UIImage *bitmapImage = [UIImage imageWithData:imageData];
    
    return bitmapImage;
} // createNEXRADImageWithUpperLeftLat

- (UIImage *)createNEXRADImageWithUpperLeftLat:(double)upperLeftLat
                                  upperLeftLon:(double)upperLeftLon
                                      deltaLat:(double)deltaLat
                                      deltaLon:(double)deltaLon
                                         width:(uint32_t)width
                                        height:(uint32_t)height
                                      useCONUS:(BOOL)useCONUS
{
    return [self createNEXRADImageWithUpperLeftLat:upperLeftLat
                                      upperLeftLon:upperLeftLon
                                     lowerRightLat:(upperLeftLat + deltaLat)
                                     lowerRightLon:(upperLeftLon - deltaLon)
                                             width:width
                                            height:height
                                          useCONUS:useCONUS];
}

- (BOOL)calculateNEXRADDataCoverageAreaUsingCONUS:(BOOL)useCONUS
{
    NSArray *nexradBlocks;
    
    if (useCONUS) nexradBlocks = [self.fisbCONUSBlocks allKeys];
    else nexradBlocks = [self.fisbBlocks allKeys];
    
    if ([nexradBlocks count] == 0) return NO;
    
    // find the min lat, min lon, max lat and max lon as well as the largest block number
    CLLocationCoordinate2D ll;
    float minLat=90.0, minLon=0.0, maxLat=0.0, maxLon=-180.0;
    uint highestBlockNumber=0;
    
    for (NSNumber *blockNum in nexradBlocks)
    {
        //NSLog(@"Block number: %d", [blockNum integerValue]);
        if ([blockNum unsignedIntegerValue] > highestBlockNumber) highestBlockNumber = [blockNum unsignedIntegerValue];
        ll = [self convertBlockNumberToLatLon:[blockNum unsignedIntValue]];
        if (ll.latitude < minLat) minLat = ll.latitude;
        if (ll.latitude > maxLat) maxLat = ll.latitude;
        if (ll.longitude < minLon) minLon = ll.longitude;
        if (ll.longitude > maxLon) maxLon = ll.longitude; //NSLog(@"lon = %f.  maxLon = %f.  block = %d", ll.longitude, maxLon, [blockNum unsignedIntegerValue]); }
    }
    
    CLLocationCoordinate2D lowerLeft, upperRight;
    lowerLeft.latitude = minLat;
    lowerLeft.longitude = minLon;
    upperRight.latitude = maxLat;
    upperRight.longitude = maxLon;
    
    //NSLog(@"lower left: %.4f,%.4f.  upper right: %.4f,%.4f.  max block: %d", lowerLeft.latitude, lowerLeft.longitude, upperRight.latitude, upperRight.longitude, highestBlockNumber);
    
    // The lat/lon for a block is the lower left corner. So need to add the width & height of one block to the upper right value.
    if (highestBlockNumber < 405000)  // below 60 degree latitude
    {
        upperRight.latitude += (4.0 / 60.0);
        upperRight.longitude += (48.0 / 60.0);
    }
    else       // at or above 60 degrees latitude
    {
        upperRight.latitude += (4.0 / 60.0);
        upperRight.longitude += (96.0 / 60.0);
    }
    
    double deltaLat = upperRight.latitude - lowerLeft.latitude;
    double deltaLon = upperRight.longitude - lowerLeft.longitude;
    
    CLLocationCoordinate2D centerPoint = CLLocationCoordinate2DMake(upperRight.latitude - (0.5 * deltaLat), upperRight.longitude - (0.5 * deltaLon));
    
    MKCoordinateSpan span = MKCoordinateSpanMake(deltaLat, deltaLon);
    
    if (useCONUS) self.conusWeatherCoverageArea = MKCoordinateRegionMake(centerPoint, span);
    else self.nexradWeatherCoverageArea = MKCoordinateRegionMake(centerPoint, span);
    
    return YES;
    
} // calculateNEXRADDataCoverageArea

#pragma mark - ADS-B Payload decoding methods
- (void)decodeGraphicalOverlay:(uint8_t *)data length:(uint16_t)recordLength
{
    // This method is untested.
    // KNOWN BUGS:
    // 1) the lat/lon values are not correct. They need to be run through the
    // caclulateDegrees method, but with a different constant for precision.
    
    return;
    
    uint8_t ptr = 0;
    
    if (DEBUG_GRAPHICAL_OVERLAY_DATA) printf("\n\n\n");
    
    // bytes[0-2] are the DLAC encoded location identifier
    uint8_t locationString[5];
    uint32_t locationID = (data[0] << 24) | (data[1] << 16) | (data[2] << 8);
    locationString[0] = dlacCode[(locationID & 0xFC000000) >> 26];
    locationString[1] = dlacCode[(locationID & 0x03F00000) >> 20];
    locationString[2] = dlacCode[(locationID & 0x000FC000) >> 14];
    locationString[3] = dlacCode[(locationID & 0x00003F00) >> 8];
    locationString[4] = '\0';
    
    // byte[3] is the record reference point
    uint8_t recordRef = data[3];
    
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"Location ID for message:");
        NSLog(@"byte[0] = 0x%2X = %@.", data[0], [self bytes_to_binary:data[0] numberOfBytes:1]);
        NSLog(@"byte[1] = 0x%2X = %@.", data[1], [self bytes_to_binary:data[1] numberOfBytes:1]);
        NSLog(@"byte[2] = 0x%2X = %@.", data[2], [self bytes_to_binary:data[2] numberOfBytes:1]);
        NSLog(@"Location ID = %s\n\n", locationString);
        NSLog(@"byte[3] = 0x%2X = %@", data[3], [self bytes_to_binary:data[3] numberOfBytes:1]);
        NSLog(@"recordRef = data[3] = %d.\n\n", recordRef);
    }
    
    // the overlay record length is the next 10 bits
    uint16_t overlayRecordLength = (data[4] << 2) | ((data[5] & 0xC0) >> 6);
    
    // the report number is the next 14 bits
    uint16_t reportNumber = ((data[5] & 0x3F) << 8) | data[6];
    
    // the report year is the next 7 bits
    uint8_t reportYear = data[7] >> 1;
    
    // the overlay record identifier is the next 4 bits
    uint8_t overlayRecordID = ((data[8] & 0x1E) >> 1) + 1;
    
    // the object label flag is the next bit
    BOOL objectLabelFlag = (data[8] & 0x01);
    
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"Full graphical overlay message:");
        for (int v=4; v<overlayRecordLength+4; v++)
        {
            NSLog(@"byte[%3d] = %@", v, [self bytes_to_binary:data[v] numberOfBytes:1]);
        }
        NSLog(@" ");
        
        NSLog(@"byte[4] = 0x%2X = %@", data[4], [self bytes_to_binary:data[4] numberOfBytes:1]);
        NSLog(@"byte[5] = 0x%2X = %@", data[5], [self bytes_to_binary:data[5] numberOfBytes:1]);
        NSLog(@"overlayRecordLength = (data[4] << 2) | ((data[5] & 0xC0) >> 6) = %d.\n", overlayRecordLength);
        NSLog(@"byte[5] = 0x%2X = %@", data[5], [self bytes_to_binary:data[5] numberOfBytes:1]);
        NSLog(@"byte[6] = 0x%2X = %@", data[6], [self bytes_to_binary:data[6] numberOfBytes:1]);
        NSLog(@"reportNumber = ((data[5] & 0x3F) << 8) | data[6] = %d.\n", reportNumber);
        NSLog(@"byte[7] = 0x%2X = %@", data[7], [self bytes_to_binary:data[7] numberOfBytes:1]);
        NSLog(@"reportYear = data[7] >> 1 = %d.\n", reportYear);
        NSLog(@"byte[8] = 0x%2X = %@", data[8], [self bytes_to_binary:data[8] numberOfBytes:1]);
        NSLog(@"overlayRecordID = ((data[8] & 0x1E) >> 1) + 1 = %d.\n", overlayRecordID);
        NSLog(@"byte[8] = 0x%2X = %@", data[8], [self bytes_to_binary:data[8] numberOfBytes:1]);
        NSLog(@"objectLabelFlag = data[8] & 0x01 = %d.\n", objectLabelFlag);
        NSLog(@" ");
    }
    
    uint16_t decimalObjectLabel;
    uint8_t dlacObjectLabel[12];
    
    if (objectLabelFlag)
    {
        // object label is DLAC-encoded alphanumeric text 4 to 12 bytes long
        
        uint32_t holder;
        
        char i=9, j=0;
        
        // since the length of a DLAC encoding can be variable, we need to start using a
        // variable to index through the data
        ptr = i;
        
        while (i<19)
        {
            holder = (data[i] << 24) | (data[i+1] << 16) | (data[i+2] << 8);
            
            dlacObjectLabel[j] = dlacCode[(holder & 0xFC000000) >> 26];
            //NSLog(@"j = %d. first char = %c", j, dlacObjectLabel[j]);
            if (dlacObjectLabel[j] == 0x03)
            {
                dlacObjectLabel[j] = '\0';
                break;
            }
            
            dlacObjectLabel[++j] = dlacCode[(holder & 0x03F00000) >> 20];
            //NSLog(@"j = %d. second char = %c", j, dlacObjectLabel[j]);
            if (dlacObjectLabel[j] == 0x03)
            {
                dlacObjectLabel[j] = '\0';
                break;
            }
            
            dlacObjectLabel[++j] = dlacCode[(holder & 0x000FC000) >> 14];
            //NSLog(@"j = %d. third char = %c", j, dlacObjectLabel[j]);
            if (dlacObjectLabel[j] == 0x03)
            {
                dlacObjectLabel[j] = '\0';
                break;
            }
            
            dlacObjectLabel[++j] = dlacCode[(holder & 0x00003F00) >> 8];
            //NSLog(@"j = %d. fourth char = %c", j, dlacObjectLabel[j]);
            if (dlacObjectLabel[j] == 0x03)
            {
                dlacObjectLabel[j] = '\0';
                break;
            }
            
            ptr += 3;
            i += 3;
            j++;
        }
        if (DEBUG_GRAPHICAL_OVERLAY_DATA)
        {
            NSLog(@"Decoded DLAC text is %s", dlacObjectLabel);
            NSLog(@" ");
        }
        ptr = 18;
        
    }
    else    // object label is a unsigned integer that is a database index
    {
        decimalObjectLabel = (data[9] << 8) | data[10];
        
        // advance pointer index to the next element: object type
        ptr = 11;
        
        if (DEBUG_GRAPHICAL_OVERLAY_DATA)
        {
            NSLog(@"byte[9] = 0x%2X = %@.", data[9], [self bytes_to_binary:data[9] numberOfBytes:1]);
            NSLog(@"byte[10] = 0x%2X = %@.", data[10], [self bytes_to_binary:data[10] numberOfBytes:1]);
            NSLog(@"decimalObjectLabel = (data[9] << 8) | data[10) = %d.", decimalObjectLabel);
            NSLog(@"ptr = %d.\n", ptr);
            NSLog(@" ");
        }
    }
    
    // next 3 bits are object element flag, object qualifier flag and object parameter flag
    BOOL objectElementFlag = (data[ptr] & 0x80) >> 7;
    BOOL objectQualifierFlag = (data[ptr] & 0x40) >> 6;
    BOOL objectParameterFlag = (data[ptr] & 0x20) >> 5;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
        NSLog(@"objectElementFlag = (data[%d] & 0x80) >>  = %d.\n", ptr, objectElementFlag);
        NSLog(@"objectQualifierFlag = (data[%d] & 0x40) >> 6 = %d.\n", ptr, objectQualifierFlag);
        NSLog(@"objectParameterFlag = (data[%d] & 0x20) >> 5 = %d.\n", ptr, objectParameterFlag);
        NSLog(@" ");
    }
    
    // next 5 bits are the object element
    uint8_t objectElement = nil;
    if (objectElementFlag) objectElement = data[ptr] * 0x1F;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
        NSLog(@"objectElement = data[%d] & 0x1F = %d.\n", ptr, objectElement);
        NSLog(@" ");
    }
    ptr++;
    
    // next 4 bits are the object type
    uint8_t objectType = (data[ptr] & 0xF0) >> 4;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
        NSLog(@"objectType = (data[%d] & 0xF0) >> 4 = %d.\n", ptr, objectType);
        NSLog(@" ");
    }
    
    // next 4 bits are object status
    uint8_t objectStatus = data[ptr] & 0x0F;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
        NSLog(@"objectStatus = data[%d] & 0x0F = %d.\n", ptr, objectStatus);
        NSLog(@" ");
    }
    ptr++;
    
    // if the objectElementFlag is non-zero, the next 3 bytes are the object qualifier
    uint32_t objectQualifier = 0;
    if (objectElementFlag)
    {
        objectQualifier = (data[ptr] << 16) | (data[ptr+1] << 8) | data[ptr+2];
        ptr += 3;
    }
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        if (objectElementFlag == 0) NSLog(@"No object qualifier because object element flag is 0");
        else
        {
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
            NSLog(@"objectQualifier = (data[%d] << 16) | (data[%d] << 8) | data[%d] = %d.\n", ptr, ptr+1, ptr+2, objectQualifier);
            NSLog(@" ");
        }
    }
    
    // if the objectParameter flag is nonzero, the next 5 bits are the object parameter type
    uint8_t objectParameterType = 0;
    uint16_t objectParameterValue = 0;
    if (objectParameterFlag) objectParameterType = (data[ptr] & 0xF8) >> 3;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        if (objectParameterFlag == 0) NSLog(@"No object parameter type since object element flag is 0.");
        else
        {
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
            NSLog(@"objectParameterType = (data[%d] & 0xF8) >> 3 = %d.\n", ptr, objectParameterType);
            NSLog(@" ");
        }
    }
    
    // if the objectParameter flag is nonzero, the next 11 bits are object parameter value
    if (objectParameterFlag)
    {
        objectParameterValue = ((data[ptr] & 0x07) << 8) | data[ptr+1];
        ptr += 2;
    }
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        if (objectParameterFlag == 0) NSLog(@"No object parameter value since object element flag is 0");
        else
        {
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
            NSLog(@"objectParameterValue = ((data[%d] & 0x07) << 8) | data[%d] = %d.\n", ptr, ptr+1, objectParameterValue);
            NSLog(@" ");
        }
    }
    
    // next 2 bits are the record applicability options:
    //  0 = no date/time used
    //  1 = start time only
    //  2 = end time only
    //  3 = both start & end times
    uint8_t recordApplicabilityOptions = (data[ptr] & 0xC0) >> 6;
    
    // next 2 bits are the date/time format:
    //  0 = no date/time used.
    //  1 = month, day, hours & mins
    //  2 = days, hours & mins
    //  3 = hours & mins
    uint8_t dateTimeFormat = (data[ptr] & 0x30) >> 4;
    
    // next 4 bits are the overlay geometry options
    uint8_t overlayGeometryOptions = data[ptr] & 0x0F;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
        NSLog(@"recordApplicabilityOptions = (data[ptr] & 0x0C) >> 2 = %d.\n", recordApplicabilityOptions);
        NSLog(@"dateTimeFormat = data[ptr] & 0x03 = %d.\n", dateTimeFormat);
        NSLog(@"overlayGeometryOptions = data[ptr] & 0x0F = %d.\n", overlayGeometryOptions);
        NSLog(@" ");
    }
    ptr++;
    
    // next 2 bits are the overlay operator
    uint8_t overlayOperator = (data[ptr] & 0xC0) >> 6;
    
    // next 6 bits are the overlay verticies count
    uint8_t overlayVerticiesCount = (data[ptr] & 0x3F) + 1;
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
        NSLog(@"overlayOperator = data[ptr] & 0xC0 = %d.\n", overlayOperator);
        NSLog(@"overlayVerticiesCount = (data[ptr] & 0x3F) + 1 = %d.\n", overlayVerticiesCount);
        NSLog(@" ");
    }
    ptr++;
    
    // The next byte(s) depend on the value of recordApplicabilityOptions. If =1, then start
    // times are given. if =2, end times are given. If =3, both times are given.
    // Within the start and end times, the month, day and minute can optionally be present,
    // as dictated by the dateTimeFormat value. The hour value does not depend upon dateTimeFormat
    // and will always be present.
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@" ");
        NSLog(@"Determining start time options:");
        NSLog(@"recordApplicabilityOptions = 0x%2X.", recordApplicabilityOptions);
        NSLog(@"dateTimeFormat = 0x%2X.", dateTimeFormat);
    }
    
    uint8_t startMonth = nil, endMonth = nil;
    uint8_t startDay = nil, endDay = nil;
    uint8_t startHour = nil, endHour = nil;
    uint8_t startMin = nil, endMin = nil;
    
    if (recordApplicabilityOptions & 0x01)    // recordApplicabilityOptions == 1 or == 3
    {
        if (dateTimeFormat == 1)
        {
            startMonth = data[ptr];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA)
            {
                NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                NSLog(@"startMonth = data[ptr] = %d.\n", startMonth);
            }
            ptr++;
        }
        else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No start month given.");
        
        if (dateTimeFormat ^ 0x3)
        {
            startDay = data[ptr];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA)
            {
                NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                NSLog(@"startDay = data[ptr] = %d.\n", startDay);
            }
            ptr++;
        }
        else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No start day given.");
        
        // next byte is a mandatory start hour field
        startHour = data[ptr];
        if (DEBUG_GRAPHICAL_OVERLAY_DATA)
        {
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
            NSLog(@"startHour = data[ptr] = %d.\n", startHour);
        }
        ptr++;
        
        // next byte is optional start min field
        if (dateTimeFormat > 0)
        {
            startMin = data[ptr];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA)
            {
                NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                NSLog(@"startMin = data[ptr] = %d.\n", startMin);
            }
            ptr++;
        }
        else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No start day given.");
        
    }
    else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No start times given.");
    
    // if the recordApplicabilityOptions indicates a valid end time, the next byte(s) are
    // the end month or the end time (or both, using two bytes) depending upon the value
    // of dateTimeFormat
    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
    {
        NSLog(@" ");
        NSLog(@"Determining end time options:");
        NSLog(@"recordApplicabilityOptions = 0x%2X.", recordApplicabilityOptions);
        NSLog(@"dateTimeFormat = 0x%2X.", dateTimeFormat);
    }
    
    if (recordApplicabilityOptions & 0x01)    // recordApplicabilityOptions == 1 or == 3
    {
        if (dateTimeFormat == 1)
        {
            endMonth = data[ptr];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA)
            {
                NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                NSLog(@"endMonth = data[ptr] = %d.\n", endMonth);
            }
            ptr++;
        }
        else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No start month given.");
        
        if (dateTimeFormat ^ 0x3)
        {
            endDay = data[ptr];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA)
            {
                NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                NSLog(@"endDay = data[ptr] = %d.\n", endDay);
            }
            ptr++;
        }
        else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No start day given.");
        
        // next byte is a mandatory start hour field
        endHour = data[ptr];
        if (DEBUG_GRAPHICAL_OVERLAY_DATA)
        {
            NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
            NSLog(@"endHour = data[ptr] = %d.\n", endHour);
        }
        ptr++;
        
        // next byte is optional minute field
        if (dateTimeFormat > 0)
        {
            endMin = data[ptr];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA)
            {
                NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                NSLog(@"endMin = data[ptr] = %d.\n", endMin);
            }
            ptr++;
        }
        else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No end minute given.");
    }
    else if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"No end times given.");
    
    // remaining bytes are the overlay verticies list
    NSMutableArray *overlayVerticies = nil;
    
    int8_t  zTop=0;
    int16_t x=0, y=0, zBottom=0;
    
    uint8_t alpha=0;
    uint16_t rx=0, ry=0, rlat=0, rlon=0;
    
    int32_t latBottom=0, lonBottom=0, latTop=0, lonTop=0;
    uint32_t lowbits=0, highbits=0;
    
    if (overlayVerticiesCount)
    {
        overlayVerticies = [[NSMutableArray alloc] initWithCapacity:overlayVerticiesCount];
        for (int j=0; j<overlayVerticiesCount; j++)
        {
            [overlayVerticies addObject:[NSNumber numberWithInt:data[ptr]]];
            if (DEBUG_GRAPHICAL_OVERLAY_DATA) NSLog(@"Vertex #%d = %d.", j, data[ptr]);
            
            // The format of the vertex data depends on the type of vertex.
            // All verticies will be for the same type of geometry, i.e. all verticies in the
            // list will be part of the same shape. There is one exception to this:
            // ellipse, circular prism and point geometries are all singular. So if the
            // geometry specifies one of these three types, and there are multiple verticies,
            // then it should be interpreted as multiple shapes.
            // Verticies are ordered from start to end in a counter-clockwise fashion.
            // Start and end verticies are only the same for a closed polygon.
            // A line will be represented by a polygon-type geometry with only two verticies.
            
            switch (overlayGeometryOptions) {
                case 1:  // low resolution 2D polygon
                    highbits = 0; lowbits = 0;
                    
                    // first 12 bits are x coordinate (Cartesian, magnetic) w/ 5m resolution
                    highbits = (data[ptr] << 4);
                    lowbits = (data[ptr+1] & 0xF0) >> 4;
                    x = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 12 bits are y coordinate
                    highbits = (data[ptr+1] & 0x0F) << 8;
                    lowbits = data[ptr+2];
                    y = highbits | lowbits;
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"x = %@.\ny = %@.",
                              [self bytes_to_binary:x numberOfBytes:2],
                              [self bytes_to_binary:y numberOfBytes:2]);
                        NSLog(@"Low Res 2D polygon vertex %d: x=%d, y=%d.", j, x*5, y*5);
                    }
                    
                    ptr += 3;
                    
                    break;
                    
                case 2:  // high resolution 3D polygon
                    highbits = 0; lowbits = 0;
                    
                    // first 14 bits are x coordinate (Cartesian, magnetic) w/ 1.25m resolution
                    highbits = (data[ptr] << 6);
                    lowbits = (data[ptr+1] & 0xFC) >> 2;
                    x = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 14 bits are y coordinate
                    highbits = (data[ptr+1] & 0x03) << 12;
                    lowbits = (data[ptr+3] & 0xF0) >> 4;
                    y = highbits | (data[ptr+2] << 4) | lowbits;
                    
                    // next 4 bits are z coordinate (AGL) w/ 3m resolution
                    zBottom = data[ptr+3] & 0x0F;
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"x = %@.\ny = %@.\nz = %@.",
                              [self bytes_to_binary:x numberOfBytes:2],
                              [self bytes_to_binary:y numberOfBytes:2],
                              [self bytes_to_binary:zBottom numberOfBytes:2]);
                        NSLog(@"high resolution 3D polygon vertex %d: x=%.6f, y=%.6f. z=%d.",
                              j, (float)x * 1.25, (float)y * 1.25, zBottom*3);
                    }
                    
                    ptr += 4;
                    
                    break;
                    
                case 3:  // extended range 3D polygon (MSL)
                    highbits = 0; lowbits = 0;
                    
                    // first 19 bits are longitude w/ 0.000687 degree resolution
                    lowbits = (data[ptr+2] & 0xE0) >> 5;
                    lonBottom =  (data[ptr] << 11) | (data[ptr+1] << 3) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 19 bits are latitude
                    highbits = (data[ptr+2] & 0x1F) << 14;
                    lowbits = (data[ptr+4] & 0xFC) >> 2;
                    latBottom = highbits | data[ptr+3] << 6 | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 10 bits are z coordinate w/ 100 ft resolution
                    highbits = (data[ptr+4] & 0x03) << 8;
                    zBottom = highbits | data[ptr+5];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"\nlat = %@.\nlon = %@.\nz = %@.",
                              [self bytes_to_binary:latBottom numberOfBytes:4],
                              [self bytes_to_binary:lonBottom numberOfBytes:4],
                              [self bytes_to_binary:zBottom numberOfBytes:2]);
                        NSLog(@"extended range 3D polygon (MSL) vertex %d: lat=%.6f. lon=%.6f. z=%d.",
                              j, (float)latBottom * 0.000687, (float)lonBottom * 0.000687, zBottom*100);
                    }
                    
                    ptr += 6;
                    
                    break;
                    
                case 4:  // extended range 3D polygon (AGL)
                    highbits = 0; lowbits = 0;
                    
                    // first 19 bits are longitude w/ 0.000687 degree resolution
                    lowbits = (data[ptr+2] & 0xE0) >> 5;
                    lonBottom =  (data[ptr] << 11) | (data[ptr+1] << 3) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 19 bits are latitude
                    highbits = (data[ptr+2] & 0x1F) << 14;
                    lowbits = (data[ptr+4] & 0xFC) >> 2;
                    latBottom = highbits | data[ptr+3] << 6 | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 10 bits are z coordinate w/ 100 ft resolution
                    highbits = (data[ptr+4] & 0x03) << 8;
                    zBottom = highbits | data[ptr+5];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"\nlat = %@.\nlon = %@.\nz = %@.",
                              [self bytes_to_binary:latBottom numberOfBytes:4],
                              [self bytes_to_binary:lonBottom numberOfBytes:4],
                              [self bytes_to_binary:zBottom numberOfBytes:2]);
                        NSLog(@"extended range 3D polygon (AGL) vertex %d: lat=%.6f. lon=%.6f. z=%d.",
                              j, (float)latBottom * 0.000687, (float)lonBottom * 0.000687, zBottom*100);
                    }
                    
                    ptr += 6;
                    
                    break;
                    
                case 5:  // low resolution 2D ellipse
                    highbits = 0; lowbits = 0;
                    
                    // first 12 bits are x coordinate (Cartesian, magnetic) w/ 5m resolution
                    highbits = (data[ptr] << 4);
                    lowbits = (data[ptr+1] & 0xF0) >> 4;
                    x = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 12 bits are y coordinate
                    highbits = (data[ptr+1] & 0x0F) << 8;
                    lowbits = data[ptr+2];
                    y = highbits | lowbits;
                    
                    // next 8 bits are radius in the x direction w/ 5m resolution
                    rx = data[ptr+3];
                    
                    // next 8 bits are radius in the y direction w/ 5m resolution
                    ry = data[ptr+4];
                    
                    // next 8 bits are the alpha dimension w/ 1 deg resolution
                    alpha = data[ptr+5];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"x = %@.\ny = %@.\nrx = %@.\nry = %@.\nalpha = %@.",
                              [self bytes_to_binary:x numberOfBytes:2],
                              [self bytes_to_binary:y numberOfBytes:2],
                              [self bytes_to_binary:rx numberOfBytes:1],
                              [self bytes_to_binary:ry numberOfBytes:1],
                              [self bytes_to_binary:alpha numberOfBytes:1]);
                        NSLog(@"Low Res 2D ellipse vertex %d: x=%d, y=%d. rx=%d. ry=%d. alpha=%d.",
                              j, x*5, y*5, rx*5, ry*5, alpha);
                    }
                    
                    ptr += 6;
                    
                    break;
                    
                case 6:  // high resolution 3D ellipse
                    highbits = 0; lowbits = 0;
                    
                    // first 14 bits are x coordinate (Cartesian, magnetic) w/ 1.25m resolution
                    highbits = (data[ptr] << 6);
                    lowbits = (data[ptr+1] & 0xFC) >> 2;
                    x = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 14 bits are y coordinate
                    highbits = (data[ptr+1] & 0x03) << 12;
                    lowbits = (data[ptr+3] & 0xF0) >> 4;
                    y = highbits | (data[ptr+2] << 4) | lowbits;
                    
                    // next 4 bits are the bottom z coordinate (AGL) w/ 3m resolution
                    zBottom = data[ptr+3] & 0x0F;
                    
                    // next 4 bits are the top z coordinate (AGL) w/ 3m resolution
                    zTop = (data[ptr+4] & 0x0F) >> 4;
                    
                    highbits = 0; lowbits = 0;
                    // next 10 bits are radius in the x direction w/ 1.25m resolution
                    highbits = (data[ptr+4] & 0x0F) << 6;
                    lowbits = (data[ptr+5] & 0x0FC) >> 2;
                    rx = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 10 bits are radius in the y direction w/ 1.25m resolution
                    highbits = (data[ptr+5] & 0x03) << 8;
                    ry = highbits | data[6];
                    
                    // next 8 bits are the alpha dimension w/ 1 deg resolution
                    alpha = data[ptr+7];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+6, data[ptr+6], [self bytes_to_binary:data[ptr+6] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+7, data[ptr+7], [self bytes_to_binary:data[ptr+7] numberOfBytes:1]);
                        NSLog(@"\nx = %@.\ny = %@.\nz(bottom) = %@.\nz(top) = %@.",
                              [self bytes_to_binary:x numberOfBytes:2],
                              [self bytes_to_binary:y numberOfBytes:2],
                              [self bytes_to_binary:zBottom numberOfBytes:2],
                              [self bytes_to_binary:zTop numberOfBytes:1]);
                        NSLog(@"\nrx = %@.\nry = %@.\nalpha = %@.",
                              [self bytes_to_binary:rx numberOfBytes:2],
                              [self bytes_to_binary:ry numberOfBytes:2],
                              [self bytes_to_binary:alpha numberOfBytes:1]);
                        NSLog(@"high resolution 3D ellipse vertex %d:\n x=%.6f, y=%.6f. z(bottom) = %d. z(top = %d. rx=%.6f. ry=%.6f. alpha=%d.", j, (float)x * 1.25,
                              (float)y * 1.25, zBottom * 3, zTop * 3, (float)rx * 1.25,
                              (float)ry * 1.25, alpha);
                    }
                    
                    ptr += 8;
                    
                    break;
                    
                case 7:  // extended range circular prism (MSL)
                    highbits = 0; lowbits = 0;
                    
                    // first 18 bits are bottom longitude w/ 0.001373 degree resolution
                    lowbits = (data[ptr+2] & 0xC0) >> 6;
                    lonBottom = (data[ptr] << 10) | (data[ptr+1] << 2)| lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 18 bits are bottom latitude
                    highbits = (data[ptr+2] & 0x3F) << 12;
                    lowbits = (data[ptr+4] & 0xF0) >> 4;
                    latBottom = highbits | (data[ptr+3] << 4) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 18 bits are the top longitude
                    highbits = (data[ptr+4] & 0xF0) << 10;
                    lowbits = (data[ptr+6] & 0xFC) >> 2;
                    lonTop = highbits | (data[ptr+5] << 2) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 18 bits are the top longitude
                    highbits = (data[ptr+6] & 0x03) << 16;
                    latTop = highbits | (data[ptr+7] << 8) | data[ptr+8];
                    
                    // next 7 bits are bottom z coordinate w/ 5 foot resolution
                    zBottom = data[ptr+9] >> 1;
                    
                    highbits = 0; lowbits = 0;
                    // next 7 bits are top z coordinate w/ 500 foot resolution
                    highbits = (data[ptr+9] & 0x01) << 6;
                    lowbits = (data[ptr+10] & 0xFC) >> 2;
                    zTop = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 9 bits are the longitudinal radius w/ 0.2 nmi resolution
                    highbits = (data[ptr+10] & 0x03) << 7;
                    lowbits = (data[ptr+11] & 0xFE) >> 1;
                    rlon = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 9 bits are the latitudinal radius
                    highbits = (data[ptr+11] & 0x01) << 8;
                    rlat = highbits | data[ptr+12];
                    
                    // last 8 bits are the alpha dimension w/ 1 deg resolution
                    alpha = data[ptr+13];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+6, data[ptr+6], [self bytes_to_binary:data[ptr+6] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+7, data[ptr+7], [self bytes_to_binary:data[ptr+7] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+8, data[ptr+8], [self bytes_to_binary:data[ptr+8] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+9, data[ptr+9], [self bytes_to_binary:data[ptr+9] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+10, data[ptr+10], [self bytes_to_binary:data[ptr+10] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+11, data[ptr+11], [self bytes_to_binary:data[ptr+11] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+12, data[ptr+12], [self bytes_to_binary:data[ptr+12] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+13, data[ptr+13], [self bytes_to_binary:data[ptr+13] numberOfBytes:1]);
                        NSLog(@"\nlat(bottom) = %@.\nlon(bottom)y = %@.\nlat(top) = %@.\nlon(top) = %@.",
                              [self bytes_to_binary:latBottom numberOfBytes:3],
                              [self bytes_to_binary:lonBottom numberOfBytes:3],
                              [self bytes_to_binary:latTop numberOfBytes:3],
                              [self bytes_to_binary:lonTop numberOfBytes:3]);
                        NSLog(@"\nz(bottom) = %@.\nz(top) = %@.\nr(lat) = %@.\nr(lon) = %@.\nalpha = %@",
                              [self bytes_to_binary:zBottom numberOfBytes:1],
                              [self bytes_to_binary:zTop numberOfBytes:1],
                              [self bytes_to_binary:rlat numberOfBytes:2],
                              [self bytes_to_binary:rlon numberOfBytes:2],
                              [self bytes_to_binary:alpha numberOfBytes:1]);
                        NSLog(@"extended range circular prism (MSL) vertex %d:\nlat(bottom) = %.6f. lon(bottom) = %.6f. lat(top) = %.6f. lon(top) = %.6f.\nz(bottom) = %d. z(top) = %d. r(lat) = %.6f. r(lon) = %.6f. alpha=%d.", j, (float)latBottom * 0.001373,
                              (float)lonBottom * 0.001373, (float)latTop * 0.001373,
                              (float)latTop * 0.001373, zBottom * 5, zTop * 500, rlat * 0.2,
                              rlon * 0.2, alpha);
                    }
                    
                    ptr += 14;
                    
                    break;
                    
                case 8:  // extended range circular prism (AGL)
                    highbits = 0; lowbits = 0;
                    
                    // first 18 bits are bottom longitude w/ 0.001373 degree resolution
                    lowbits = (data[ptr+2] & 0xC0) >> 6;
                    lonBottom = (data[ptr] << 10) | (data[ptr+1] << 2)| lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 18 bits are bottom latitude
                    highbits = (data[ptr+2] & 0x3F) << 12;
                    lowbits = (data[ptr+4] & 0xF0) >> 4;
                    latBottom = highbits | (data[ptr+3] << 4) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 18 bits are the top longitude
                    highbits = (data[ptr+4] & 0xF0) << 10;
                    lowbits = (data[ptr+6] & 0xFC) >> 2;
                    lonTop = highbits | (data[ptr+5] << 2) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 18 bits are the top longitude
                    highbits = (data[ptr+6] & 0x03) << 16;
                    latTop = highbits | (data[ptr+7] << 8) | data[ptr+8];
                    
                    // next 7 bits are bottom z coordinate w/ 5 foot resolution
                    zBottom = data[ptr+9] >> 1;
                    
                    highbits = 0; lowbits = 0;
                    // next 7 bits are top z coordinate w/ 500 foot resolution
                    highbits = (data[ptr+9] & 0x01) << 6;
                    lowbits = (data[ptr+10] & 0xFC) >> 2;
                    zTop = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 9 bits are the longitudinal radius w/ 0.2 nmi resolution
                    highbits = (data[ptr+10] & 0x03) << 7;
                    lowbits = (data[ptr+11] & 0xFE) >> 1;
                    rlon = highbits | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 9 bits are the latitudinal radius
                    highbits = (data[ptr+11] & 0x01) << 8;
                    rlat = highbits | data[ptr+12];
                    
                    // last 8 bits are the alpha dimension w/ 1 deg resolution
                    alpha = data[ptr+13];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+6, data[ptr+6], [self bytes_to_binary:data[ptr+6] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+7, data[ptr+7], [self bytes_to_binary:data[ptr+7] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+8, data[ptr+8], [self bytes_to_binary:data[ptr+8] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+9, data[ptr+9], [self bytes_to_binary:data[ptr+9] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+10, data[ptr+10], [self bytes_to_binary:data[ptr+10] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+11, data[ptr+11], [self bytes_to_binary:data[ptr+11] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+12, data[ptr+12], [self bytes_to_binary:data[ptr+12] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+13, data[ptr+13], [self bytes_to_binary:data[ptr+13] numberOfBytes:1]);
                        NSLog(@"\nlat(bottom) = %@.\nlon(bottom)y = %@.\nlat(top) = %@.\nlon(top) = %@.",
                              [self bytes_to_binary:latBottom numberOfBytes:3],
                              [self bytes_to_binary:lonBottom numberOfBytes:3],
                              [self bytes_to_binary:latTop numberOfBytes:3],
                              [self bytes_to_binary:lonTop numberOfBytes:3]);
                        NSLog(@"\nz(bottom) = %@.\nz(top) = %@.\nr(lat) = %@.\nr(lon) = %@.\nalpha = %@",
                              [self bytes_to_binary:zBottom numberOfBytes:1],
                              [self bytes_to_binary:zTop numberOfBytes:1],
                              [self bytes_to_binary:rlat numberOfBytes:2],
                              [self bytes_to_binary:rlon numberOfBytes:2],
                              [self bytes_to_binary:alpha numberOfBytes:1]);
                        NSLog(@"extended range circular prism (AGL) vertex %d:\nlat(bottom) = %.6f. lon(bottom) = %.6f. lat(top) = %.6f. lon(top) = %.6f.\nz(bottom) = %d. z(top) = %d. r(lat) = %.6f. r(lon) = %.6f. alpha=%d.", j, (float)latBottom * 0.001373,
                              (float)lonBottom * 0.001373, (float)latTop * 0.001373,
                              (float)latTop * 0.001373, zBottom * 5, zTop * 500, rlat * 0.2,
                              rlon * 0.2, alpha);
                    }
                    
                    ptr += 14;
                    
                    break;
                    
                case 9:  // extended range 3D point (AGL)
                    highbits = 0; lowbits = 0;
                    
                    // first 19 bits are longitude w/ 0.000687 degree resolution
                    lowbits = (data[ptr+2] & 0xE0) >> 5;
                    lonBottom =  (data[ptr] << 11) | (data[ptr+1] << 3) | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 19 bits are latitude
                    highbits = (data[ptr+2] & 0x1F) << 14;
                    lowbits = (data[ptr+4] & 0xFC) >> 2;
                    latBottom = highbits | data[ptr+3] << 6 | lowbits;
                    
                    highbits = 0; lowbits = 0;
                    // next 10 bits are z coordinate w/ 100 ft resolution
                    highbits = (data[ptr+4] & 0x03) << 8;
                    zBottom = highbits | data[ptr+5];
                    
                    if (DEBUG_GRAPHICAL_OVERLAY_DATA)
                    {
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr, data[ptr], [self bytes_to_binary:data[ptr] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+1, data[ptr+1], [self bytes_to_binary:data[ptr+1] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+2, data[ptr+2], [self bytes_to_binary:data[ptr+2] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+3, data[ptr+3], [self bytes_to_binary:data[ptr+3] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+4, data[ptr+4], [self bytes_to_binary:data[ptr+4] numberOfBytes:1]);
                        NSLog(@"ptr = %d. byte[ptr] = 0x%2X = %@.", ptr+5, data[ptr+5], [self bytes_to_binary:data[ptr+5] numberOfBytes:1]);
                        NSLog(@"\nlat = %@.\nlon = %@.\nz = %@.",
                              [self bytes_to_binary:latBottom numberOfBytes:4],
                              [self bytes_to_binary:lonBottom numberOfBytes:4],
                              [self bytes_to_binary:zBottom numberOfBytes:2]);
                        NSLog(@"extended range 3D point (AGL) vertex %d: lon=%.6f, lat=%.6f. z=%d.",
                              j, (float)lonBottom * 0.000687, (float)latBottom * 0.000687, zBottom*100);
                    }
                    break;
                    
                default:
                    break;
                    
            }
            
        }
    }
    
    if (DEBUG_GRAPHICAL_OVERLAY_SUMMARY)
    {
        printf("\n");
        NSLog(@"Location ID = %s", locationString);
        
        NSLog(@"Overlay Record Length = %d", overlayRecordLength);
        
        NSLog(@"Report identifier info:");
        NSLog(@"   reportNumber = %d", reportNumber);
        NSLog(@"   reportYear = %d", reportYear);
        if ((reportNumber == 0) && (reportYear == 0)) NSLog(@"   --> no associated text record.");
        
        NSLog(@"Overlay Record ID is %d", overlayRecordID);
        
        if (objectLabelFlag == 0)
        {
            if (decimalObjectLabel == 0) NSLog(@"Object refers to an unlabeled airspace");
            else if (decimalObjectLabel == 1) NSLog(@"Object references the airport as a whole");
            else NSLog(@"Object label is %d", decimalObjectLabel);
        }
        else NSLog(@"Object label is %s", dlacObjectLabel);
        
        switch (objectType) {
            case 0:
                NSLog(@"Object type is: Airport / Heliport");
                break;
                
            case 1:
                NSLog(@"Object type is: Runway / Helipad");
                break;
                
            case 2:
                NSLog(@"Object type is: Taxiway");
                break;
                
            case 3:
                NSLog(@"Object type is: Apron");
                break;
                
            case 4:
                NSLog(@"Object type is: Frequency Area");
                break;
                
            case 5:
                NSLog(@"Object type is: Signage");
                break;
                
            case 6:
                NSLog(@"Object type is: Approach lighting");
                break;
                
            case 7:
                NSLog(@"Object type is: Airport lighting");
                break;
                
            case 8:
                NSLog(@"Object type is: Obstruction");
                break;
                
            case 9:
                NSLog(@"Object type is: Construction area");
                break;
                
            case 10:
                NSLog(@"Object type is: Communication equipment");
                break;
                
            case 11:
                NSLog(@"Object type is: Navigation equipment");
                break;
                
            case 12:
                NSLog(@"Object type is: Surveillance equipment");
                break;
                
            case 13:
                NSLog(@"Object type is: Weather equipment");
                break;
                
            case 14:
                NSLog(@"Object type is: Airspace");
                break;
                
            default:
                NSLog(@"Object type is: reserved value");
                break;
        }
        
        if (objectElementFlag == 0) NSLog(@"  --> No additional element information.");
        else
        {
            if (objectType == 1)    // runway
            {
                if (objectElement == 0) NSLog(@"  --> Full runway");
                if (objectElement == 1) NSLog(@"  --> Touchdown (first third)");
                if (objectElement == 2) NSLog(@"  --> Midpoint (middle third)");
                if (objectElement == 3) NSLog(@"  --> Rollout (last third)");
                if (objectElement == 4) NSLog(@"  --> Threshold");
                if (objectElement == 5) NSLog(@"  --> Runway intersection");
                if (objectElement == 6) NSLog(@"  --> Runway shoulder");
                if (objectElement == 7) NSLog(@"  --> Stopway");
                if (objectElement == 8) NSLog(@"  --> Clearway");
                if (objectElement == 9) NSLog(@"  --> Touchdown/liftoff area (helipad/heliport)");
                if (objectElement == 10) NSLog(@"  --> Final approach and takeoff area (helipad/heliport)");
                if (objectElement == 11) NSLog(@"  --> Marking");
                if (objectElement == 12) NSLog(@"  --> LAHSO line");
                if (objectElement == 13) NSLog(@"  --> RAS - arrival");
                if (objectElement == 14) NSLog(@"  --> RAS departure");
                if (objectElement > 14) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 2)   // taxiway
            {
                if (objectElement == 0) NSLog(@"  --> Taxiway segment");
                if (objectElement == 1) NSLog(@"  --> Taxiway shoulder");
                if (objectElement == 2) NSLog(@"  --> Guidance line");
                if (objectElement == 3) NSLog(@"  --> Intersection marking");
                if (objectElement == 4) NSLog(@"  --> Holding position (Stopbar)");
                if (objectElement == 5) NSLog(@"  --> Exit line");
                if (objectElement == 6) NSLog(@"  --> Penalty box");
                if (objectElement > 6) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 3)   // apron
            {
                if (objectElement == 0) NSLog(@"  --> Apron area");
                if (objectElement == 1) NSLog(@"  --> Parking stand area");
                if (objectElement == 2) NSLog(@"  --> Stand guidance line");
                if (objectElement == 3) NSLog(@"  --> Parking stand location");
                if (objectElement == 4) NSLog(@"  --> Deicing stand");
                if (objectElement > 4) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 5)   // signage
            {
                if (objectElement == 0) NSLog(@"  --> Approach lights");
                if (objectElement == 1) NSLog(@"  --> Sequence flashing lights");
                if (objectElement == 2) NSLog(@"  --> Runway alignment indicator lights");
                if (objectElement == 3) NSLog(@"  --> Runway lead in lights");
                if (objectElement == 4) NSLog(@"  --> VASI");
                if (objectElement == 5) NSLog(@"  --> PAPI");
                if (objectElement > 5) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 7)    // airport lighting
            {
                if (objectElement == 0) NSLog(@"  --> All airport lighting");
                if (objectElement == 1) NSLog(@"  --> Threshold lights");
                if (objectElement == 2) NSLog(@"  --> Touchdown zone lights");
                if (objectElement == 3) NSLog(@"  --> Stop bar lights");
                if (objectElement == 4) NSLog(@"  --> Runway end identifier lights");
                if (objectElement == 5) NSLog(@"  --> Runway turnoff lights");
                if (objectElement == 6) NSLog(@"  --> Runway edge lights");
                if (objectElement == 7) NSLog(@"  --> Runway centerline lights");
                if (objectElement == 8) NSLog(@"  --> Runway remaining lights");
                if (objectElement == 9) NSLog(@"  --> Taxiway centerline lights");
                if (objectElement == 10) NSLog(@"  --> Taxiway edge lights");
                if (objectElement == 11) NSLog(@"  --> Rotating beacon");
                if (objectElement > 11) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 8)   // obstruction object
            {
                if (objectElement == 0) NSLog(@"  --> Unknown");
                if (objectElement == 1) NSLog(@"  --> Tower");
                if (objectElement == 2) NSLog(@"  --> Crane");
                if (objectElement == 3) NSLog(@"  --> Vehicle");
                if (objectElement > 3) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 10)   // communication equipment
            {
                if (objectElement == 0) NSLog(@"  --> Airport terminal info service");
                if (objectElement == 1) NSLog(@"  --> Remote communication outlet");
                if (objectElement == 2) NSLog(@"  --> VOR Voice");
                if (objectElement == 3) NSLog(@"  --> Remote transmitter/receiver");
                if (objectElement == 4) NSLog(@"  --> Local airport advisory");
                if (objectElement == 5) NSLog(@"  --> Remote communication air/ground facility");
                if (objectElement == 6) NSLog(@"  --> HIWAS");
                if (objectElement > 6) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 11)    // navigation equipment
            {
                if (objectElement == 0) NSLog(@"  --> Outer marker beacon");
                if (objectElement == 1) NSLog(@"  --> Middle marker beacon");
                if (objectElement == 2) NSLog(@"  --> Inner market beacon");
                if (objectElement == 3) NSLog(@"  --> Glide slope/path");
                if (objectElement == 4) NSLog(@"  --> Localizer");
                if (objectElement == 5) NSLog(@"  --> Locator at middle marker");
                if (objectElement == 6) NSLog(@"  --> Locator at outer marker");
                if (objectElement == 7) NSLog(@"  --> DME");
                if (objectElement == 8) NSLog(@"  --> VOR/VORTAC");
                if (objectElement == 9) NSLog(@"  --> MLS Elevation");
                if (objectElement == 10) NSLog(@"  --> MLS Azimuth");
                if (objectElement == 11) NSLog(@"  --> Simplified directional facility");
                if (objectElement == 12) NSLog(@"  --> Localizer direction aid");
                if (objectElement == 13) NSLog(@"  --> LAAS/WAAS");
                if (objectElement == 14) NSLog(@"  --> Non-directional beacon");
                if (objectElement > 14) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 12)   // surveillance equipment
            {
                if (objectElement == 0) NSLog(@"  --> Airport surface surveillance");
                if (objectElement == 1) NSLog(@"  --> Terminal surveillance");
                if (objectElement == 2) NSLog(@"  --> Precision approach radar");
                if (objectElement == 3) NSLog(@"  --> Ground control approach");
                if (objectElement == 4) NSLog(@"  --> Multilateration system");
                if (objectElement == 5) NSLog(@"  --> TIS - addressed");
                if (objectElement == 6) NSLog(@"  --> TIS - broadcast");
                if (objectElement > 6) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 13)   // weather equipment
            {
                if (objectElement == 0) NSLog(@"  --> D-ATIS");
                if (objectElement == 1) NSLog(@"  --> ATIS");
                if (objectElement == 2) NSLog(@"  --> AWOS");
                if (objectElement == 3) NSLog(@"  --> ASOS");
                if (objectElement == 4) NSLog(@"  --> RVR - touchdown");
                if (objectElement == 5) NSLog(@"  --> RVR - midpoint");
                if (objectElement == 6) NSLog(@"  --> RVR - rollout");
                if (objectElement == 7) NSLog(@"  --> Terminal weather info for pilots");
                if (objectElement > 7) NSLog(@"  --> <reserved value for future use>");
            }
            else if (objectType == 14)    // airspace
            {
                if (objectElement == 0) NSLog(@"  --> TFR");
                if (objectElement == 1) NSLog(@"  --> Parachute jumping/sky diving");
                if (objectElement == 2) NSLog(@"  --> TRSA");
                if (objectElement == 3) NSLog(@"  --> Airport advisor area");
                if (objectElement == 4) NSLog(@"  --> VFR flyway");
                if (objectElement == 5) NSLog(@"  --> VFR corridor");
                if (objectElement == 6) NSLog(@"  --> VFR transition route");
                if (objectElement == 7) NSLog(@"  --> Terminal area VFR route");
                if (objectElement == 8) NSLog(@"  --> Prohibited area");
                if (objectElement == 9) NSLog(@"  --> Restricted area");
                if (objectElement == 10) NSLog(@"  --> MOA");
                if (objectElement == 11) NSLog(@"  --> Warning area");
                if (objectElement == 12) NSLog(@"  --> MTR");
                if (objectElement == 13) NSLog(@"  --> ADIZ");
                if (objectElement > 13) NSLog(@"  --> <reserved value for future use>");
            }
        }
        
        switch (objectStatus) {
            case 0:
                NSLog(@"Object status: Closed");
                break;
                
            case 1:
                NSLog(@"Object status: Closed-conditional");
                break;
                
            case 2:
                NSLog(@"Object status: Arrival only");
                break;
                
            case 3:
                NSLog(@"Object status: Departure only");
                break;
                
            case 4:
                NSLog(@"Object status: Displaced");
                break;
                
            case 5:
                NSLog(@"Object status: Braking action");
                break;
                
            case 6:
                NSLog(@"Object status: Obscured/missing");
                break;
                
            case 7:
                NSLog(@"Object status: Unmarked");
                break;
                
            case 8:
                NSLog(@"Object status: Unlighted");
                break;
                
            case 9:
                NSLog(@"Object status: In service");
                break;
                
            case 10:
                NSLog(@"Object status: Inoperative");
                break;
                
            case 11:
                NSLog(@"Object status: Unavailable");
                break;
                
            case 12:
                NSLog(@"Object status: Surface condition");
                break;
                
            case 13:
                NSLog(@"Object status: Reduced");
                break;
                
            case 14:
                NSLog(@"Object status: Unsafe");
                break;
                
            case 15:
                NSLog(@"Object status: In effect");
                break;
                
            default:
                break;
        }
        
        if (objectQualifierFlag == 0) NSLog(@"No qualifier information");
        else
        {
            if (objectQualifier & 0x01) NSLog(@"  --> Medium");
            else if (objectQualifier & 0x02) NSLog(@"  --> Poor");
            else if (objectQualifier & 0x04) NSLog(@"  --> Nil");
            else if (objectQualifier & 0x08) NSLog(@"  --> Personnel and equipment working");
            else if (objectQualifier & 0x10) NSLog(@"  --> Debris, equipment or spill");
            else if (objectQualifier & 0x20) NSLog(@"  --> Water");
            else if (objectQualifier & 0x40) NSLog(@"  --> Ice");
            else if (objectQualifier & 0x80) NSLog(@"  --> Packed snow");
            else if (objectQualifier & 0x100) NSLog(@"  --> Loose snow");
            else if (objectQualifier & 0x200) NSLog(@"  --> Wet snow");
            else if (objectQualifier & 0x400) NSLog(@"  --> Slush");
            else if (objectQualifier & 0x800) NSLog(@"  --> Sand");
            else if (objectQualifier & 0x1000) NSLog(@"  --> Mud");
            else if (objectQualifier & 0x2000) NSLog(@"  --> Crack, ruts or loose pavement");
            else if (objectQualifier & 0x4000) NSLog(@"  --> Frost heave");
            else if (objectQualifier & 0x8000) NSLog(@"  --> Below");
            else if (objectQualifier & 0x10000) NSLog(@"  --> At and below");
            else if (objectQualifier & 0x20000) NSLog(@"  --> Above");
            else if (objectQualifier & 0x40000) NSLog(@"  --> Within (radius of)");
            else if (objectQualifier & 0x80000) NSLog(@"  --> Beyond (radius of)");
            else if (objectQualifier & 0x100000) NSLog(@"  --> <reserved for future use>");
            else if (objectQualifier & 0x200000) NSLog(@"  --> <reserved for future use>");
            else if (objectQualifier & 0x400000) NSLog(@"  --> <reserved for future use>");
            else if (objectQualifier & 0x800000) NSLog(@"  --> <reserved for future use>");
        }
        
        if (objectParameterFlag == 0) NSLog(@"No parameter information");
        else
        {
            if (objectParameterType == 0) NSLog(@"  --> <reserved value>");
            else if (objectParameterType == 1) NSLog(@"  --> Distance: %.2f nm", (float)objectParameterValue * 0.25);
            else if (objectParameterType == 2) NSLog(@"  --> Distance: %.2f sm", (float)objectParameterValue * 0.25);
            else if (objectParameterType == 3) NSLog(@"  --> Length: %d ft", objectParameterValue * 10);
            else if (objectParameterType == 4) NSLog(@"  --> Length: %d m", objectParameterValue * 3);
            else if (objectParameterType == 5) NSLog(@"  --> Height (MSL): %d ft", objectParameterValue * 10);
            else if (objectParameterType == 6) NSLog(@"  --> Height (AGL): %d ft", objectParameterValue * 10);
            else if (objectParameterType == 7) NSLog(@"  --> Height: %d m", objectParameterValue * 3);
            else if (objectParameterType == 8) NSLog(@"  --> Width: %d ft", objectParameterValue * 5);
            else if (objectParameterType == 9) NSLog(@"  --> Width: %d m", objectParameterValue * 3);
            else if (objectParameterType == 10) NSLog(@"  --> Depth: %.2f inches", (float)objectParameterValue * 0.25);
            else if (objectParameterType == 11) NSLog(@"  --> Depth: %d inches", objectParameterValue);
            else if (objectParameterType == 12) NSLog(@"  --> Depth: %d ft", objectParameterValue);
            else if (objectParameterType == 13) NSLog(@"  --> Weight: %d lbs", objectParameterValue * 500);
            else if (objectParameterType == 14) NSLog(@"  --> Friction: %d MU", objectParameterValue);
            else if (objectParameterType == 15) NSLog(@"  --> Direction (magnetic): %d degrees", objectParameterValue);
            else if (objectParameterType == 16) NSLog(@"  --> Direction (true): %d degrees", objectParameterValue);
            else if (objectParameterType == 17) NSLog(@"  --> Visibilty: %d ft", objectParameterValue * 100);
            else if (objectParameterType == 18) NSLog(@"  --> Visibilty: %d m", objectParameterValue * 25);
            else if (objectParameterType == 19) NSLog(@"  --> Visibilty: %.2f sm", (float)objectParameterValue * 0.0625);
            else if (objectParameterType == 20) NSLog(@"  --> Speed: %d kts", objectParameterValue);
            else if (objectParameterType == 21) NSLog(@"  --> Speed: %d kph", objectParameterValue);
            else if (objectParameterType == 22) NSLog(@"  --> Frequency: %.6f MHz", (float)objectParameterValue * .025);
            else if (objectParameterType == 23) NSLog(@"  --> Frequency: %.6f MHz", (float)objectParameterValue * .00833);
            else if (objectParameterType > 23) NSLog(@"  --> <reserved value for future use>");
        }
        
        // print the starting and ending times
        if (recordApplicabilityOptions == 0) NSLog(@"No starting or ending times provided.");
        else
        {
            if ((recordApplicabilityOptions == 1) || (recordApplicabilityOptions == 3))
            {
                if (dateTimeFormat == 0) NSLog(@"No date/time used.");
                else if (dateTimeFormat == 1) NSLog(@"Start time: month = %d. day = %d. time = %2d:%2d (UTC)", startMonth, startDay, startHour, startMin);
                else if (dateTimeFormat == 2) NSLog(@"Start time: day = %d. time = %2d:%2d (UTC)", startDay, startHour, startMin);
                else NSLog(@"Start time: %2d:%2d (UTC)", startHour, startMin);
            }
            
            if ((recordApplicabilityOptions == 2) || (recordApplicabilityOptions == 3))
            {
                if (dateTimeFormat == 0) NSLog(@"No date/time used.");
                else if (dateTimeFormat == 1) NSLog(@"End time: month = %d. day = %d. time = %2d:%2d (UTC)", endMonth, endDay, endHour, endMin);
                else if (dateTimeFormat == 2) NSLog(@"End time: day = %d. time = %2d:%2d (UTC)", endDay, endHour, endMin);
                else NSLog(@"End time: %2d:%2d (UTC)", endHour, endMin);
            }
        }
        
        // print the geometry
        switch (overlayGeometryOptions) {
            case 0:
                NSLog(@"Overlay geometry is: none.");
                break;
                
            case 1:
                NSLog(@"Overlay geometry is: low res 2D polygon.");
                break;
                
            case 2:
                NSLog(@"Overlay geometry is: high res 3D polygon");
                break;
                
            case 3:
                NSLog(@"Overlay geometry is: extended range 3D polygon (MSL)");
                break;
                
            case 4:
                NSLog(@"Overlay geometry is: extended range 3D polygon (AGL)");
                break;
                
            case 5:
                NSLog(@"Overlay geometry is: low res 2D ellipse.");
                break;
                
            case 6:
                NSLog(@"Overlay geometry is: high res 3D ellipse.");
                break;
                
            case 7:
                NSLog(@"Overlay geometry is: extended range circular prism (MSL).");
                break;
                
            case 8:
                NSLog(@"Overlay geometry is: extended range circular prism (AGL).");
                break;
                
            case 9:
                NSLog(@"Overlay geometry is: extended range 3D point.");
                break;
                
            default:
                NSLog(@"Object status is a reserved value");
                break;
        }
        
        if (overlayOperator == 0) NSLog(@"  --> vertex geometries are independent");
        else if (overlayOperator == 1) NSLog(@"  --> vertex geometries should be concatenated.");
        else if (overlayOperator == 2) NSLog(@"  --> vertex geometries should be subtracted/excluded");
        else NSLog(@"  --> <reserved value>");
        
        if (overlayGeometryOptions == 0) NSLog(@"  --> There are no vertices in this overlay");
        else
        {
            //NSNumber *vertex = nil;
            NSLog(@"  --> There are %d vertices in this overlay", overlayVerticiesCount);
            /*
             for (int x=0; x<overlayVerticiesCount; x++)
             {
             vertex = (NSNumber *)[overlayVerticies objectAtIndex:x];
             NSLog(@"vertex %d = 0x%2X = %@.", x+1, [vertex intValue], [self bytes_to_binary:[vertex intValue] numberOfBytes:1]);
             }
             */
        }
    }
}

- (NSMutableDictionary *)decodeAPDU:(uint8_t *)data length:(uint16_t)frameLength
{
    /*
    First two bytes of the iFrame were length and frame type.
    Second 4 bytes were the APDU header info.
    Passed in is the 7th byte in the iFrame, where the payload header starts:
      - The record format is the first 4 bits
      - The product version is the second 4 bits
      - The record count is the third 4 bits
      - The fourth 4 bits are a spare
      - The 9th - 11th bytes are the location identifier
      - The 12th byte is the record reference
      - The record data begins on the 13th byte
    */
    
    if (DEBUG_APDU_PAYLOAD_DATA) [self printBuffer:@"Full message" buffer:data length:frameLength];
    
    NSMutableDictionary *receivedItems = [[NSMutableDictionary alloc] init];
    uint8_t recordFormat = 0;
    uint8_t productVersion = 0;
    uint8_t reportCount = 0;
    uint32_t locationID = 0;
    uint8_t recordRef = 0;
    uint8_t firstChar, secondChar, thirdChar, fourthChar;
    uint16_t textRecordLength = 0;
    uint16_t reportNumber = 0;
    uint8_t reportYear = 0;
    uint8_t reportStatus = 0;

    recordFormat = (data[0] & 0xF0) >> 4;
    productVersion = data[0] & 0x0F;
    
    reportCount = (data[1] & 0xF0) >> 4;

    locationID = (data[2] << 24) | (data[3] << 16) | (data[4] << 8);
    firstChar = (locationID & 0xFC000000) >> 26; 
    secondChar = (locationID & 0x03F00000) >> 20; 
    thirdChar = (locationID & 0x000FC000) >> 14;
    fourthChar = (locationID & 0x00003F00) >> 8;

    recordRef = data[5];
    
    textRecordLength = (data[6] << 8) | data[7];
    
    reportNumber = ((data[8] << 8) | (data[9] & 0xFC)) >> 2;
    uint8_t high, low;
    high = (data[9] & 0x03) << 5;
    low = (data[10] & 0xF8) >> 3;
    reportYear |= high;
    reportYear |= low;
    reportStatus = (data[10] & 0x04) >> 2;
    uint8_t textData[(textRecordLength + 1)];
    
    if (DEBUG_APDU_PAYLOAD_DATA)
    {
        NSLog(@"APDU Payload:");
        printf ("byte 0 = 0x%.2X. byte 1 = 0x%.2X. byte 2 = 0x%.2X. \n", data[0], data[1], data[2]);
        printf ("byte 3 = 0x%.2X. byte 4 = 0x%.2X. byte 5 = 0x%.2X. \n", data[3], data[4], data[5]);
        printf ("Record format = %d. Product version = %d. Record count = %d.\n", recordFormat, productVersion, reportCount);
        printf ("location ID = %d% d %d %d or ", firstChar, secondChar, thirdChar, fourthChar);
        printf ("location ID = %c%c%c%c\n", dlacCode[firstChar], dlacCode[secondChar], dlacCode[thirdChar], dlacCode[fourthChar]);
        printf ("Record Ref = 0x%.2X or %d.\n\n", recordRef, recordRef);
        printf ("byte 6 = 0x%.2X. byte 7 = 0x%.2X. byte 8 = 0x%.2X. \n", data[6], data[7], data[8]);
        printf ("byte 9 = 0x%.2X. byte 10 = 0x%.2X. byte 11 = 0x%.2X. \n", data[9], data[10], data[11]);
        printf ("Text record length = %d. Report # = %d. Report year = %d.\n", textRecordLength, reportNumber, reportYear);
        printf ("Report status = %s.\n\n", (reportStatus)?"active":"cancelled");
    }

    uint8_t dataStartPos = 11;

    for (uint8_t i=0; i<reportCount; i++)
    {
        if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"Decoding record %d...", reportCount);
        
        switch (recordFormat) {
            case 0:
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"Record format is zero: no data in ADPU payload.");
                break;
                
            case 1: {
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"APDU payload is unformatted ASCII.");
                memcpy(textData, &data[dataStartPos], textRecordLength);
                textData[textRecordLength] = '\0';
                NSString *asciiMessage = [[NSString alloc] initWithFormat:@"%s", textData];
                NSNumber *asciiTimestamp = [[NSNumber alloc] initWithDouble:[NSDate timeIntervalSinceReferenceDate]];

                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"Received message: %@.\n\n\n", asciiMessage);
                [receivedItems setObject:asciiMessage forKey:asciiTimestamp];

                break;
                
            }
            case 2: {
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"APDU payload is unformatted DLAC.");
                uint8_t firstChar, secondChar, thirdChar, fourthChar;
                uint32_t holder;
                NSMutableString *decodedDLACText = [[NSMutableString alloc] init];
                
                for (int j=0; j<textRecordLength; j+=3)
                {
                    holder = 0;
                    holder = (data[j+dataStartPos] << 24) | (data[j+1+dataStartPos] << 16) | (data[j+2+dataStartPos] << 8);
                    firstChar = (holder & 0xFC000000) >> 26; 
                    secondChar = (holder & 0x03F00000) >> 20; 
                    thirdChar = (holder & 0x000FC000) >> 14;
                    fourthChar = (holder & 0x00003F00) >> 8;
                    
                    // if one of the characters is zero, that signifies the end of text
                    if (firstChar == 0) j = textRecordLength;
                    else if (secondChar == 0)
                    {
                        [decodedDLACText appendFormat:@"%c", dlacCode[firstChar]];
                        j = textRecordLength;
                    }
                    else if (thirdChar == 0)
                    {
                        [decodedDLACText appendFormat:@"%c%c", dlacCode[firstChar], dlacCode[secondChar]];
                        j = textRecordLength;
                    }
                    else if (fourthChar == 0)
                    {
                        [decodedDLACText appendFormat:@"%c%c%c", dlacCode[firstChar], dlacCode[secondChar], dlacCode[thirdChar]];
                        j = textRecordLength;
                    }
                    else 
                        [decodedDLACText appendFormat:@"%c%c%c%c", dlacCode[firstChar], dlacCode[secondChar],
                         dlacCode[thirdChar], dlacCode[fourthChar]];
                }
                
                NSNumber *dlacTimestamp = [[NSNumber alloc] initWithDouble:[NSDate timeIntervalSinceReferenceDate]];

                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"Received message: %@.\n\n\n", decodedDLACText);
                [receivedItems setObject:decodedDLACText forKey:dlacTimestamp];

                break;
                
            }
            case 3:
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"APDU payload is unformatted DLAC w/ dictionary.\n\n\n");
                break;
                
            case 4:
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"APDU payload is formatted ASN.1/PER text.\n\n\n");
                break;
                
            case 8:
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"APDU payload is a graphical overlay.\n\n\n");
                [self decodeGraphicalOverlay:&data[2] length:frameLength-2];
                break;
                
            default:
                if (DEBUG_APDU_PAYLOAD_DATA) DLog(@"APDU payload is using a reserved format number: %d.\n\n\n", recordFormat);
                break;
        }

        dataStartPos += textRecordLength;
    }
    return (receivedItems);
}

// Decode NOTAMS (including TFRs) and Service Status - Product ID 8
- (void)decodeProductID8:(uint8_t *)data length:(uint16_t)frameLength
{
    if (DEBUG_PRODUCTID_8) NSLog(@"Message with product ID = 8");
    NSMutableDictionary *newItems = [self decodeAPDU:data length:frameLength];
    
    // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED NOTAMS
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:newItems withType:kNOTAM];
    
    self.adsbDataAvailable |= kNOTAM;
}

// Decode Aerodrome and Airspace D-ATIS - Product ID 9
- (void)decodeProductID9:(uint8_t *)data length:(uint16_t)frameLength
{
    if (DEBUG_PRODUCTID_9) NSLog(@"Message with product ID = 9");
    NSMutableDictionary *newItems = [self decodeAPDU:data length:frameLength];

    // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED D-ATIS
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:newItems withType:kNOTAM];
    
    self.adsbDataAvailable |= kDATIS;
}

// Decode Aerodrome and Airspace TWIP - Product ID 10
- (void)decodeProductID10:(uint8_t *)data length:(uint16_t)frameLength
{
    if (DEBUG_PRODUCTID_10) NSLog(@"Message with product ID = 10");
    NSMutableDictionary *newItems = [self decodeAPDU:data length:frameLength];
    
    // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED TWIPS
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:newItems withType:kTWIP];
    
    self.adsbDataAvailable |= kTWIP;
}

// Decode Aerodrome and Airspace AIRMET - Product ID 11
- (void)decodeProductID11:(uint8_t *)data length:(uint16_t)frameLength
{
    if (DEBUG_PRODUCTID_11) NSLog(@"Message with product ID = 11");
    NSMutableDictionary *newItems = [self decodeAPDU:data length:frameLength];
    
    // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED AIRMETS
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:newItems withType:kAIRMET];
    
    self.adsbDataAvailable |= kAIRMET;
}

// Decode Aerodrome and Airspace SIGMET & Conductive SIGMET - Product ID 12
- (void)decodeProductID12:(uint8_t *)data length:(uint16_t)frameLength
{
    if (DEBUG_PRODUCTID_12) NSLog(@"Message with product ID = 12");
    NSMutableDictionary *newItems = [self decodeAPDU:data length:frameLength];
    
    // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED SIGMETS
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:newItems withType:kSIGMET];

    self.adsbDataAvailable |= kSIGMET;
}

// Decode Aerodrome and Airspace SUA Status - Product ID 13
- (void)decodeProductID13:(uint8_t *)data length:(uint16_t)frameLength
{
    if (DEBUG_PRODUCTID_13) NSLog(@"Message with product ID = 13");
    NSMutableDictionary *newItems = [self decodeAPDU:data length:frameLength];
    
    // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED AIRSPACE UPDATES
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:newItems withType:kSUAUPDATE];
    
    self.adsbDataAvailable |= kSUAUPDATE;
}

// decode the incoming NEXRAD FISB data into uncompressed data
- (void)decodeGraphicalWeatherData:(uint8_t *)data length:(uint16_t)iFrameDataLength isCONUS:(BOOL)isCONUS
{
    // First two bytes of the iFrame were length and frame type.
    // Second 4 bytes were the APDU header info.
    // Here we're start parsing at the 7th byte where the block reference indicator starts
    // Block reference indicator is first 3 bytes
    BOOL elementIdentifier = data[0] & 0x80;
    BOOL hemisphere = data[0] & 0x40;
    uint8_t conusRes = (data[0] & 0x30) >> 4;
    uint32_t blockNumber = 0;
    blockNumber = (data[0] & 0x0F) << 16;
    blockNumber |= (data[1] << 8);
    blockNumber |= data[2];
    
    uint8_t conusResolution;
    if (conusRes == 0) conusResolution = kHighResoltuionCONUSDataScaling;
    else if (conusRes == 1) conusResolution = kMediumResolutionCONUSDataScaling;
    else conusResolution = kLowResolutionCONUSDataScaling;
    
    if (DEBUG_GRAPHICAL_WEATHER_RECEIVED)
    {
        NSLog(@"Product ID is %@:", (isCONUS)?@"64":@"63");
        printf ("byte 0 = 0x%.2X. byte 1 = 0x%.2X. byte 2 = 0x%.2X. \n", data[0], data[1], data[2]);
        printf ("Element Identifier = %s. Hemisphere = %s.\n", (elementIdentifier) ? "RLE data" : "Empty data",
                (hemisphere) ? "Southern" : "Northern");
        printf ("Bin scale factor: %d  ", conusResolution);
        printf ("block number = 0x%.6X or %d.\n", blockNumber, blockNumber);
        [self convertBlockNumberToLatLon:blockNumber];
    }
    
    if (DEBUG_PRODUCTID_64_B && isCONUS) NSLog(@"data = 0x%.2X Block # %d. Bin scale factor = %d.", data[0], blockNumber, conusResolution);
    
    uint8_t emptyBlockData[kNumberOfBinsPerBlock];
    for (int i=0; i<kNumberOfBinsPerBlock; i++) emptyBlockData[i] = 0;
    
    // Decompress RLE data into memory
    if (elementIdentifier)
    {
        uint8_t blockData[kNumberOfBinsPerBlock];
        uint16_t ptr = 3;
        uint16_t numberOfBins, i, j;
        
        for (uint8_t b=0; b<kNumberOfBinsPerBlock; b++) blockData[b] = 9;
        
        j=0;
        while (ptr < iFrameDataLength)
        {
            numberOfBins = ((data[ptr] & 0xF8) >> 3) + 1;
            if (DEBUG_PRODUCTID_63_B)
                printf("block # %d: data is 0x%2X. %d bins with intensity %d\n", blockNumber, data[ptr], numberOfBins, data[ptr] & 0x07);
            for (i = 0; i<numberOfBins; i++)
            {
                blockData[j] = (data[ptr] & 0x07);
                j++;
            }
            ptr++;
        }
        
        if (DEBUG_PRODUCTID_63_C)
        {
            printf("Block number: %d.\n", blockNumber);
            for (int k=0; k<32; k++) printf("%2d ", blockData[k]);
            printf("\n");
            for (int k=32; k<64; k++) printf("%2d ", blockData[k]);
            printf("\n");
            for (int k=64; k<96; k++) printf("%2d ", blockData[k]);
            printf("\n");
            for (int k=96; k<128; k++) printf("%2d ", blockData[k]);
            printf("\n");
        }
        
        if (isCONUS)
        {
            [self expandCONUSDataBlocks:[NSData dataWithBytes:blockData length:kNumberOfBinsPerBlock]
                           isEmptyBlock:NO
                            blockNumber:blockNumber
                            scaleFactor:conusResolution];
        }
        else
        {
            NSData *block = [[NSData alloc] initWithBytes:blockData length:128];
            [self.fisbBlocks setObject:block forKey:[NSNumber numberWithUnsignedLong:blockNumber]];
        }
    }
    else  // The block represents a group of empty block
    {
        // Upper nibble of byte 3 is the initial bitmap data, the lower nibble is the length of the bitmap
        uint8_t bitmapLength = data[3] & 0xF;
        
        // Determine which blocks are empty, aka decode the bitmap
        if (isCONUS)
        {
            [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:blockNumber scaleFactor:conusResolution];
            if (data[3] & 0x10) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + 1) scaleFactor:conusResolution];
            if (data[3] & 0x20) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + 2) scaleFactor:conusResolution];
            if (data[3] & 0x40) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + 3) scaleFactor:conusResolution];
            if (data[3] & 0x80) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + 4) scaleFactor:conusResolution];
            
            for (uint8_t i=1; i<bitmapLength; i++)
            {
                if (data[i+3] & 0x01) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) - 3) scaleFactor:conusResolution];
                if (data[i+3] & 0x02) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) - 2) scaleFactor:conusResolution];
                if (data[i+3] & 0x04) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) - 1) scaleFactor:conusResolution];
                if (data[i+3] & 0x08) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) - 0) scaleFactor:conusResolution];
                if (data[i+3] & 0x10) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) + 1) scaleFactor:conusResolution];
                if (data[i+3] & 0x20) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) + 2) scaleFactor:conusResolution];
                if (data[i+3] & 0x40) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) + 3) scaleFactor:conusResolution];
                if (data[i+3] & 0x80) [self expandCONUSDataBlocks:nil isEmptyBlock:YES blockNumber:(blockNumber + (i*8) + 4) scaleFactor:conusResolution];
            }
        }
        else
        {
            [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:blockNumber]];
            if (data[3] & 0x10) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + 1)]];
            if (data[3] & 0x20) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + 2)]];
            if (data[3] & 0x40) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + 3)]];
            if (data[3] & 0x80) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + 4)]];
            
            for (uint8_t i=1; i<bitmapLength; i++)
            {
                if (data[i+3] & 0x01) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) - 3)]];
                if (data[i+3] & 0x02) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) - 2)]];
                if (data[i+3] & 0x04) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) - 1)]];
                if (data[i+3] & 0x08) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) - 0)]];
                if (data[i+3] & 0x10) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) + 1)]];
                if (data[i+3] & 0x20) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) + 2)]];
                if (data[i+3] & 0x40) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) + 3)]];
                if (data[i+3] & 0x80) [self.fisbBlocks setObject:[NSNull null] forKey:[NSNumber numberWithUnsignedLong:(blockNumber + (i*8) + 4)]];
            }
        }
        
        if (DEBUG_GRAPHICAL_WEATHER_RECEIVED)
        {
            NSLog(@"Block number %d is empty...", blockNumber);
            if (data[3] & 0x10) NSLog(@"Block number %d is empty...", blockNumber + 1);
            if (data[3] & 0x20) NSLog(@"Block number %d is empty...", blockNumber + 2);
            if (data[3] & 0x40) NSLog(@"Block number %d is empty...", blockNumber + 3);
            if (data[3] & 0x80) NSLog(@"Block number %d is empty...", blockNumber + 4);
            
            for (uint8_t i=1; i<bitmapLength; i++)
            {
                if (data[i+3] & 0x01) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) - 3));
                if (data[i+3] & 0x02) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) - 2));
                if (data[i+3] & 0x04) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) - 1));
                if (data[i+3] & 0x08) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) - 0));
                if (data[i+3] & 0x10) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) + 1));
                if (data[i+3] & 0x20) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) + 2));
                if (data[i+3] & 0x40) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) + 3));
                if (data[i+3] & 0x80) NSLog(@"Block number %d is empty...", (blockNumber + (i*8) + 4));
            }
        }
    }
}

// Decode Global Block Representation - Regional NEXRAD - Product ID 63
- (void)decodeProductID63:(uint8_t *)data length:(uint16_t)iFrameDataLength
{
    if (DEBUG_GRAPHICAL_WEATHER_RECEIVED) NSLog (@"Regional NEXRAD (63)");
    [self decodeGraphicalWeatherData:data length:iFrameDataLength isCONUS:NO];

    // TODO: DO SOMETHING INTERESTING WITH NEXRAD DATA
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:@"<<< NEXRAD data received >>>" forKey:[NSDate date]] withType:kRegionalNEXRAD];

    // REMINDER: a 10 second period elapsing without receiving another NEXRAD updates signifies the end of the NEXRAD broadcast.
    self.adsbDataAvailable |= kRegionalNEXRAD;
    
    // update the time received
    self.timeOfLastNEXRADReception = [NSDate dateWithTimeIntervalSinceNow:0];
}

// Decode Global Block Representation - CONUS NEXRAD - Product ID 64
- (void)decodeProductID64:(uint8_t *)data length:(uint16_t)iFrameDataLength
{
    if (DEBUG_GRAPHICAL_WEATHER_RECEIVED) NSLog (@"CONUS NEXRAD (64)");
    [self decodeGraphicalWeatherData:data length:iFrameDataLength isCONUS:YES];
    
    // TODO: DO SOMETHING INTERESTING WITH CONUS DATA
    create_delegate_instance_d
    [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:@"<<< CONUS NEXRAD data received >>>" forKey:[NSDate date]] withType:kConusNEXRAD];
    
    // REMINDER: a 10 second period elapsing without receiving another CONUS updates signifies the end of the CONUS broadcast.
    
    self.adsbDataAvailable |= kConusNEXRAD;
    
    // update the time received
    self.timeOfLastCONUSNEXRADReception = [NSDate dateWithTimeIntervalSinceNow:0];
}

// Decode Generic Textual Block Product Type 2 DLAC - Product ID 413
- (void)decodeProductID413:(uint8_t *)data length:(uint16_t)iFrameDataLength
{
    // First two bytes of the iFrame were length and frame type.
    // Second 4 bytes were the APDU header info.
    // Here we're start parsing at the 7th byte where the DLAC-encoded text begins

    uint8_t firstChar, secondChar, thirdChar, fourthChar;
    uint32_t holder;
    NSMutableString *fisbText = [[NSMutableString alloc] initWithCapacity:iFrameDataLength];
    
    for (int i=0; i<iFrameDataLength; i+=3)
    {
        holder = 0;
        holder = (data[i] << 24) | (data[i+1] << 16) | (data[i+2] << 8);
        firstChar = (holder & 0xFC000000) >> 26; 
        secondChar = (holder & 0x03F00000) >> 20; 
        thirdChar = (holder & 0x000FC000) >> 14;
        fourthChar = (holder & 0x00003F00) >> 8;
        [fisbText appendFormat:@"%c%c%c%c", dlacCode[firstChar], dlacCode[secondChar], dlacCode[thirdChar], dlacCode[fourthChar]];
    }
    
    // break the product up into individual components
    NSArray *sentences = [fisbText componentsSeparatedByString:[NSString stringWithFormat:@"%c", 0x1e]];

    for (NSString *s in sentences)
    {
        if ([s hasPrefix:@"M"])
        {
            if (DEBUG_PRODUCTID_413) NSLog(@"METAR text: %@", s);
            
            // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED METAR
            create_delegate_instance_d
            [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:s forKey:[NSDate date]] withType:kMETAR];

            self.adsbDataAvailable |= kMETAR;
        }
        else if ([s hasPrefix:@"T"])
        {
            if (DEBUG_PRODUCTID_413) NSLog(@"TAF text: %@", s);
            
            // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED TAF
            create_delegate_instance_d
            [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:s forKey:[NSDate date]] withType:kTAF];
            
             self.adsbDataAvailable |= kTAF;
        }
        else if ([s hasPrefix:@"SP"])       // SPECIAL AIRMET
        {
            if (DEBUG_PRODUCTID_413) NSLog(@"SPECI text: %@", s);
            
            // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED SPECIAL METAR
            create_delegate_instance_d
            [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:s forKey:[NSDate date]] withType:kMETAR];
            
            self.adsbDataAvailable |= kMETAR;
        }
        /*  SPECIAL USE AIRSPACE UPDATES are not supported in the current ADS-B broadcast */
        else if ([s hasPrefix:@"SU"])
        {
            if (DEBUG_PRODUCTID_413) NSLog(@"SUA text: %@", s);
            create_delegate_instance_d
            [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:s forKey:[NSDate date]] withType:kSUAUPDATE];
         
            self.adsbDataAvailable |= kSUAUPDATE;
        }
        else if ([s hasPrefix:@"P"])
        {
            if (DEBUG_PRODUCTID_413) NSLog(@"PIREP text: %@", s);
            
            // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED PIREP
            create_delegate_instance_d
            [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:s forKey:[NSDate date]] withType:kPIREP];
            
            self.adsbDataAvailable |= kPIREP;

        }
        else if ([s hasPrefix:@"W"])
        {
            if (DEBUG_PRODUCTID_413) NSLog(@"WINDS text: %@", s);
            
            // TODO: DO SOMETHING INTERESTING HERE WITH THE DECODED WINDS
            create_delegate_instance_d
            [d.iPadMainVC displayReceivedTextualProducts:[NSMutableDictionary dictionaryWithObject:s forKey:[NSDate date]] withType:kWINDS];
            
            self.adsbDataAvailable |= kWINDS;
        }
        //else if (DEBUG_PRODUCTID_413) NSLog(@"Other generic text message received: %@.", s);
    }
}

//=========================================================
// getBits -- retrieve random number of bits from a stream
//			  return MSB first
//=========================================================
static uint8_t*	pBits;
static uint8_t	nBitsLeft;

static void getBitsInit( uint8_t* p )
{
	pBits = p;
	nBitsLeft = 8;
}
static uint32_t getBit(void)
{
	uint32_t	r;
    
	if( nBitsLeft == 0 )
	{
		pBits++;
		nBitsLeft = 8;
	}
    
	nBitsLeft--;
    
	r = *pBits;
    
	r >>= nBitsLeft;
	r &= 0x01;
    
	return r;
}
static uint32_t getBits( uint8_t numBits )
{
	uint32_t	val = 0;
    
	while( numBits-- )
	{
		val <<= 1;
		val |= getBit();
	}
    
	return val;
}

- (void) decodeSingleIFrame:(uint8_t *)data length:(uint16_t)iFrameLength
//- (uint16_t) decodeSingleIFrame:(uint8_t *)data length:(uint16_t)iFrameLength
{
    // Bytes 0 and 1 are the length and frame type. We're here because frameLength was not 0 and frameType
    // indicates FIS-B APDU data.
    // So here we'll parse APDU header, which starts at the 3rd byte in the iFrame (frame data section):
    // Applications Methods Flag bit is first.
    // Geographic Locator Flag bit is second.
    // Provider-specific Flag bit is third.
    // Product ID is next 11 bits
    // Segmentation flag is bit 15
    // Time Option bits are bits 16 & 17
    // Hours is bits 18-21
    // Minutes is 22-28
    // Finally, the frame data starts at byte[6]
    
	uint8_t*	p;
	uint8_t		flagAppMethod = 0;
	uint8_t		flagGeoLocator = 0;
	uint8_t		flagProviderSpec = 0;
    
    uint16_t	productID;
    uint8_t		segFlag;
    uint8_t		timeOpts;
	uint8_t		month = 0;
	uint8_t		day = 0;
    uint8_t		hours;
    uint8_t		mins;
	uint8_t		secs = 0;
    uint16_t	productFileLength = 0;
    uint16_t	apduNumber = 0;
    uint16_t	productFileID = 0;
    
	uint16_t	length;
    
    
	getBitsInit( &data[2] );
    
	if( getBits(1) )
		flagAppMethod = 1;
    
	if( getBits(1) )
		flagGeoLocator = 1;
    
	if( getBits(1) )
		flagProviderSpec = 1;
    
	// 11 bits productID
    productID = getBits(11);
    
	if( flagAppMethod ) {
		getBits(8);
	}
    
	if( flagGeoLocator ) {
		getBits(20);
	}
    
	segFlag = getBits(1);
    
	timeOpts = getBits(2);
    
	// 00 - No day, No sec
	// 01 - sec
	// 10 - day
	// 11 - day, sec
	if( timeOpts & 0x02 ) {
		month = getBits(4);
		day   = getBits(5);
	}
	hours = getBits(5);
	mins  = getBits(6);
	if( timeOpts & 0x01 ) {
		secs = getBits(6);
	}
    
	if( segFlag ) {
        
        if (0)     // do it the DO-267A way
        {
            productFileLength = getBits(12);
            apduNumber = getBits(12);
            
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"Segmentation block 24 bit representation:");
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"Product file length = %d", productFileLength);
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"APDU Number = %d", apduNumber);
        }
        else        // do it the mitre way
        {
            productFileID = getBits(10);
            productFileLength = getBits(9);
            apduNumber = getBits(9);
            
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"Segmentation block 28 bit representation:");
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"Product file ID = %d", productFileID);
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"Product file length = %d", productFileLength);
            if (DEBUG_APDU_HEADER_DATA) NSLog(@"APDU Number = %d", apduNumber);
            
        }
        printf("------> skipping iframe product ID=%d %d %d\r\n", productID, segFlag, iFrameLength);
        return;
	}
    
	p = ++pBits;	// skip padding bits (0 to 7 bits)
    
	length = iFrameLength - (p - data);
    
    if (DEBUG_APDU_HEADER_DATA) {
        printf("------> iframe product ID=%d %d %d\r\n", productID, segFlag, iFrameLength);
        //[self printBuffer:@"\ndecodeSingleIFrame. Data is:" buffer:data length: iFrameLength];
    }
    

    switch (productID) {
        case 8:		[self decodeProductID8:p length:length];	break;
        case 9:		[self decodeProductID9:p length:length];	break;
        case 10:	[self decodeProductID10:p length:length];	break;
        case 11:	[self decodeProductID11:p length:length];	break;
        case 12:	[self decodeProductID12:p length:length];	break;
        case 13:	[self decodeProductID13:p length:length];	break;
        case 63:	[self decodeProductID63:p length:length];	break;
        case 64:	[self decodeProductID64:p length:length];	break;
        case 413:	[self decodeProductID413:p length:length];	break;
            
        default:
            //printf("product ID %d.\r\n", productID);
            break;
    }
} // decodeSingleIFrame

#pragma mark - GDL90 message parsing methods
- (void)parseDeviceReport:(uint8_t *)msg length:(NSUInteger)len
{
    /*
     <Byte#1 Upper Byte of Battery Voltage>
     <Byte#2 Lower Byte of Battery Voltage>
     <Byte#5 System Flags>
     */
    uint16_t vbat = 0;
    vbat = msg[0] << 8;
    vbat |= msg[1];
    float batLevel = (float)(vbat - 3500) / 600.0f;
    if (batLevel > 1.0) self.batteryVoltage = 1.0;
    else if (batLevel < 0) self.batteryVoltage = 0.0;
    else self.batteryVoltage = batLevel;
    
    if( msg[4] & 0x04 ) self.isCharging = YES;
    else self.isCharging = NO;
    
    // trigger a notification that new ownship message was recevied
    NSNotification *statusReceived = [NSNotification notificationWithName:@"BlackjackDeviceDataUpdated" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:statusReceived];
}

- (void)parseDetailedSatData:(uint8_t *)msg length:(NSUInteger)len
{
    NSMutableString *satDataString = [[NSMutableString alloc] initWithString:@""];
    for (int i=0; i<len; i++) [satDataString appendFormat:@"%c", msg[i]];
    if (DEBUG_PGSA_INFO) DLog(@"%@", satDataString);
    
    // Break the data into an array of elements
    NSString *separator = [NSString stringWithFormat:@"%c%c", 0x0D, 0x0A];
    NSArray *sentences = [satDataString componentsSeparatedByString:separator];
    satDataString = nil;
    
    // First sentence is the PGSA sentence
    NSArray *elementsInPGSASentence = [[sentences objectAtIndex:0] componentsSeparatedByString:@","];
    
    // Sentence will look like:
    //		eg1. PGSA,A,1,,,,,,,,,,,,,0.0,0.0,0.0*30
    //		eg2. PGSA,A,3,24,14,22,31,11,,,,,,,,3.7,2.3,2.9*3D
    //
    // Skytraq chipset:
    // e.g. PGSA,A,1,,,,,,,,,,,,,0.0,0.0,0.0*30     (no signal)
    //
    // uBlox chipset:
    // e.g. PGSA,A,1,,,,,,,,,,,,,99.99,99.99,99.99*30      (no signal)
    //      PGSA,A,3,02,29,13,12,48,10,25,05,,,,,3.93,2.06,3.35*0D
    
    /* Wikipedia (http://en.wikipedia.org/wiki/Dilution_of_precision_(GPS)) has a good synopsis on how to interpret
     DOP values:
     
     DOP Value	Rating		Description
     ---------	---------	----------------------
     1			Ideal		This is the highest possible confidence level to be used for applications demanding
     the highest possible precision at all times.
     1-2		Excellent	At this confidence level, positional measurements are considered accurate enough to meet
     all but the most sensitive applications.
     2-5		Good		Represents a level that marks the minimum appropriate for making business decisions.
     Positional measurements could be used to make reliable in-route navigation suggestions to
     the user.
     5-10		Moderate	Positional measurements could be used for calculations, but the fix quality could still be
     improved. A more open view of the sky is recommended.
     10-20		Fair		Represents a low confidence level. Positional measurements should be discarded or used only
     to indicate a very rough estimate of the current location.
     >20		Poor		At this level, measurements are inaccurate by as much as 300 meters and should be discarded.
     
     */
    
    // extract whether the fix type is 1=no fix, 2=2D fix or 3=3D fix
    self.fixType = [[elementsInPGSASentence objectAtIndex:2] intValue];
    self.gpsPositionIsValid = (self.fixType > 1) ? YES : NO;
    if (DEBUG_PGSA_INFO) NSLog(@"%s. fix value = %d. GPS position %@ valid.", __FUNCTION__, self.fixType, (self.gpsPositionIsValid)?@"is":@"is not");
    
    // extract HDOP
    if (DEBUG_PGSA_INFO)
    {
        NSNumber *hdop = [NSNumber numberWithFloat:[[elementsInPGSASentence objectAtIndex:16] floatValue]];
        if (DEBUG_PGSA_INFO) NSLog(@"%s. HDOP value = %f.", __FUNCTION__, [hdop floatValue]);
    }
    
    // Extract the number of satellites used in the position fix calculation.
    // WAAS/EGNOS/MSAS satellite numbers are 33, 34, 35, 37, 39, 42, 44, 47, 48, 50, 51 according to
    // here: http://www.gpsinformation.org/dale/dgps.htm#waas & http://gpsinformation.net/exe/waas.html
    NSString *satInDOP;
    NSMutableArray *satsInDOPCalc = [[NSMutableArray alloc] init];
    self.waasInUse = NO;
    for (int i=3; i<15; i++)
    {
        satInDOP = [elementsInPGSASentence objectAtIndex:i];
        if ([satInDOP length] > 0)
        {
            [satsInDOPCalc addObject:satInDOP];
            if ([satInDOP intValue] > 32) self.waasInUse = YES;
        }
        satInDOP = nil;
    }
    self.numOfSatInUse = [NSNumber numberWithInt:[satsInDOPCalc count]];
    self.satsUsedInPosCalc = satsInDOPCalc;
    
    if (DEBUG_PGSA_INFO)
    {
        NSLog(@"%s. fix value = %d.", __FUNCTION__, self.fixType);
        NSLog(@"%s. # of satellites used in DOP calc: %d", __FUNCTION__, [self.numOfSatInUse intValue]);
        NSMutableString *logTxt = [NSMutableString stringWithString:@"Satellites used in DOP calc: "];
        for (NSString *s in self.satsUsedInPosCalc)
        {
            [logTxt appendFormat:@"%@, ", s];
        }
        NSLog(@"%s. %@", __FUNCTION__, logTxt);
    }
    
    satsInDOPCalc = nil;
    
    // Next sentences are the PGSV sentences
    NSArray *elementsInPGSVSentence = nil;
    for (char i=1; i<[sentences count] - 1; i++)
    {
        if (DEBUG_PGSV_INFO) NSLog(@"Sentence is %@", [sentences objectAtIndex:i]);
        elementsInPGSVSentence = [[sentences objectAtIndex:i] componentsSeparatedByString:@","];
        
        self.numOfSatInView = [NSNumber numberWithInt:[[elementsInPGSVSentence objectAtIndex:3] intValue]];
        if (DEBUG_PGSV_INFO) NSLog(@"%s. number of satellites in view = %d.", __FUNCTION__, [self.numOfSatInView intValue]);
        
        // handle the case of the uBlox chip returning no satellites
        if ([self.numOfSatInView intValue] == 0)
        {
            [self.dictOfSatInfo removeAllObjects];
        }
        else
        {
            // If this is first GSV sentence, reset the dictionary of satellite info
            if ([[elementsInPGSVSentence objectAtIndex:2] intValue] == 1) [self.dictOfSatInfo removeAllObjects];
            
            NSNumber *satNum, *satElev, *satAzi, *satSNR, *inUse;
            NSMutableArray *satInfo;
            
            // The number of satellites described in a sentence can vary up to 4.
            int numOfSatsInSentence;
            if ([elementsInPGSVSentence count] < 10) numOfSatsInSentence = 1;
            else if ([elementsInPGSVSentence count] < 14) numOfSatsInSentence = 2;
            else if ([elementsInPGSVSentence count] < 18) numOfSatsInSentence = 3;
            else numOfSatsInSentence = 4;
            
            for (int i=0; i<numOfSatsInSentence; i++)
            {
                int index = i*4 + 4;
                inUse = [NSNumber numberWithBool:NO];
                
                satNum = [NSNumber numberWithInt:[[elementsInPGSVSentence objectAtIndex:index] intValue]];
                satElev = [NSNumber numberWithInt:[[elementsInPGSVSentence objectAtIndex:(index+1)] intValue]];
                satAzi = [NSNumber numberWithInt:[[elementsInPGSVSentence objectAtIndex:(index+2)] intValue]];
                satSNR = [NSNumber numberWithFloat:[[elementsInPGSVSentence objectAtIndex:(index+3)] floatValue]];
                
                // On random occasions, either the data is bad or the parsing fails. Handle any not-a-number conditions.
                if (isnan([satSNR floatValue]) != 0) satSNR = [NSNumber numberWithFloat:0.0];
                
                for (NSNumber *n in self.satsUsedInPosCalc)
                {
                    if ([n intValue] == [satNum intValue])
                    {
                        inUse = [NSNumber numberWithBool:YES];
                        break;
                    }
                }
                satInfo = [NSMutableArray arrayWithObjects:satAzi, satElev, satSNR, inUse, nil];
                
                [self.dictOfSatInfo setObject:satInfo forKey:satNum];
            }
        }
    }
    
    // Post a notification to the view controllers that the satellite data has been updated
    NSNotification *satDataUpdated = [NSNotification notificationWithName:@"SatelliteDataUpdated" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:satDataUpdated];
}

- (void)parseHeartbeatMessage:(uint8_t *)msg length:(NSUInteger)len
{
    // Passed in is the Heartbeat message without the start & end flag bytes,
    // message ID byte or checksum bytes.
    if (DEBUG_HEARTBEAT_MESSAGE)
    {
        NSLog(@"\n%s. Heartbeat message is:",__FUNCTION__);
        for (int j=0; j<len; j++) printf("0x%.2X ", msg[j]);
        printf("\n");
    }
    
    self.batteryIsLow = msg[0] & 0x40;
    BOOL xgps170Running = msg[0] & 0x01;
    
    // Extract the time of the message
    uint32_t timeStamp = ((msg[1] & 0x80) << 9) | (msg[3] << 8) | msg[2];
    float hourFrac = (float)timeStamp / 3600.0f;
    uint8_t hour = floor(hourFrac);
    float minFrac = hourFrac - hour;
    uint8_t min = floor(minFrac * 60.0f);
    float secsFrac = (minFrac * 60) - min;
    uint8_t secs = round(secsFrac * 60.0f);
    
    // Extract the number of uplink messages received
    uint8_t uplinkMsgCount = (msg[4] & 0xF8) >> 3;
    uint16_t basicAndLongMsgCount = ((msg[4] & 0x02) << 8) | msg[5];

    if (DEBUG_HEARTBEAT_MESSAGE)
    {
        NSLog (@"Heartbeat time: 0x%.6X or %d = %2d:%2d:%2dZ", timeStamp, timeStamp, hour, min, secs);
        NSLog (@"Position is valid: %@. Battery low: %@. Device running: %@.",
               (self.gpsPositionIsValid)?@"Yes":@"No", (self.batteryIsLow)?@"Yes":@"No", (xgps170Running)?@"Yes":@"No");
        NSLog (@"%d uplink messages received. %d basic & long messages received.", uplinkMsgCount, basicAndLongMsgCount);
    }
    
    // trigger a notification that new heartbeatmessage was recevied
    NSNotification *heartbeatReceived = [NSNotification notificationWithName:@"HeartbeatMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:heartbeatReceived];

}

- (void)parseUplinkMessage:(uint8_t *)msg length:(NSUInteger)len
{
    if (DEBUG_UPLINK_MESSAGE) [self printBuffer:@"Entering parseUplinkMessage. Msg is:" buffer:msg length:len];

    if (len < 10) return;   // check for badly formed message
    
    // First three bytes are the time of reception, LSB first
    //NSString *timeOfReception = [self calculateZuluTime:msg[2] middleByte:msg[1] lowByte:msg[0]];
    //if (DEBUG_UPLINK_MESSAGE) NSLog(@"timeOfReception = %@ Zulu", timeOfReception);

    unsigned char *uatPayload = &msg[3];
    // Message payload is 432 bytes. The first 8 are a UAT-specific header
    // and the remaining 424 bytes are Application Data organized into I-Frames.
    // IFrames: first 9 bits are length of valid frame data, next 3 bits are for resolution, next 4 bits are for
    // frame type, and remaining bits are frame data
    
    // Bytes 0-2: 23 bits of ground station latitude info (does not include LSB bit 8 of byte 2)
    uint32_t lat = 0;
    lat = uatPayload[0];
    lat <<= 8;
    lat |= uatPayload[1];
    lat <<= 8;
    lat |= (uatPayload[2] & 0xFE);
    lat >>= 1;
    
    bool isSouth = lat & 0x800000;
    float degLat = lat * kLatLonBitResolution;
    if (isSouth) degLat *= -1;
    
    if (DEBUG_UPLINK_MESSAGE)
    {
        printf ("Start of UAT payload header:");
        printf ("\nbyte 0 = 0x%.2X. byte 1 = 0x%.2X. byte 2 = 0x%.2X. \n", uatPayload[0], uatPayload[1], uatPayload[2]);
        printf ("latitude = 0x%.6X or %.4f.\n", lat, degLat);
    }
    
    // bytes 2-5: 24 bits of longitude info (starts with bit 8 of byte 2)
    uint32_t lon = 0;
    lon = uatPayload[3];
    lon <<= 8;
    lon |= uatPayload[4];
    lon <<= 8;
    lon |= (uatPayload[5] & 0xFE);
    lon >>= 1;
    if (uatPayload[2] & 0x01) lon |= 0x800000;
    
    bool isWest = lon & 0x800000;
    float degLon = (lon & 0x7fffff) * kLatLonBitResolution;
    if (isWest) degLon = -1 * (180.0 - degLon);
    
    if (DEBUG_UPLINK_MESSAGE)
    {
        printf ("\nbyte 2 = 0x%.2X. byte 3 = 0x%.2X. byte 4 = 0x%.2X. byte 5 = 0x%.2X. \n",
                uatPayload[2], uatPayload[3], uatPayload[4], uatPayload[5]);
        printf ("longitude = 0x%.6X or %.4f.\n", lon, degLon);
    }
    
    // byte 5, LSB: the position valid bit
    bool positionValid = (uatPayload[5] & 0x01);
    if (DEBUG_UPLINK_MESSAGE) printf ("\nbyte 5 = 0x%.2X. Position %s valid.\n", uatPayload[5],
                                     (positionValid) ? "is" : "is not");

    // Add tower position to the array
    [self.towerLocations setObject:[NSNumber numberWithFloat:degLon] forKey:[NSNumber numberWithFloat:degLat]];
    
    // byte 6, bit 3: application data valid bit
    bool applicationDataValid = (uatPayload[6] & 0x20);
    if (DEBUG_UPLINK_MESSAGE) printf ("\nbyte 6 = 0x%.2X. Application data %s valid.\n", uatPayload[6],
                                     (applicationDataValid) ? "is" : "is not");
    if (applicationDataValid == NO) return;
    
    // byte 6, bits 4-8: slot ID
    uint8_t slotID = uatPayload[6] & 0x1f;
    if (DEBUG_UPLINK_MESSAGE) printf ("\nbyte 6 = 0x%.2X. slot ID is 0x%.2X.\n", uatPayload[6],slotID);

    // byte 7, bit 1-4: TIS-B site ID. If zero, the broadcasting station is not broadcasting TIS-B data
    uint8_t tisbSiteID = (uatPayload[7] & 0xf0) >> 4;
    if (DEBUG_UPLINK_MESSAGE) printf ("\nbyte 7 = 0x%.2X. TIS-B site ID = %d.\n", uatPayload[7], tisbSiteID);
    self.towerList |= (1 << tisbSiteID);
    
    // byte 9-432: application data (multiple iFrames).
    uint16_t ptr=8;
    uint16_t iFrameLength = 1;
    uint8_t frameType = 0;
    uint8_t iFrameCount = 0;
    
    if (DEBUG_UPLINK_MESSAGE) printf("Start of UAT payload:application data (iFrames).\n");
    
    while ((ptr < kUATPayloadApplicationDataLength) && (iFrameLength > 0))
    {
        iFrameLength = uatPayload[ptr] << 1;
        iFrameLength |= (uatPayload[ptr+1] & 0x80) >> 7;
        
        if (DEBUG_UPLINK_MESSAGE)
        {
            printf ("\nptr = %d. byte A = 0x%.2X. byte B = 0x%.2X.\n", ptr, uatPayload[ptr], uatPayload[ptr+1]);
            printf ("iFrameLength = 0x%.6X or %d\n", iFrameLength, iFrameLength);
        }
        
        frameType = (uatPayload[ptr+1] & 0xf);
        if (DEBUG_UPLINK_MESSAGE)
        {
            printf ("\nbyte C = 0x%.2X.\n", uatPayload[ptr+1]);
            printf ("frameType = 0x%X or %d\n", frameType, frameType);
            printf ("Application data is %s.\n", (frameType == 0) ? "FIS-B APDU" : "reserved use data");
        }
            
        if ((iFrameLength > 0) && (frameType == 0) && ((ptr+iFrameLength) < len))
        {
            [self decodeSingleIFrame:&uatPayload[ptr] length:iFrameLength+2];
            self.messageCount++;
        }
        else break;
        
        iFrameCount++;
        if (DEBUG_UPLINK_MESSAGE) NSLog(@"iFrame count = %d.", iFrameCount);
        
        ptr += iFrameLength+2;

    }
    
    // trigger a notification that new heartbeatmessage was recevied
    NSNotification *uplinkReceived = [NSNotification notificationWithName:@"UplinkMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:uplinkReceived];

} // parseUplinkMessage

- (void)parseOwnshipReport:(uint8_t *)msg length:(NSUInteger)len
{
    // Passed in is the Ownship Report without the header byte or trailing 0x7e flag
    if (DEBUG_OWNSHIP_REPORT_MESSAGE)
    {
        NSLog(@"\n%s. Message is:",__FUNCTION__);
        for (int j=0; j<len; j++) printf("0x%.2X ", msg[j]);
        printf("\n");
    }

    // bytes 1-3 are the ICAO participant address
    
    // bytes 4-6 are the latitude, 7-9 are the longitude
    float degLat = [self calculateDegrees:msg[4] middleByte:msg[5] lowByte:msg[6]];
    float degLon = [self calculateDegrees:msg[7] middleByte:msg[8] lowByte:msg[9]];
    self.latitude = [NSNumber numberWithFloat:degLat];
    self.longitude = [NSNumber numberWithFloat:degLon];
    
    // byte 10 + upper nibble of byte 11 is altitude
    uint16_t upper = msg[10] << 4;
    uint16_t lower = (msg[11] & 0xF0) >> 4;
    //uint16_t alt = (msg[10] << 4) | ((msg[11] & 0xF0) >> 4);
    uint16_t alt = upper | lower;
    alt *= 25;
    alt -= 1000;
    self.altitude = [NSNumber numberWithInt:alt];
    
    // lower nibble of byte 11 are miscellaneous indicators
    BOOL isAirborne = msg[11] & 0x08;
    BOOL isExtrapolated = msg[11] & 0x04;
    uint8_t trackType = msg[11] & 0x03;
    
    // upper nibble of byte 12 is NIC, lower nibble is NAC for position
    uint8_t nic = (msg[12] & 0xF0) >> 4;
    uint8_t nacp = msg[12] & 0x0F;
    
    // byte 13 + upper nibble of byte 14 is horizontal velocity
    upper = msg[13] << 4;
    lower = (msg[14] & 0xF0) >> 4;
    //uint16_t horizVel = (msg[13] << 4) | ((msg[14] & 0xF0) >> 4);
    uint16_t horizVel = upper | lower;
    if (DEBUG_OWNSHIP_REPORT_MESSAGE) printf("---> horiz vel = %d.\n", horizVel);
    self.velocity = [NSNumber numberWithInt:horizVel];
    
    // lower nibble of byte 14 + byte 15 is vertical velocity
    int16_t vertVel = ((msg[14] & 0x0F) << 4) | msg[15];
    vertVel *= 64;
    //float vel = sqrtf((float)(horizVel * horizVel) + (float)(vertVel * vertVel));
    //self.velocity = [NSNumber numberWithFloat:vel];
    
    // byte 16 is track/heading
    BOOL trackHeadingIsValid = trackType & 0xf;
    BOOL trackHeadingIsTrueTrackAngle = (trackType & 0x1) & (trackType ^ 0x02);
    BOOL trackHeadingIsHeading = trackType & 0x2;
    BOOL trackHeadingIsTrueHeading = trackType & 0x3;
    float directionAngle = msg[16] * kTrackHeadingResolution;
    if (DEBUG_OWNSHIP_REPORT_MESSAGE) printf("---> msg16 = 0x%.2X. dirAngle = %.1f.\n", msg[16], directionAngle);
    self.heading = [NSNumber numberWithFloat:directionAngle];

    // byte 17 is the emitter category
    
    // bytes 18-25 are call sign
    
    // upper nibble of byte 26 is emergency/priorty code
    uint8_t empCode = (msg[26] & 0xF0) >> 4;
    
    if (DEBUG_OWNSHIP_REPORT_MESSAGE)
    {
        NSLog(@"\n%s.",__FUNCTION__);
        printf ("Latitude is %.4f degrees. Longitude is %.4f degrees.\n", degLat, degLon);
        printf ("Altitude is %d.\n", alt);
        printf ("You are %s.\n", (isAirborne)?"airborne":"on the ground");
        printf ("Position is %s.\n", (isExtrapolated)?"from ADS-B":"Extrapolated from previous report");
        printf ("NIC is %d. NACp is %d.\n", nic, nacp);
        printf ("Horizontal velocity is %d kts. Vertical velocity is %d fpm.\n", horizVel, vertVel);
        if (trackHeadingIsValid)
        {
            if (trackHeadingIsTrueTrackAngle) printf ("True Track Angle is %.1f.\n", directionAngle);
            else if (trackHeadingIsHeading) printf ("Magnetic heading is %.1f.\n", directionAngle);
            else if (trackHeadingIsTrueHeading) printf ("True heading is %.1f.\n", directionAngle);
            else printf ("tt value is not being interpreted correctly.\n");
        }
        else printf ("Track/Heading is not valid.\n");
        printf ("Emergency priority code is %d.\n", empCode);
    }
    
    // trigger a notification that new ownship message was recevied
    NSNotification *ownshipReceived = [NSNotification notificationWithName:@"OwnshipMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:ownshipReceived];

} // parseOwnshipReport

- (void)parseOwnshipGeometricAltitudeReport:(uint8_t *)msg length:(NSUInteger)len
{
    // The msg is the GDL-90 message. See section 3.8 on page 28
    // Passed in is the Ownship Geometric Altitude Report without the header byte or trailing 0x7e flag
    if (DEBUG_OWNSHIP_GA_REPORT_MESSAGE)
    {
        NSLog(@"\n%s. Message is:",__FUNCTION__);
        for (int j=0; j<len; j++) printf("0x%.2X ", msg[j]);
        printf("\n");
    }
    
    // bytes 0-1 are the altitude
    int16_t alt = (msg[0] << 8) | msg[1];
    alt *= 5;
    
    // bytes 2-3 are the vertical metrics
    BOOL vertWarnIndicator = msg[2] & 0x80;
    uint16_t vfom = ((msg[2] & 0x7F) << 8) | msg[3];
    
    if (DEBUG_OWNSHIP_GA_REPORT_MESSAGE)
    {
        NSLog(@"\n%s.",__FUNCTION__);
        printf ("Altitude is = %d.\n", alt);
        printf ("Vertical warning indicator is %s.\n", (vertWarnIndicator)?"Set":"Off");
        printf ("Vertical Figure of Merit is 0x%.6X.\n", vfom);
    }
    
    // trigger a notification that new ownship altitude report was recevied
    NSNotification *ownshipAltReceived = [NSNotification notificationWithName:@"OwnshipAltitudeMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:ownshipAltReceived];
}

- (void)parseShortTrafficReport:(uint8_t *)msg length:(NSUInteger)len
{
    ShortTrafficObjectInfo stoi;
    
    // upper nibble of first byte is the traffic alert status
    stoi.status = 0;
    stoi.status = (msg[0] & 0xF0) >> 4;
    
    /* lower nibble of first byte is the address type:
     * 0 = ADS-B with ICAO address
     * 1 = ADS-B with self-assigned address
     * 2 = TIS-B with ICAO address
     * 3 = TIS-B with track file ID
     * 4 = surface vehicle
     * 5 = ground station beacon
     * 6-15 = reserved
     */
    stoi.addressType = 0;
    stoi.addressType = msg[0] & 0x0F;
    
    // next three bytes are the traffic's ICAO address
    stoi.icaoAddress = 0;
    stoi.icaoAddress = (msg[1] << 16) | (msg[2] << 8) | msg[3];
    
    // next 3 bytes are latitude value with resolution = 180 / (2^23) degrees
    stoi.latitude = 0;
    stoi.latitude = [self calculateDegrees:msg[4] middleByte:msg[5] lowByte:msg[6]];

    // next 3 bytes are longitude value with resolution = 180 / (2^23) degrees
    stoi.longitude = 0;
    stoi.longitude = [self calculateDegrees:msg[7] middleByte:msg[8] lowByte:msg[9]];;
    
    // next 12 bits are altitude
    uint16_t upper = msg[10] << 4;
    uint16_t lower = (msg[11] & 0xF0) >> 4;
    uint16_t alt = upper | lower;
    alt *= 25;
    alt -= 1000;
    stoi.altitude = alt;

    /* next nibble is miscellaneous indicators:
     * bit 3   bit 2    bit 1    bit 0
     *   x       x        0        0    = trackHeading not valid
     *   x       x        0        1    = trackHeading is true track angle
     *   x       x        1        0    = trackHeading is magnetic heading
     *   x       x        1        1    = trackHeading is true heading
     *   x       0        x        x    = report is updated
     *   x       1        x        x    = report is extrapolated
     *   0       x        x        x    = on ground
     *   1       x        x        x    = airborne
     */
    stoi.miscIndicators = msg[11] & 0x0F;
    
    // Next nibble is the navigation integrity category (nic). (See GDL-90 spec pg. 21.)
    stoi.nic = 0;
    stoi.nic = (msg[12] & 0xF0) >> 4;
    
    // Next nibble is navigation accuracy category for position (nacP)
    // (See GDL-90 spec pg. 21.)
    stoi.nacP = 0;
    stoi.nacP = msg[12] & 0x0F;
    
    // next 12 bits are horizontal velocity in knots. (0xFFF = unknown)
    stoi.horizVelocity = 0;
    upper = msg[13] << 4;
    lower = (msg[14] & 0xF0) >> 4;
    stoi.horizVelocity = upper | lower;
    
    // next 12 bits are vertical velocity in units of 64 fpm. (0xFFF = unknown)
    stoi.vertVelocity = 0;
    upper = (msg[14] & 0x0F) << 8;
    lower = msg[15];
    stoi.vertVelocity = (upper | lower) * 64;
    
    // next nibble is the track heading with resolution = 360/256
    stoi.trackHeading = (float)msg[16] * 360.0 / 256.0;
    
    // next byte is emitter category. see GDL-90 spec page 23
    stoi.emitterCategory = msg[17];
    
    // next 8 bytes are callsign
    stoi.callsign[0] = msg[18];
    stoi.callsign[1] = msg[19];
    stoi.callsign[2] = msg[20];
    stoi.callsign[3] = msg[21];
    stoi.callsign[4] = msg[22];
    stoi.callsign[5] = msg[23];
    stoi.callsign[6] = msg[24];
    stoi.callsign[7] = msg[25];

    // next 4 bits are emergency/priority code
    stoi.emergencyPriorityCode = (msg[26] & 0xF0) >> 4;
    
    if (DEBUG_SHORT_TRAFFIC_MESSAGE)
    {
        NSLog(@"Traffic info:");

        NSLog(@"byte[0] = 0x%2X. Traffic status = %d.", msg[0], stoi.status);
        NSLog(@"byte[0] = 0x%2X. Address type = %d.", msg[0], stoi.addressType);
        NSLog(@"byte[1] = 0x%2X. byte[2] = 0x%2X. byte[3] = 0x%2X.\nICAO Address = 0x%4X = %d.", msg[1], msg[2], msg[3], stoi.icaoAddress, stoi.icaoAddress);
        NSLog(@"byte[4] = 0x%2X. byte[5] = 0x%2X. byte[6] = 0x%2X.\nlatitude = %.6f.", msg[4], msg[5], msg[6], stoi.latitude);
        NSLog(@"byte[7] = 0x%2X. byte[8] = 0x%2X. byte[9] = 0x%2X.\nlongitude = %.6f.", msg[7], msg[8], msg[9], stoi.longitude);
        NSLog(@"byte[10] = 0x%2X. byte[11] = 0x%2X. altitude = %d.", msg[10], msg[11], stoi.altitude);
        NSLog(@"byte[11] = 0x%2X. misc indicators = 0x%2X = %d.", msg[11], stoi.miscIndicators, stoi.miscIndicators);
        NSLog(@"byte[12] = 0x%2X. nic = %d.", msg[12], stoi.nic);
        NSLog(@"byte[12] = 0x%2X. nacP = %d.", msg[12], stoi.nacP);
        NSLog(@"byte[13] = 0x%2X. byte[14] = 0x%2X. horizVel = %d.", msg[13], msg[14], stoi.horizVelocity);
        NSLog(@"byte[14] = 0x%2X. byte[15] = 0x%2X. horizVel = %d.", msg[14], msg[15], stoi.vertVelocity);
        NSLog(@"byte[16] = 0x%2X. track / heading = %.1f.", msg[16], stoi.trackHeading);
        NSLog(@"byte[17] = 0x%2X. Address type = %d.", msg[17], stoi.emitterCategory);
        NSLog(@"byte[18] = 0x%2X. byte[19] =x 0x%2X. byte[20] = 0x%2X. byte[21] = 0x%2X.", msg[18], msg[19], msg[20], msg[21]);
        NSLog(@"byte[22] = 0x%2X. byte[23] = 0x%2X. byte[24] = 0x%2X. byte[25] = 0x%2X.", msg[22], msg[23], msg[24], msg[25]);
        NSLog(@"Call sign is %c%c%c%c%c%c%c%c.", stoi.callsign[0], stoi.callsign[1], stoi.callsign[2], stoi.callsign[3], stoi.callsign[4], stoi.callsign[5], stoi.callsign[6], stoi.callsign[7]);
        NSLog(@"byte[26] = 0x%2X. priority code = %d.", msg[26], stoi.emergencyPriorityCode);
    }
    
    // TODO: DO SOMETHING INTERESTING WITH THE TRAFFIC DATA
    create_delegate_instance_d
    [d.iPadMainVC displayShortTrafficReport:&stoi];
    
    // trigger a notification traffic report data has been updated
    NSNotification *trafficDataUpdated = [NSNotification notificationWithName:@"ShortTrafficMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:trafficDataUpdated];

    self.messageCount++;
    self.adsbDataAvailable |= kTRAFFIC;
}

- (void)parseBasicAndLongTrafficReport:(uint8_t *)msg length:(NSUInteger)len
{
    LongTrafficObjectInfo ltoi;
    ltoi.callSignFlightPlanID[0] = '\0';
    ltoi.statusDescr[0] = '\0';
    
    NSNotification *trafficDataUpdated = nil;
    
    // Passed in is the Basic UAT Report without the header byte or trailing 0x7e flag
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf("\nparseTrafficReport. Message is:");
        for (int j=0; j<len; j++) printf("0x%.2X ", msg[j]);
        printf("\n");
    }
    
    // First three bytes are the time of reception, LSB first.
    NSArray *timeReceived = [self calculateZuluTime:msg[2] middleByte:msg[1] lowByte:msg[0]];
    ltoi.hour = (uint8_t)[[timeReceived objectAtIndex:0] unsignedCharValue];
    ltoi.min = (uint8_t)[[timeReceived objectAtIndex:1] unsignedCharValue];
    ltoi.sec = (uint8_t)[[timeReceived objectAtIndex:2] unsignedCharValue];
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"timeOfReception = %d:%d:%d Zulu", ltoi.hour, ltoi.min, ltoi.sec);
    
    // ADS-B message payload starts at byte 4 of the passed in message.
    // The message payload begins with a 4 byte header, and the state vector follows.
    // byte [3]: first 5 bits are the payload type code, indicating whether the traffic info is ADS-B,
    // ADS-R, TIS-B or surface vehicle
    uint8_t payloadTypeCode = 0;
    payloadTypeCode = (msg[3] & 0xF8) >> 3;
    
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"\nbyte [3] = 0x%.2X. payload type code = %@\n", msg[3], [self bytes_to_binary:payloadTypeCode numberOfBytes:1]);
    switch (payloadTypeCode) {
        case 0:
            ltoi.adsbTarget = YES;
            ltoi.tisbTarget = NO;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = NO;
            ltoi.icaoAddressIsSelfAssigned = NO;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is ADS-B target with ICAO 24 bit address");
            break;
            
        case 1:
            ltoi.adsbTarget = YES;
            ltoi.tisbTarget = NO;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = NO;
            ltoi.icaoAddressIsSelfAssigned = YES;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is ADS-B target with temporary self-assigned address");
            break;
            
        case 2:
            // product ID 2 are always this payloadTypeCode at Daytona
            ltoi.adsbTarget = NO;
            ltoi.tisbTarget = YES;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = YES;
            ltoi.icaoAddressIsSelfAssigned = NO;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is TIS-B or ADS-rebroadcast target with ICAO 24 bit address");
            break;
            
        case 3:
            ltoi.adsbTarget = NO;
            ltoi.tisbTarget = YES;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = NO;
            ltoi.icaoAddressIsSelfAssigned = NO;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is TIS-B target with track file identifier");
            break;
            
        case 4:
            ltoi.adsbTarget = NO;
            ltoi.tisbTarget = NO;
            ltoi.surfaceVehicle = YES;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = NO;
            ltoi.icaoAddressIsSelfAssigned = NO;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is a surface vehicle");
            break;
            
        case 5:
            ltoi.adsbTarget = NO;
            ltoi.tisbTarget = NO;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = YES;
            ltoi.adsrTarget = NO;
            ltoi.icaoAddressIsSelfAssigned = NO;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is a fixed ADS-B beacon");
            break;
            
        case 6:
            ltoi.adsbTarget = NO;
            ltoi.tisbTarget = NO;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = YES;
            ltoi.icaoAddressIsSelfAssigned = YES;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Traffic is ADS-rebroadcast target with non-ICAO address");
            break;
            
        case 7:
            ltoi.adsbTarget = NO;
            ltoi.tisbTarget = NO;
            ltoi.surfaceVehicle = NO;
            ltoi.adsbBeacon = NO;
            ltoi.adsrTarget = NO;
            ltoi.icaoAddressIsSelfAssigned = NO;
            if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"ERROR! Traffic is using a reserved value.");
            break;
            
        default:
            break;
    }
    
    // byte [3]: last 3 bits of the fourth byte are the address qualifier
    uint8_t addressQualifier = 0;
    addressQualifier = msg[3] & 0x07;
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"\nbyte [3] = 0x%.2X. address qualifier = %@.\n", msg[3],
                                     [self bytes_to_binary:addressQualifier numberOfBytes:1]);
    
    // bytes [4-6]: 24 bit number representing the ICAO address of an object
    uint32_t icaoAddr = 0;
    icaoAddr = msg[4] << 16;
    icaoAddr |= msg[5] << 8;
    icaoAddr |= msg[6];
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf ("\nbyte 4 = 0x%.2X. byte 5 = 0x%.2X. byte 6 = 0x%.2X. \n", msg[4], msg[5], msg[6]);
        printf ("icaoAddress = 0x%.6X.\n", icaoAddr);
    }
    ltoi.icaoAddress = icaoAddr;
    
    // bytes [7-9]: 23 bits of latitude info (does not include bit 8 of byte 9)
    uint32_t lat = 0;
    lat = msg[7];
    lat <<= 8;
    lat |= msg[8];
    lat <<= 8;
    lat |= (msg[9] & 0xFE);
    lat >>= 1;
    
    bool isSouth = lat & 0x800000;
    float degLat = lat * kLatLonBitResolution;
    if (isSouth) degLat *=1;
    ltoi.latitude = degLat;
    
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf ("\nbyte 7 = 0x%.2X. byte 8 = 0x%.2X. byte 9 = 0x%.2X. \n", msg[7], msg[8], msg[9]);
        printf ("latitude = 0x%.6X.\n", lat);
        printf ("latitude is %.4f.\n", degLat);
    }
    
    // bytes [9-12]: 24 bits of longitude info (starts with bit 8 of byte 9)
    uint32_t lon = 0;
    lon = msg[10];
    lon <<= 8;
    lon |= msg[11];
    lon <<= 8;
    lon |= (msg[12] & 0xFE);
    lon >>= 1;
    if (msg[9] & 0x01) lon |= 0x800000;
    
    bool isWest = lon & 0x800000;
    float degLon = (lon & 0x7fffff) * kLatLonBitResolution;
    if (isWest) degLon = -1 * (180.0 - degLon);
    ltoi.longitude = degLon;
    
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf ("\nbyte 9 = 0x%.2X. byte 10 = 0x%.2X. byte 11 = 0x%.2X. byte 12 = 0x%.2X. \n", msg[9], msg[10], msg[11], msg[12]);
        printf ("longitude = 0x%.6X.\n", lon);
        printf ("latitude is %.4f.\n", degLon);
    }
    
    // byte [12], bit 8: altitude type 
    bool altitudeType = msg[12] & 0x01;
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) printf ("\nbyte 12 = 0x%.2X. altitude type is %d.\n", msg[12], (int)altitudeType);
    ltoi.altitudeType = altitudeType;
    
    // byte [13-14]: 12 bits of altitude info (only uses upper nibble of byte 14)
    uint16_t codedAltitude = 0;
    codedAltitude = msg[13] << 4;
    codedAltitude |= ((msg[14] & 0xf0) >> 4);
    int32_t geometricAltitude = (codedAltitude * 25) - 1025;
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf ("\nbyte 13 = 0x%.2X. byte 14 = 0x%.2X.\n", msg[13], msg[14]);
        printf ("codedAltitude = 0x%.4X.\n", codedAltitude);
        printf ("geometricAltitude is %d.", geometricAltitude);
    }
    ltoi.altitude = geometricAltitude;
    
    // byte 14: lowest 4 bits representing Navigation Integrity Category (pg. 41)
    float nicList[17] = {-1, 37040, 14816, 7408, 3704, 1852, 1111.2, 555.6, 370.4, 185.2, 75, 25, 7.5, -1, -1, -1, -1};
    char index = msg[14] & 0x04;
    float nic = nicList[index];
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE) printf ("\nbyte 14 = 0x%.2X. nic is %d.\n", msg[14], (int)nic);
    ltoi.nic = nic;
    
    // byte 15, bits 0 & 1: Airborne / ground state
    bool airborne = !(msg[15] & 0x80);
    bool supersonic = msg[15] & 0x40;
    char agState = msg[15] & 0x02;
    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf ("\nbyte 15 = 0x%.2X. air/ground state is %d\n", msg[15], agState);
        printf ("object is%sin the air.\n", (airborne) ? " " : " not ");
        printf ("object is%ssupersonic.\n", (supersonic) ? " " : " not ");
    }
    ltoi.airborne = airborne;
    ltoi.supersonic = supersonic;
    
    // Bytes 15-18: bits 4-8 of byte 15 thru bit 1 of byte 18. Contains information about an object's airborne
    // vertical and horizontal velocity vectors, or alternatively the speed and heading of an object on the ground.
    // Determine data based on Airborne/ground state:
    bool horizVelocityIsSoutherly, horizVelocityIsWesterly;
    uint16_t vel = 0;
    uint16_t tracking = 0;
    int16_t northVelocityMagnitude = 0;
    int16_t eastVelocityMagnitude = 0;
    uint8_t groundspeed = 0;
    char multiplier = 1;
    char tahType = 0;
    float objectHeading = 0;
    
    if (airborne)   // data is horizontal velocity
    {
        if (supersonic) multiplier = 4;
        
        vel = (msg[15] & 0x0f) << 6;
        vel |= ((msg[16] & 0xfd) >> 2);
        horizVelocityIsSoutherly = msg[15] & 0x10;
        
        if (vel == 0)
        {
            ltoi.northerlyVelocity = 0;
            ltoi.northerlyVelocityAvail = NO;
        }
        else
        {
            northVelocityMagnitude = (vel - 1) * multiplier;
            ltoi.northerlyVelocityAvail = YES;
            
            if (horizVelocityIsSoutherly) northVelocityMagnitude *= -1;
            ltoi.northerlyVelocity = northVelocityMagnitude;
        }
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            printf ("\nbyte 15 = 0x%.2X. byte 16 = 0x%.2X.\n", msg[15], msg[16]);
            printf ("velocity = 0x%.4X. multiplier is %d.\n", vel, multiplier);
            printf ("northVelocityMagnitude is %d. direction is %s.\n", northVelocityMagnitude,
                    (horizVelocityIsSoutherly) ? "south" : "north");
        }
        
        vel = 0;
        if (msg[16] & 0x01) vel = 0x200;
        vel |= msg[17] << 1;
        if (msg[18] & 0x80) vel |= 0x01;
        horizVelocityIsWesterly = msg[16] & 0x02;
        
        if (vel == 0)
        {
            ltoi.easterlyVelocity = 0;
            ltoi.easterlyVelocityAvail = NO;
        }
        else
        {
            eastVelocityMagnitude = (vel-1) * multiplier; // (vel * multiplier) - multiplier;
            ltoi.easterlyVelocityAvail = YES;
            
            if (horizVelocityIsWesterly) eastVelocityMagnitude *= -1;
            ltoi.easterlyVelocity = eastVelocityMagnitude;
        }
        
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            printf ("\nbyte 16 = 0x%.2X. byte 17 = 0x%.2X. byte 18 = 0x%.2X.\n", msg[16], msg[17], msg[18]);
            printf ("velocity = 0x%.4X. multiplier is %d.\n", vel, multiplier);
            printf ("eastVelocityMagnitude is %d. direction is %s.\n", eastVelocityMagnitude,
                    (horizVelocityIsWesterly) ? "west" : "east");
        }
        
    }
    else    // object is on the ground
    {
        vel = (msg[15] & 0x0f) << 6;
        vel |= ((msg[16] & 0xfd) >> 2);
        groundspeed = vel - 1;
        
        if (vel == 0)
        {
            ltoi.groundSpeed = 0;
            ltoi.groundSpeedAvail = NO;
        }
        else
        {
            ltoi.groundSpeed = groundspeed;
            ltoi.groundSpeedAvail = NO;
        }
        
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            printf ("\nbyte 15 = 0x%.2X. byte 16 = 0x%.2X.\n", msg[15], msg[16]);
            printf ("velocity = 0x%.4X.\n", vel);
            printf ("groundspeed is %d.\n", groundspeed);
        }
    }
    
    tracking = msg[17] << 1;
    if (msg[18] & 0x80) tracking |= 0x01;
    objectHeading = tracking * 0.703125;
    ltoi.heading = objectHeading;
    
    tahType = msg[16] & 0x02;
    
    switch (tahType) {
        case 1:
            ltoi.trueTrackAngle = YES;
            ltoi.magneticHeading = NO;
            ltoi.trueHeading = NO;
            break;
            
        case 2:
            ltoi.trueTrackAngle = NO;
            ltoi.magneticHeading = YES;
            ltoi.trueHeading = NO;
            break;
            
        case 3:
            ltoi.trueTrackAngle = NO;
            ltoi.magneticHeading = NO;
            ltoi.trueHeading = YES;
            break;
            
        default:    // this condition indicates that heading data is no avialable.
            ltoi.trueTrackAngle = NO;
            ltoi.magneticHeading = NO;
            ltoi.trueHeading = NO;
            ltoi.heading = -1;
            break;
    }

    if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
    {
        printf ("\nbyte 17 = 0x%.2X. byte 18 = 0x%.2X.\n", msg[17], msg[18]);
        printf ("tracking = 0x%.4X. heading is %.1f.\n", tracking, objectHeading);
    }

    // Byte 18, bits 2-8 and byte 19, bits 1-4: Vertical velocity or A/V size
    if (airborne)
    {
        bool vertVelocitySourceisBarometric = msg[18] & 0x40;
        bool vertVelocityIsDownward = msg[18] & 0x20;
        uint16_t verticalRate = 0;
        uint16_t verticalRateFPM = 0;
        
        verticalRate = (msg[18] & 0x1f) << 4;
        verticalRate |= ((msg[19] & 0xf0) >> 4);
        verticalRateFPM = (verticalRate * 64) - 64;
        
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            printf ("\nbyte 18 = 0x%.2X. byte 19 = 0x%.2X.\n", msg[18], msg[19]);
            printf ("vertical velocity source is %s.\n", (vertVelocitySourceisBarometric) ? "Barometric" : "GNSS");
            printf ("direction of vertical velocity is %s.\n", (vertVelocityIsDownward) ? "downward" : "upward");
            printf ("verticalRate = 0x%.4X. verticalRateFPM is %d.\n", verticalRate, verticalRateFPM);
        }
        ltoi.verticalVelocitySource = vertVelocitySourceisBarometric;
        ltoi.verticalVelocityDirection = vertVelocityIsDownward;
        ltoi.verticalVelocityRate = verticalRateFPM;
        
        ltoi.vehicleSizeIsValid = NO;
        ltoi.length = 0;
        ltoi.width = 0;
        ltoi.gpsAntennaInfoIsValid = NO;
        ltoi.gpsAntennaOffsetAxis = NO;
        ltoi.gpsAntennaOffsetSide = NO;
        ltoi.gpsAntennaOffsetDistance = 0;
    }
    else
    {
        float width = 0;
        uint8_t length = 0;
        uint8_t lengthCode = 0;
        bool widthCode = 0;
        
        lengthCode = ((msg[18] & 0x70) >> 4);
        widthCode = msg[18] & 0x08;
        
        length = (lengthCode * 10) + 15;
        if ((lengthCode == 0) && (widthCode == 0)) length = 0;
        
        float widthList[15] = {23, 28.5, 34, 33, 38, 39.5, 45, 45, 52, 59.5, 67, 72.5, 80, 80, 90};
        char index = ((msg[18] & 0x78) >> 3);
        width = widthList[index];
        
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            printf ("\nbyte 18 = 0x%.2X.\n", msg[18]);
            printf ("lengthCode = 0x%.2X. widthCode = %d. index = %d.\n", lengthCode, (widthCode) ? 1 : 0, index);
            printf ("length = %d. width = %.1f.\n", length, width);
        }
        ltoi.length = length;
        ltoi.width = width;
        if ((msg[18] & 0x78) == 0) ltoi.vehicleSizeIsValid = NO;
        else ltoi.vehicleSizeIsValid = YES;
        
        // per spec: "When on ground, the UAT transmitting subsystem shall alternate the transmission of lateral and longitudinal offsets
        // each second."
        // This version of the SDK does not handle this condition.
        if (((msg[18] & 0x01) && (msg[19] & 0xC0)) == 0) ltoi.gpsAntennaInfoIsValid = NO;
        else
        {
            ltoi.gpsAntennaInfoIsValid = YES;
            if (msg[18] & 0x02) // GPS antenna offset is lateral
            {
                ltoi.gpsAntennaOffsetAxis = NO;
                ltoi.gpsAntennaOffsetSide = (msg[18] & 0x01);
                ltoi.gpsAntennaOffsetDistance = ((msg[19] & 0xC0) >> 6) * 2;
            }
            else                // GPS antenna offset is longitudinal
            {
                ltoi.gpsAntennaOffsetAxis = YES;
                ltoi.gpsAntennaOffsetSide = NO;
                
                uint8_t longitudinalOffset = 0;
                longitudinalOffset = (msg[18] & 0x01) << 4;
                longitudinalOffset |= ((msg[19] & 0xF0) >> 4);
                ltoi.gpsAntennaOffsetDistance = (longitudinalOffset - 1) * 2;
            }
        }
        
        ltoi.verticalVelocitySource = NO;
        ltoi.verticalVelocityDirection = NO;
        ltoi.verticalVelocityRate = 0;
    }
    
    // If the address qualifier is 2, 3 or 6 then the last nibble in the State Vector is the TIS-B site ID.
    // Otherwise the last nibble contains UTC coupling & uplink feedback values
    
    if ((addressQualifier == 2) || (addressQualifier == 3) || (addressQualifier == 6))
    {
        ltoi.tisbSiteID = msg[19] & 0x0F;
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            NSLog(@"\nbyte 19 = 0x%.2X or %@\n", msg[19], [self bytes_to_binary:msg[19] numberOfBytes:1]);
            printf ("TIS-B site ID = %d. %s errors.\n", ltoi.tisbSiteID, (ltoi.tisbSiteID > 0) ? "No" : "WARNING:");
        }
        
        ltoi.uplinkFeedback = 0;
        ltoi.adsbTransmittingSourceCoupledToUTCTime = NO;
    }
    else
    {
        ltoi.adsbTransmittingSourceCoupledToUTCTime = msg[19] & 0x08;
        if (payloadTypeCode < 2)
        {
            uint8_t uplinkFeedbackBits = msg[19] & 0x07;
            float uplinkList[8] = {0, 13, 21, 25, 28, 30, 31, 32};
            ltoi.uplinkFeedback = uplinkList[uplinkFeedbackBits];            
        }
        
        if (DEBUG_BASIC_TRAFFIC_MESSAGE || DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            NSLog(@"\nbyte 19 = 0x%.2X or %@\n", msg[19], [self bytes_to_binary:msg[19] numberOfBytes:1]);
            printf ("UTC Coupling = %s. uplinkFeedback = %d.\n",
                    (ltoi.adsbTransmittingSourceCoupledToUTCTime) ? "Yes" : "No", ltoi.uplinkFeedback);
        }
        
        ltoi.tisbSiteID = 0;
    }
    
    if (payloadTypeCode == 0)
    // The payload for basic message will end here. Long message payloads will continue.
    {
        ltoi.isBasicMessage = YES;
    }
    
    else if ((payloadTypeCode == 1) || (payloadTypeCode == 3)) // Continuing on with the extended report information...
    {
        if (DEBUG_LONG_TRAFFIC_MESSAGE) NSLog(@"Parsing long message");
        ltoi.isBasicMessage = NO;
        
        Base40Bytes bb;        
        uint8_t emitterCategory = 0;
        uint8_t callSignFlightPlanID[9];
        uint16_t data = 0;
        uint8_t callSignCharEncoding[40] = {
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
            'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
            'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
            'U', 'V', 'W', 'X', 'Y', 'Z', ' ', '\0', '\0', '\0'
        };
        
        data = (msg[20] << 8) | msg[21];
        bb = [self convert_from_base40:data];
        emitterCategory = bb.B2;
        callSignFlightPlanID[0] = callSignCharEncoding[bb.B1];
        callSignFlightPlanID[1] = callSignCharEncoding[bb.B0];
        
        data = (msg[22] << 8) | msg[23];
        bb = [self convert_from_base40:data];
        callSignFlightPlanID[2] = callSignCharEncoding[bb.B2];
        callSignFlightPlanID[3] = callSignCharEncoding[bb.B1];
        callSignFlightPlanID[4] = callSignCharEncoding[bb.B0];
        
        data = (msg[24] << 8) | msg[25];
        bb = [self convert_from_base40:data];
        callSignFlightPlanID[5] = callSignCharEncoding[bb.B2];
        callSignFlightPlanID[6] = callSignCharEncoding[bb.B1];
        callSignFlightPlanID[7] = callSignCharEncoding[bb.B0];
        
        callSignFlightPlanID[8] = '\0';
        
        NSArray *emitters = [NSArray arrayWithObjects:@"None", @"Light: <15,000 lbs", @"Small: 15,000 - 75,000 lbs",
                             @"Large: 75,000 to 300,000 lbs", @"High Vortex Large", @"Heavy: >300,000 lbs",
                             @"Highly maneuverable", @"Rotocraft", @"Unknown", @"Glider/Sailplane", @"Lighter than air",
                             @"Parachutist / Sky diver", @"Ultralight/Hanglider/Paraglider", @"Unknown", @"UAV",
                             @"Space/transatmospheric vehicle", @"Unknown", @"Surface Vehicle - Emergency vehicle",
                             @"Surface vehicle - surface vehicle", @"Point Obstacle", @"Cluster Obstacle",
                             @"Line Obstacle", @"Unknown", @"Unknown", @"Unknown", @"Unknown", @"Unknown", @"Unknown",
                             @"Unknown", @"Unknown", @"Unknown", @"Unknown", @"Unknown", @"Unknown", @"Unknown",
                             @"Unknown", @"Unknown", @"Unknown", @"Unknown", @"Unknown",nil];
        strcpy(ltoi.emitterCategoryDescr, [[emitters objectAtIndex:emitterCategory] UTF8String]);
        strcpy(ltoi.callSignFlightPlanID, (const char *)callSignFlightPlanID);
        
        if (DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            printf ("\nbyte 20 = 0x%.2X. byte 21 = 0x%.2X.\n", msg[20], msg[21]);
            printf ("\nbyte 22 = 0x%.2X. byte 23 = 0x%.2X.\n", msg[22], msg[23]);
            printf ("\nbyte 24 = 0x%.2X. byte 25 = 0x%.2X.\n", msg[24], msg[25]);
            printf ("Emitter category is %d.\n", emitterCategory);
            printf ("Call sign / flight plan ID is %s.\n", callSignFlightPlanID);
        }        
        
        uint8_t status = (msg[26] & 0xE0) >> 5;
        NSArray *statusTypes = [NSArray arrayWithObjects:@"No Emergency", @"General Emergency", @"Medical Emergency"
                                @"Minimum Fuel", @"No Communications", @"Unlawful Interference", @"Downed Aircraft"
                                @"Unknown", nil];
        strcpy(ltoi.statusDescr, [[statusTypes objectAtIndex:status] UTF8String]);
        ltoi.mopsVersion = (msg[26] & 0x1C) >> 2;
        ltoi.sil = msg[26] & 0x02;
        ltoi.transmitMSO = (msg[27] & 0xFC) >> 2;
        ltoi.sda = msg[27] & 0x02;
        ltoi.nacP = (msg[28] & 0xF0) >> 4;
        ltoi.nacV = (msg[28] & 0x0E) >> 1;
        ltoi.nicBaro = msg[28] & 0x01;
        ltoi.capabilityCodes = (msg[29] & 0xE0) >> 5;
        ltoi.operationalModes = (msg[29] & 0x1C) >> 2;
        ltoi.csid = msg[29] & 0x02;
        ltoi.silSUPP = msg[29] & 0x01;
        ltoi.geoVertAcc = (msg[30] & 0xC0) >> 6;
        ltoi.saFlag = msg[30] & 0x20;
        ltoi.nicSUPP = msg[30] & 0x10;
        
        if (DEBUG_LONG_TRAFFIC_MESSAGE)
        {
            NSLog(@"\nbyte[26] = %@.", [self bytes_to_binary:msg[26] numberOfBytes:1]);
            NSLog(@"status = %d. MOPS version = %d. SIL = %d.", status, ltoi.mopsVersion, ltoi.sil);
            NSLog(@"\nbyte[27] = %@.", [self bytes_to_binary:msg[27] numberOfBytes:1]);
            NSLog(@"transmit MSO = %d. SDA = %d.", ltoi.transmitMSO, ltoi.sda);
            NSLog(@"\nbyte[28] = %@.", [self bytes_to_binary:msg[28] numberOfBytes:1]);
            NSLog(@"NACp = %d. NACv = %d. NICbaro = %d.", ltoi.nacP, ltoi.nacV, ltoi.nicBaro);
            NSLog(@"\nbyte[29] = %@.", [self bytes_to_binary:msg[29] numberOfBytes:1]);
            NSLog(@"capability codes = %d. operational modes = %d. csid = %d. SILsupp = %d.",
                  ltoi.capabilityCodes, ltoi.operationalModes, ltoi.csid, ltoi.silSUPP);
            NSLog(@"\nbyte[30] = %@.", [self bytes_to_binary:msg[30] numberOfBytes:1]);
            NSLog(@"Geo Vert Acc = %d. SA flag = %d. NICsupp = %d.", ltoi.geoVertAcc, ltoi.saFlag, ltoi.nicSUPP);
        }        

    }
    else
    {
        DLog(@"UNHANDLED LONG MESSAGE: Payload type code is %d.", payloadTypeCode);
        return;
    }
    
    // TODO: DO SOMETHING INTERESTING WITH THE TRAFFIC DATA
    create_delegate_instance_d
    [d.iPadMainVC displayBasicAndLongTrafficReport:&ltoi];
    
    // set the appropriate notification
    trafficDataUpdated = [NSNotification notificationWithName:@"BasicAndLongTrafficMessageReceived" object:self];
    [[NSNotificationCenter defaultCenter] postNotification:trafficDataUpdated];
    
    self.messageCount++;
    self.adsbDataAvailable |= kTRAFFIC;

}   // parseBasicAndLongTrafficReport

- (void)parseGDL90Message:(uint8_t *)msg length:(NSUInteger)len
{
    uint16_t i=0;
    uint16_t length=0;
    int parsedMessageIndex = 0;
    uint8_t msgChar = 0;
    uint8_t parsedMsg[900];
    
    // The message has the leading and trailing flag bytes trimmed. But the byte-stuffing will still need
    // to be removed.
    if (len < 5)
    {
        if (DEBUG_GDL90_MESSAGE_PARSING) [self printBuffer:@"Bad GDL90 message:" buffer:msg length:len];
        return;
    }
    
    if (DEBUG_GDL90_MESSAGE_PARSING)
    {
        NSLog(@"\n%s.", __FUNCTION__);
        [self printBuffer:@"Raw message entering GDL90 parser is:" buffer:msg length:len];
    }
    
    // 0x7D is a control-escape character inserted into the GDL90 message which must be removed.
    // The character following the 0x7D must be XOR'd with 0x20 to return it it's proper value.
    // Spin through the sentence and make the substitutions. (This has to be done here instead of
    // in parseBlackjackStreamData because of the case of a control-escape character getting separated
    // from it's following char by breaks in the stream flow.)
    while (i<len)
    {
        //if (DEBUG_GDL90_MESSAGE_PARSING) printf("i = %d. char is 0X%.2X...", i, msg[i]);
        if (msg[i] == 0x7d)
        {
            //if (DEBUG_GDL90_MESSAGE_PARSING) printf("...skipping 0x7D...");
            i++;                          // skip the 0x7D char
            msgChar = msg[i] ^ 0x20;      // XOR the character with 0x20 to restore the correct value
            memcpy(&parsedMsg[parsedMessageIndex], &msgChar, 1);
            //if (DEBUG_GDL90_MESSAGE_PARSING) printf("...adding 0x%.2X...", msgChar);
        }
        else
        {
            memcpy(&parsedMsg[parsedMessageIndex], &msg[i], 1);
            //if (DEBUG_GDL90_MESSAGE_PARSING) printf("...adding 0x%.2X...", msg[i]);
        }
        //if (DEBUG_GDL90_MESSAGE_PARSING) printf("length of message = %d.\n", parsedMessageIndex);
        parsedMessageIndex++;
        i++;
    }
    length = parsedMessageIndex;
    if (DEBUG_GDL90_MESSAGE_PARSING_RESULT)
    {
        NSLog(@"%s.", __FUNCTION__);
        [self printBuffer:@"parsed message is:" buffer:parsedMsg length:length];
    }
    
    switch (parsedMsg[0]) {
        case 0x00:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Heartbeat message received.");
            [self parseHeartbeatMessage:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x07:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Uplink Data message received.");
            [self parseUplinkMessage:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x0a:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Ownship Report message received.");
            [self parseOwnshipReport:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x0b:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Ownship Geometric Altitude message received.");
            [self parseOwnshipGeometricAltitudeReport:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x14:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Traffic Report message received.");
            [self parseShortTrafficReport:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x1E:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Basic Report message received.");
            [self parseBasicAndLongTrafficReport:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x1F:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Long Report message received.");
            [self parseBasicAndLongTrafficReport:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x7A:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Device status message received.");
            [self parseDeviceReport:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x7C:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Device message 0x7C received.");
            [self parseDeviceResponse:&parsedMsg[1] length:(len-2)];
            break;
            
        case 0x79:
            if (DEBUG_SHOW_RECEIVED_MESSAGES) NSLog(@"Device message 0x79 received.");
            [self parseDetailedSatData:&parsedMsg[1] length:(len-2)];
            break;
            
        default:
            if (DEBUG_SHOW_RECEIVED_MESSAGES)
            {
                NSLog(@"Other message received:");
                [self printBuffer:@"Other message contents:" buffer:parsedMsg length:len-2];
            }
            break;
    }
}

- (void)parseADSBStreamData:(uint8_t *)pLine length:(NSUInteger)len
{
    int i=0;
    int length = 0;
    int parsedMessageIndex = 0;
    BOOL midMessage = NO;
    uint8_t msgBuffer[34000];    // maximum size of individual GDL90 message (UAT payload) is 432 bytes
    
    // Need to look out for 2 situations:
    // 1) The buffer ends mid-message. If this happens, copy the sentence fragment to the NSData object so that it
    //    can be retained, and accessed the next time this parsing method is called.
    // 2) The stream data is starting mid-sentence, i.e. the first char is not a GDL90 flag byte (0x7E). If there
    //    is a sentence fragment available, the new data needs to be appended to the fragment. If there is not an
    //    existing sentence fragment, then ignore the characters in the stream up to the first flag byte because
    //    we're entering the data stream for the very first time.
    
    if (pLine[0] == 0x7e)   // either the end of the previous message or the start of a new one
    {
        if (DEBUG_STREAM_PARSING) NSLog(@"Second stream character is 0x%x.", pLine[1]);
        if (pLine[1] == 0x7e)   
        {   
            if (DEBUG_STREAM_PARSING) NSLog(@"Stream started on the end of the previous message.");
            // The stream data is the flag byte at the end of the previous message. If there is a valid
            //  previous message fragment, send it off to be parsed.
            if (self.gdl90MessageFragmentContainsValidData)
            {
                if (DEBUG_STREAM_PARSING) NSLog(@"Sending the previous message fragment to the parser...");
                // get the data from the previous fragment, and add on to it.
                length = [self.gdl90MessageFragment length];
                [self.gdl90MessageFragment getBytes:msgBuffer length:length];
                self.gdl90MessageFragment=nil;
                
                if (DEBUG_STREAM_PARSING) [self printBuffer:@"...previous message fragement is:" buffer:msgBuffer length:length];
                [self parseGDL90Message:msgBuffer length:length];
                
                i+=2;
            }
            else
            {
                if (DEBUG_STREAM_PARSING) NSLog(@"No previous message fragment. Advancing to start of next message.");
                i+=2; // advances the index to the start of the message
            }
        }
        else
        {
            // The stream is starting on the flag byte of a new message
            if (DEBUG_STREAM_PARSING) NSLog(@"Starting with the flag byte of a new message.");
            i++;   // advances the index to the start of the message
        }
    }
    
    else    // the stream data started mid-message
    {
        if (self.gdl90MessageFragmentContainsValidData)
        {
            if (DEBUG_STREAM_PARSING) NSLog(@"Entering stream mid-message. Appending data to previous message fragment");
            // get the data from the previous fragment, and add on to it.
            length = [self.gdl90MessageFragment length];
            [self.gdl90MessageFragment getBytes:msgBuffer length:length];
            self.gdl90MessageFragment = nil;
            self.gdl90MessageFragmentContainsValidData = NO;
            midMessage = YES;
            parsedMessageIndex = length;
            if (DEBUG_STREAM_PARSING) [self printBuffer:@"buffer contains:" buffer:msgBuffer length:length];
        }
        else
        {
            if (DEBUG_STREAM_PARSING) NSLog(@"Entering stream mid-message. No previous data. Ignore any partial sentences becuase the CRC check will discard them.");
            /*
             while ((pLine[i] != 0x7e) && (i < len)) i++;
             i+=2;                    // skip past the flag byte that starts the coming message. 
             length = 0;
             midMessage = NO;
             */
        }
    }
    
    // at this point, we're either mid-message or on the first character of a new message (after the flag byte)
    if (DEBUG_STREAM_PARSING) if (i<len) NSLog(@"...beginning to parse stream data.");
    while (i < len)
    {
        midMessage = YES;
        // Advance through the buffer, "un-byte stuffing" along the way (see pg 5/section 2.2.1 of the GDL90 spec),
        // until the end of the message is found.
        
        if (pLine[i] == 0x7E)
        {
            if (DEBUG_STREAM_PARSING)
            {
                NSLog(@"Flag byte found at i = %d", i);
                printf("Completed GDL90 sentence is:");
                for (int j=0; j<length; j++) printf("0x%.2X ", msgBuffer[j]);
                printf("\n");
            }
            
            [self parseGDL90Message:msgBuffer length:length];
        
            i+=2;                // skip past the flag byte that starts the coming message.
            midMessage = NO;
            parsedMessageIndex = 0;
        }
        else
        {
            midMessage = YES;
            
            if (DEBUG_STREAM_PARSING) printf("i = %d. adding char 0X%.2X...\n", i, pLine[i]);
            memcpy(&msgBuffer[parsedMessageIndex], &pLine[i], 1);
            parsedMessageIndex++;
            i++;
            
            length = parsedMessageIndex;  
        }
    }
    
    // If parsing stopped mid-message, save the partially parse message to fragment variable so that it's
    // available when this method is called again.
    if (midMessage)
    {
        if (DEBUG_STREAM_PARSING) NSLog(@"...parsing ended mid-message. Saving message fragment for later.");
        self.gdl90MessageFragment = [[NSData alloc] initWithBytes:msgBuffer length:length];
        self.gdl90MessageFragmentContainsValidData = YES;
        if (DEBUG_STREAM_PARSING)
        {
            int l = [self.gdl90MessageFragment length];
            uint8_t buffer[5000];
            [self.gdl90MessageFragment getBytes:buffer length:l];
            [self printBuffer:@"Saved message fragment is:" buffer:buffer length:l];
        }
    }
}

# pragma mark - Test Data Methods
- (void)loadTestDataFromFilesystem
{
    unsigned long	lSize = 0;
    unsigned char   *buffer = NULL;
    size_t result;

    FILE *fp;
    NSString *fileName = @"ADSB_Test_Data.bin";
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *fileNameWithPath = [documentsDirectory stringByAppendingPathComponent:fileName];
    
    fp=fopen([fileNameWithPath UTF8String], "rb");
    if (fp == NULL)
    {
        DLog(@"Unable to read %@ from documents directory.", fileName);
    }
    else
    {
        // obtain file size:
        fseek (fp , 0 , SEEK_END);
        
        lSize = ftell( fp ) - 1;
        rewind (fp);
        //NSLog(@"size of file: %ld", lSize);
        
        // allocate memory to contain the whole file:
        buffer = (unsigned char*) malloc (sizeof(unsigned char)*lSize);
        if (buffer == NULL)
        {
            NSLog(@"Unable to allocate memory to read file.");
            fclose(fp);
            exit(0);
        }
        
        // copy the file into the buffer:
        result = fread (buffer,1,lSize,fp);
        if (result != lSize)
        {
            NSLog(@"Couldn't read in whole file.");
            free(buffer);
            fclose(fp);
            exit(0);
        }
        
        fclose (fp);
        fp = NULL;
        
        [self parseADSBStreamData:buffer length:lSize];
        
        free(buffer);
    }
}


# pragma mark - Methods to Manage Sample Data
-(void)runTestData
{
    [self loadTestDataFromFilesystem]; return;

    // Uncomment the lines below to run test data automatically when the app starts.
    // NOTE: WHEN USING THIS SAMPLE DATA, YOU WILL NEED TO HIT REFRESH BUTTONS IN THE SAMPLE APP
    //       IN ORDER TO VIEW THE DATA. (ON THE TRAFFIC TAB, TAP THE 'NO TRAFFIC DATA' LINE TO REFRESH)
    
    ////////////// Weather data
    // Create a three dot test pattern in the region 44˚56'N, 124˚W to 45˚32'N, 120˚W
    //[self parseUplinkMessage:sampleFISBGraphicsA length:(sizeof(sampleFISBGraphicsA)/sizeof(unsigned char))];
    //[self parseUplinkMessage:sampleFISBGraphicsB length:(sizeof(sampleFISBGraphicsB)/sizeof(unsigned char))];
    
    // Scattered weather over the region of 25˚N, 84˚W to 30˚N, 80˚W
    //[self parseADSBStreamData:gulfOfMexicoFISBWeatherData1 length:(sizeof(gulfOfMexicoFISBWeatherData1)/sizeof(unsigned char))];

    ////////////// Traffic data
    // Two traffic targets:
    //      target 1: 29.1006˚,-81.2652˚. altitude 2675'. Northerly velocity 43 kts. Easterly velocity: 96 kts. No tail number.
    //      target 2: 29.1008˚,-81.2646˚. altitude 2625'. Northerly velocity 43 kts. Easter velocity: 96 kts.  Tail number N50317.
    //[self parseADSBStreamData:sampleTrafficDataA length:(sizeof(sampleTrafficDataA)/sizeof(unsigned char))];
    
    // Three traffic targets:
    //      target 1: 29.1006˚,-81.2652˚. Altitude is 2675. Northerly velocity 43 kts. Easterly velocity 96 kts. No tail number.
    //      target 2: 29.1008˚,-81.2646˚. Altitude is 2625. Northerly velocity 46 kts. Easterly velocity 96 kts. No tail number.
    //      target 3: 29.1013˚,-81.2636˚. Altitude is 2575. Northerly velocity 49 kts. Easterly velocity 95 kts. No tail number.
    //[self parseADSBStreamData:sampleTrafficDataB length:(sizeof(sampleTrafficDataB)/sizeof(unsigned char))];

    // Five traffic targets, one repeated:
    //      target 1: 29.1011˚,-81.2652˚. altitude 2675'. Northerly vel. 48 kts. Easterly vel. 96 kts. No tail number.
    //      target 2: 29.1008˚,-81.2646˚. altitude 2625'. Northerly vel. 46 kts. Easterly vel. 96 kts. Tail number N50317.
    //      target 3: 29.1011˚,-81.2640˚. altitude 2600'. Northerly vel. 48 kts. Easterly vel. 96 kts. No tail number.
    //      target 4: 29.1007˚,-81.3259˚. altitude 1175'. Northerly vel. 16 kts. Easterly vel. 92 kts. Tail number 00000000.
    //      target 5: 29.1014˚,-81.2633˚. altitude 2575'. Northerly vel. 50 kts. Easterly vel. 95 kts. Tail number 00000000.
    //      target 6: 29.1021˚,-81.2620˚. altitude 2500'. Northerly vel. 53 kts. Easterly vel. 94 kts. Tail number N50317.
    //[self parseADSBStreamData:sampleTrafficDataC length:(sizeof(sampleTrafficDataC)/sizeof(unsigned char))];
    
    /////////////// Sample TAF data
    // 5 TAF messages
    //      TAF KSLE 260900ZAM 251315 08006KT P6SM FEW060 BKN120 EFM0400 VRB03KT P6SM SCT250=
    //      TAF KPDX 260900ZAM 251315 08006KT P6SM FEW060 BKN120 EFM0400 VRB03KT P6SM SCT250=
    //      TAF KEUG 260900ZAM 251315 08006KT P6SM FEW060 BKN120 EFM0400 VRB03KT P6SM SCT250=
    //      TAF KAST 260900ZAM 251315 08006KT P6SM FEW060 BKN120 EFM0400 VRB03KT P6SM SCT250=
    //      TAF KHIO 260900ZAM 251315 08006KT P6SM FEW060 BKN120 EFM0400 VRB03KT P6SM SCT250=
    //[self parseUplinkMessage:sampleFISBText length:(sizeof(sampleFISBText)/sizeof(unsigned char))];
    
    ////////////// Mixed data
    // 30 traffic reports plus scattered weather over the region of 25˚N, 84˚W to 30˚N, 80˚W
    //[self parseADSBStreamData:largeMixedDataSampleA length:(sizeof(largeMixedDataSampleA)/sizeof(unsigned char))];
    
    // 4 METAR messages plus significant weather over the region of 25˚N, 84˚W to 30˚N, 80˚W
    //      METAR KMCO 222053Z 24006KT 10SM -RA FEW055 BKN090 OVC150 26/21 A3007 DRMK AO2 RAB2
    //      METAR KMCO 222053Z 24006KT 10SM -RA FEW055 BKN090 OVC150 26/21 A3007 DRMK AO2 RAB2
    //      METAR KMCF 222055Z AUTO 16005KT 10SM CLR 24/24 A3007 RMK AO2 LTG DSNT FE RAE2042 SL
    //      METAR KMCF 222055Z AUTO 16005KT 10SM CLR 24/24 A3007 RMK AO2 LTG DSNT FE RAE2042 SL
    //[self parseADSBStreamData:largeMixedDataSampleB length:(sizeof(largeMixedDataSampleB)/sizeof(unsigned char))];
}


@end
